#line 1 "normalize.brg"


/**
 * @file
 *
 * Remove ambiguities from the parse tree and bring it
 * into normalized form.
 *
 * Copyright Notice:
 * -----------------
 *
 * The contents of this file are subject to the Pathfinder Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://monetdb.cwi.nl/Legal/PathfinderLicense-1.1.html
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is the Pathfinder system.
 *
 * The Original Code has initially been developed by the Database &
 * Information Systems Group at the University of Konstanz, Germany and
 * the Database Group at the Technische Universitaet Muenchen, Germany.
 * It is now maintained by the Database Systems Group at the Eberhard
 * Karls Universitaet Tuebingen, Germany.  Portions created by the
 * University of Konstanz, the Technische Universitaet Muenchen, and the
 * Universitaet Tuebingen are Copyright (C) 2000-2005 University of
 * Konstanz, (C) 2005-2008 Technische Universitaet Muenchen, and (C)
 * 2008-2010 Eberhard Karls Universitaet Tuebingen, respectively.  All
 * Rights Reserved.
 *
 *
 * $Id$
 */ 


#include "pf_config.h"
#include "pathfinder.h"
#include <assert.h>
#include <stdio.h>
#include <string.h>

#include "oops.h"
#include "abssyn.h"
#include "qname.h"
#include "mem.h"

/* Easily access subtree-parts */
#include "child_mnemonic.h"

/*
 * Accessors for the burg matcher
 */
typedef struct PFpnode_t *NODEPTR_TYPE;

/* accessor to the node kind */
#define OP_LABEL(p)    ((p)->kind) 

/* accessors to left and right child node */
#define LEFT_CHILD(p)  ((p)->child[0])
#define RIGHT_CHILD(p) ((p)->child[1])

/* the state determined during bottom-up labeling is stored here */
#define STATE_LABEL(p) ((p)->state_label)

/* If something goes wrong, call PFoops */
#define PANIC(...) PFoops (OOPS_BURG, __VA_ARGS__)

#ifndef PFnormalize_PANIC
#define PFnormalize_PANIC	PANIC
#endif /* PFnormalize_PANIC */
#include <stdlib.h> /* for abort() */
#ifdef NDEBUG
#define PFnormalize_assert(x,y)	;
#else
#define PFnormalize_assert(x,y)	if(!(x)) {y; abort();}
#endif
static short PFnormalize_r1_nts[] ={ 2, 0 };
static short PFnormalize_r2_nts[] ={ 3, 0 };
static short PFnormalize_r3_nts[] ={ 5, 27, 0 };
static short PFnormalize_r4_nts[] ={ 4, 5, 0 };
static short PFnormalize_r5_nts[] ={ 0 };
static short PFnormalize_r6_nts[] ={ 6, 0 };
static short PFnormalize_r7_nts[] ={ 7, 6, 0 };
static short PFnormalize_r10_nts[] ={ 8, 0 };
static short PFnormalize_r11_nts[] ={ 9, 0 };
static short PFnormalize_r12_nts[] ={ 10, 0 };
static short PFnormalize_r13_nts[] ={ 11, 0 };
static short PFnormalize_r14_nts[] ={ 12, 0 };
static short PFnormalize_r15_nts[] ={ 14, 0 };
static short PFnormalize_r16_nts[] ={ 15, 0 };
static short PFnormalize_r17_nts[] ={ 16, 0 };
static short PFnormalize_r18_nts[] ={ 17, 0 };
static short PFnormalize_r19_nts[] ={ 13, 0 };
static short PFnormalize_r20_nts[] ={ 18, 0 };
static short PFnormalize_r21_nts[] ={ 19, 0 };
static short PFnormalize_r22_nts[] ={ 21, 0 };
static short PFnormalize_r23_nts[] ={ 26, 0 };
static short PFnormalize_r36_nts[] ={ 20, 0 };
static short PFnormalize_r40_nts[] ={ 22, 52, 28, 0 };
static short PFnormalize_r41_nts[] ={ 22, 52, 0 };
static short PFnormalize_r43_nts[] ={ 23, 0 };
static short PFnormalize_r45_nts[] ={ 24, 23, 0 };
static short PFnormalize_r46_nts[] ={ 25, 0 };
static short PFnormalize_r48_nts[] ={ 52, 0 };
static short PFnormalize_r49_nts[] ={ 25, 28, 0 };
static short PFnormalize_r51_nts[] ={ 28, 0 };
static short PFnormalize_r53_nts[] ={ 29, 0 };
static short PFnormalize_r54_nts[] ={ 29, 28, 0 };
static short PFnormalize_r56_nts[] ={ 29, 28, 28, 0 };
static short PFnormalize_r57_nts[] ={ 30, 0 };
static short PFnormalize_r58_nts[] ={ 31, 0 };
static short PFnormalize_r59_nts[] ={ 46, 0 };
static short PFnormalize_r60_nts[] ={ 47, 0 };
static short PFnormalize_r61_nts[] ={ 58, 0 };
static short PFnormalize_r62_nts[] ={ 59, 0 };
static short PFnormalize_r63_nts[] ={ 33, 41, 42, 28, 0 };
static short PFnormalize_r64_nts[] ={ 33, 32, 41, 42, 28, 0 };
static short PFnormalize_r65_nts[] ={ 33, 0 };
static short PFnormalize_r66_nts[] ={ 33, 32, 0 };
static short PFnormalize_r67_nts[] ={ 35, 0 };
static short PFnormalize_r68_nts[] ={ 34, 0 };
static short PFnormalize_r69_nts[] ={ 36, 0 };
static short PFnormalize_r70_nts[] ={ 39, 37, 28, 0 };
static short PFnormalize_r71_nts[] ={ 39, 28, 0 };
static short PFnormalize_r73_nts[] ={ 38, 0 };
static short PFnormalize_r76_nts[] ={ 40, 0 };
static short PFnormalize_r79_nts[] ={ 43, 0 };
static short PFnormalize_r81_nts[] ={ 44, 0 };
static short PFnormalize_r83_nts[] ={ 45, 0 };
static short PFnormalize_r85_nts[] ={ 28, 45, 0 };
static short PFnormalize_r86_nts[] ={ 32, 28, 0 };
static short PFnormalize_r87_nts[] ={ 33, 32, 28, 0 };
static short PFnormalize_r90_nts[] ={ 28, 51, 49, 0 };
static short PFnormalize_r91_nts[] ={ 28, 51, 48, 0 };
static short PFnormalize_r92_nts[] ={ 51, 49, 0 };
static short PFnormalize_r93_nts[] ={ 51, 48, 0 };
static short PFnormalize_r94_nts[] ={ 50, 28, 0 };
static short PFnormalize_r97_nts[] ={ 50, 52, 28, 0 };
static short PFnormalize_r98_nts[] ={ 54, 0 };
static short PFnormalize_r101_nts[] ={ 55, 0 };
static short PFnormalize_r106_nts[] ={ 56, 0 };
static short PFnormalize_r110_nts[] ={ 57, 0 };
static short PFnormalize_r115_nts[] ={ 28, 28, 28, 0 };
static short PFnormalize_r116_nts[] ={ 60, 0 };
static short PFnormalize_r117_nts[] ={ 60, 59, 0 };
static short PFnormalize_r118_nts[] ={ 61, 0 };
static short PFnormalize_r119_nts[] ={ 61, 60, 0 };
static short PFnormalize_r120_nts[] ={ 62, 0 };
static short PFnormalize_r121_nts[] ={ 62, 62, 0 };
static short PFnormalize_r137_nts[] ={ 63, 0 };
static short PFnormalize_r139_nts[] ={ 64, 0 };
static short PFnormalize_r140_nts[] ={ 63, 64, 0 };
static short PFnormalize_r142_nts[] ={ 65, 0 };
static short PFnormalize_r143_nts[] ={ 65, 64, 0 };
static short PFnormalize_r147_nts[] ={ 66, 0 };
static short PFnormalize_r148_nts[] ={ 66, 65, 0 };
static short PFnormalize_r149_nts[] ={ 67, 0 };
static short PFnormalize_r150_nts[] ={ 67, 66, 0 };
static short PFnormalize_r152_nts[] ={ 68, 0 };
static short PFnormalize_r153_nts[] ={ 68, 52, 0 };
static short PFnormalize_r154_nts[] ={ 69, 0 };
static short PFnormalize_r155_nts[] ={ 69, 52, 0 };
static short PFnormalize_r156_nts[] ={ 70, 0 };
static short PFnormalize_r157_nts[] ={ 70, 53, 0 };
static short PFnormalize_r158_nts[] ={ 71, 0 };
static short PFnormalize_r159_nts[] ={ 71, 53, 0 };
static short PFnormalize_r160_nts[] ={ 72, 0 };
static short PFnormalize_r163_nts[] ={ 73, 0 };
static short PFnormalize_r164_nts[] ={ 74, 0 };
static short PFnormalize_r165_nts[] ={ 102, 0 };
static short PFnormalize_r168_nts[] ={ 75, 0 };
static short PFnormalize_r169_nts[] ={ 76, 0 };
static short PFnormalize_r171_nts[] ={ 76, 75, 0 };
static short PFnormalize_r172_nts[] ={ 78, 0 };
static short PFnormalize_r173_nts[] ={ 77, 0 };
static short PFnormalize_r174_nts[] ={ 79, 0 };
static short PFnormalize_r175_nts[] ={ 75, 28, 0 };
static short PFnormalize_r177_nts[] ={ 80, 0 };
static short PFnormalize_r179_nts[] ={ 86, 0 };
static short PFnormalize_r180_nts[] ={ 87, 0 };
static short PFnormalize_r181_nts[] ={ 88, 0 };
static short PFnormalize_r182_nts[] ={ 93, 0 };
static short PFnormalize_r183_nts[] ={ 91, 0 };
static short PFnormalize_r184_nts[] ={ 92, 0 };
static short PFnormalize_r185_nts[] ={ 116, 0 };
static short PFnormalize_r186_nts[] ={ 81, 0 };
static short PFnormalize_r187_nts[] ={ 85, 0 };
static short PFnormalize_r188_nts[] ={ 82, 0 };
static short PFnormalize_r189_nts[] ={ 83, 0 };
static short PFnormalize_r190_nts[] ={ 84, 0 };
static short PFnormalize_r197_nts[] ={ 89, 0 };
static short PFnormalize_r199_nts[] ={ 90, 0 };
static short PFnormalize_r201_nts[] ={ 28, 90, 0 };
static short PFnormalize_r204_nts[] ={ 94, 0 };
static short PFnormalize_r205_nts[] ={ 95, 0 };
static short PFnormalize_r206_nts[] ={ 96, 0 };
static short PFnormalize_r207_nts[] ={ 97, 0 };
static short PFnormalize_r208_nts[] ={ 98, 0 };
static short PFnormalize_r209_nts[] ={ 99, 0 };
static short PFnormalize_r210_nts[] ={ 100, 101, 0 };
static short PFnormalize_r212_nts[] ={ 101, 0 };
static short PFnormalize_r219_nts[] ={ 101, 101, 0 };
static short PFnormalize_r221_nts[] ={ 101, 101, 101, 0 };
static short PFnormalize_r222_nts[] ={ 103, 0 };
static short PFnormalize_r223_nts[] ={ 103, 29, 0 };
static short PFnormalize_r226_nts[] ={ 104, 0 };
static short PFnormalize_r228_nts[] ={ 105, 0 };
static short PFnormalize_r229_nts[] ={ 106, 0 };
static short PFnormalize_r230_nts[] ={ 108, 0 };
static short PFnormalize_r231_nts[] ={ 107, 0 };
static short PFnormalize_r232_nts[] ={ 109, 0 };
static short PFnormalize_r233_nts[] ={ 112, 113, 0 };
static short PFnormalize_r234_nts[] ={ 112, 0 };
static short PFnormalize_r236_nts[] ={ 112, 114, 0 };
static short PFnormalize_r237_nts[] ={ 110, 30, 30, 0 };
static short PFnormalize_r239_nts[] ={ 111, 110, 0 };
static short PFnormalize_r244_nts[] ={ 22, 28, 0 };
static short PFnormalize_r245_nts[] ={ 22, 0 };
static short PFnormalize_r246_nts[] ={ 115, 0 };
static short PFnormalize_r247_nts[] ={ 39, 30, 30, 0 };
static short PFnormalize_r248_nts[] ={ 29, 88, 0 };
short *PFnormalize_nts[] = {
	0,
	PFnormalize_r1_nts,
	PFnormalize_r2_nts,
	PFnormalize_r3_nts,
	PFnormalize_r4_nts,
	PFnormalize_r5_nts,
	PFnormalize_r6_nts,
	PFnormalize_r7_nts,
	PFnormalize_r6_nts,
	PFnormalize_r5_nts,
	PFnormalize_r10_nts,
	PFnormalize_r11_nts,
	PFnormalize_r12_nts,
	PFnormalize_r13_nts,
	PFnormalize_r14_nts,
	PFnormalize_r15_nts,
	PFnormalize_r16_nts,
	PFnormalize_r17_nts,
	PFnormalize_r18_nts,
	PFnormalize_r19_nts,
	PFnormalize_r20_nts,
	PFnormalize_r21_nts,
	PFnormalize_r22_nts,
	PFnormalize_r23_nts,
	PFnormalize_r2_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r36_nts,
	PFnormalize_r36_nts,
	PFnormalize_r5_nts,
	PFnormalize_r36_nts,
	PFnormalize_r40_nts,
	PFnormalize_r41_nts,
	PFnormalize_r5_nts,
	PFnormalize_r43_nts,
	PFnormalize_r5_nts,
	PFnormalize_r45_nts,
	PFnormalize_r46_nts,
	PFnormalize_r5_nts,
	PFnormalize_r48_nts,
	PFnormalize_r49_nts,
	PFnormalize_r46_nts,
	PFnormalize_r51_nts,
	PFnormalize_r5_nts,
	PFnormalize_r53_nts,
	PFnormalize_r54_nts,
	PFnormalize_r51_nts,
	PFnormalize_r56_nts,
	PFnormalize_r57_nts,
	PFnormalize_r58_nts,
	PFnormalize_r59_nts,
	PFnormalize_r60_nts,
	PFnormalize_r61_nts,
	PFnormalize_r62_nts,
	PFnormalize_r63_nts,
	PFnormalize_r64_nts,
	PFnormalize_r65_nts,
	PFnormalize_r66_nts,
	PFnormalize_r67_nts,
	PFnormalize_r68_nts,
	PFnormalize_r69_nts,
	PFnormalize_r70_nts,
	PFnormalize_r71_nts,
	PFnormalize_r5_nts,
	PFnormalize_r73_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r76_nts,
	PFnormalize_r48_nts,
	PFnormalize_r5_nts,
	PFnormalize_r79_nts,
	PFnormalize_r5_nts,
	PFnormalize_r81_nts,
	PFnormalize_r51_nts,
	PFnormalize_r83_nts,
	PFnormalize_r51_nts,
	PFnormalize_r85_nts,
	PFnormalize_r86_nts,
	PFnormalize_r87_nts,
	PFnormalize_r86_nts,
	PFnormalize_r87_nts,
	PFnormalize_r90_nts,
	PFnormalize_r91_nts,
	PFnormalize_r92_nts,
	PFnormalize_r93_nts,
	PFnormalize_r94_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r97_nts,
	PFnormalize_r98_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r101_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r106_nts,
	PFnormalize_r5_nts,
	PFnormalize_r106_nts,
	PFnormalize_r5_nts,
	PFnormalize_r110_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r115_nts,
	PFnormalize_r116_nts,
	PFnormalize_r117_nts,
	PFnormalize_r118_nts,
	PFnormalize_r119_nts,
	PFnormalize_r120_nts,
	PFnormalize_r121_nts,
	PFnormalize_r121_nts,
	PFnormalize_r121_nts,
	PFnormalize_r121_nts,
	PFnormalize_r121_nts,
	PFnormalize_r121_nts,
	PFnormalize_r121_nts,
	PFnormalize_r121_nts,
	PFnormalize_r121_nts,
	PFnormalize_r121_nts,
	PFnormalize_r121_nts,
	PFnormalize_r121_nts,
	PFnormalize_r121_nts,
	0,
	PFnormalize_r121_nts,
	PFnormalize_r121_nts,
	PFnormalize_r137_nts,
	PFnormalize_r121_nts,
	PFnormalize_r139_nts,
	PFnormalize_r140_nts,
	PFnormalize_r140_nts,
	PFnormalize_r142_nts,
	PFnormalize_r143_nts,
	PFnormalize_r143_nts,
	PFnormalize_r143_nts,
	PFnormalize_r143_nts,
	PFnormalize_r147_nts,
	PFnormalize_r148_nts,
	PFnormalize_r149_nts,
	PFnormalize_r150_nts,
	PFnormalize_r150_nts,
	PFnormalize_r152_nts,
	PFnormalize_r153_nts,
	PFnormalize_r154_nts,
	PFnormalize_r155_nts,
	PFnormalize_r156_nts,
	PFnormalize_r157_nts,
	PFnormalize_r158_nts,
	PFnormalize_r159_nts,
	PFnormalize_r160_nts,
	PFnormalize_r160_nts,
	PFnormalize_r160_nts,
	PFnormalize_r163_nts,
	PFnormalize_r164_nts,
	PFnormalize_r165_nts,
	PFnormalize_r51_nts,
	PFnormalize_r5_nts,
	PFnormalize_r168_nts,
	PFnormalize_r169_nts,
	PFnormalize_r101_nts,
	PFnormalize_r171_nts,
	PFnormalize_r172_nts,
	PFnormalize_r173_nts,
	PFnormalize_r174_nts,
	PFnormalize_r175_nts,
	PFnormalize_r101_nts,
	PFnormalize_r177_nts,
	PFnormalize_r5_nts,
	PFnormalize_r179_nts,
	PFnormalize_r180_nts,
	PFnormalize_r181_nts,
	PFnormalize_r182_nts,
	PFnormalize_r183_nts,
	PFnormalize_r184_nts,
	PFnormalize_r185_nts,
	PFnormalize_r186_nts,
	PFnormalize_r187_nts,
	PFnormalize_r188_nts,
	PFnormalize_r189_nts,
	PFnormalize_r190_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r5_nts,
	PFnormalize_r51_nts,
	PFnormalize_r5_nts,
	PFnormalize_r197_nts,
	PFnormalize_r5_nts,
	PFnormalize_r199_nts,
	PFnormalize_r51_nts,
	PFnormalize_r201_nts,
	PFnormalize_r51_nts,
	PFnormalize_r51_nts,
	PFnormalize_r204_nts,
	PFnormalize_r205_nts,
	PFnormalize_r206_nts,
	PFnormalize_r207_nts,
	PFnormalize_r208_nts,
	PFnormalize_r209_nts,
	PFnormalize_r210_nts,
	PFnormalize_r210_nts,
	PFnormalize_r212_nts,
	PFnormalize_r212_nts,
	PFnormalize_r212_nts,
	PFnormalize_r212_nts,
	PFnormalize_r5_nts,
	PFnormalize_r51_nts,
	PFnormalize_r51_nts,
	PFnormalize_r219_nts,
	PFnormalize_r5_nts,
	PFnormalize_r221_nts,
	PFnormalize_r222_nts,
	PFnormalize_r223_nts,
	PFnormalize_r5_nts,
	PFnormalize_r222_nts,
	PFnormalize_r226_nts,
	PFnormalize_r5_nts,
	PFnormalize_r228_nts,
	PFnormalize_r229_nts,
	PFnormalize_r230_nts,
	PFnormalize_r231_nts,
	PFnormalize_r232_nts,
	PFnormalize_r233_nts,
	PFnormalize_r234_nts,
	PFnormalize_r233_nts,
	PFnormalize_r236_nts,
	PFnormalize_r237_nts,
	PFnormalize_r5_nts,
	PFnormalize_r239_nts,
	PFnormalize_r51_nts,
	PFnormalize_r57_nts,
	PFnormalize_r57_nts,
	PFnormalize_r57_nts,
	PFnormalize_r244_nts,
	PFnormalize_r245_nts,
	PFnormalize_r246_nts,
	PFnormalize_r247_nts,
	PFnormalize_r248_nts,
	PFnormalize_r244_nts,
};
static unsigned char PFnormalize_and_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_args_transition[2][3] = {
{    0,    0,    0}	/* row 0 */,
{    0,    1,    2}	/* row 1 */
};
static unsigned char PFnormalize_attr_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_bind_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_binds_transition[2][3] = {
{    0,    0,    0}	/* row 0 */,
{    0,    1,    2}	/* row 1 */
};
static unsigned char PFnormalize_case__transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_cases_transition[3][4] = {
{    0,    0,    0,    0}	/* row 0 */,
{    0,    2,    0,    0}	/* row 1 */,
{    0,    0,    3,    1}	/* row 2 */
};
static unsigned char PFnormalize_cast_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_castable_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_contseq_transition[6][3] = {
{    0,    0,    0}	/* row 0 */,
{    0,    2,    2}	/* row 1 */,
{    0,    2,    1}	/* row 2 */,
{    0,    3,    3}	/* row 3 */,
{    0,    2,    2}	/* row 4 */,
{    0,    2,    2}	/* row 5 */
};
static unsigned char PFnormalize_decl_imps_transition[3][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */,
{    0,    2}	/* row 2 */
};
static unsigned char PFnormalize_default__transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_div_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_elem_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_eq_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_every_transition[3][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */,
{    0,    2}	/* row 2 */
};
static unsigned char PFnormalize_except_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_exprseq_transition[6][2] = {
{    0,    0}	/* row 0 */,
{    0,    2}	/* row 1 */,
{    0,    3}	/* row 2 */,
{    0,    1}	/* row 3 */,
{    0,    2}	/* row 4 */,
{    0,    2}	/* row 5 */
};
static unsigned char PFnormalize_ext_expr_transition[2][3] = {
{    0,    0,    0}	/* row 0 */,
{    0,    1,    2}	/* row 1 */
};
static unsigned char PFnormalize_flwr_transition[3][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */,
{    0,    2}	/* row 2 */
};
static unsigned char PFnormalize_fun_decl_transition[4][3] = {
{    0,    0,    0}	/* row 0 */,
{    0,    1,    3}	/* row 1 */,
{    0,    5,    2}	/* row 2 */,
{    0,    4,    0}	/* row 3 */
};
static unsigned char PFnormalize_fun_sig_transition[2][4] = {
{    0,    0,    0,    0}	/* row 0 */,
{    0,    2,    1,    3}	/* row 1 */
};
static unsigned char PFnormalize_ge_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_gt_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_gtgt_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_idiv_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_if__transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_instof_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_intersect_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_is_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_le_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_let_transition[3][2] = {
{    0,    0}	/* row 0 */,
{    0,    2}	/* row 1 */,
{    0,    1}	/* row 2 */
};
static unsigned char PFnormalize_lib_mod_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_locpath_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_lt_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_ltlt_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_main_mod_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_minus_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_mod_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_mod_imp_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_mult_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_ne_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_or_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_ord_ret_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_orderspecs_transition[2][3] = {
{    0,    0,    0}	/* row 0 */,
{    0,    2,    1}	/* row 1 */
};
static unsigned char PFnormalize_param_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_params_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_plus_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_pragmas_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_pred_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_range_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_req_ty_transition[3][3] = {
{    0,    0,    0}	/* row 0 */,
{    0,    4,    2}	/* row 1 */,
{    0,    1,    3}	/* row 2 */
};
static unsigned char PFnormalize_schm_ats_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_schm_imp_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_some_transition[3][2] = {
{    0,    0}	/* row 0 */,
{    0,    2}	/* row 1 */,
{    0,    1}	/* row 2 */
};
static unsigned char PFnormalize_then_else_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_treat_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_typeswitch_transition[2][3] = {
{    0,    0,    0}	/* row 0 */,
{    0,    1,    2}	/* row 1 */
};
static unsigned char PFnormalize_union__transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_val_eq_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_val_ge_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_val_gt_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_val_le_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_val_lt_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_val_ne_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_var_decl_transition[2][3] = {
{    0,    0,    0}	/* row 0 */,
{    0,    1,    2}	/* row 1 */
};
static unsigned char PFnormalize_var_type_transition[3][3] = {
{    0,    0,    0}	/* row 0 */,
{    0,    3,    1}	/* row 1 */,
{    0,    0,    2}	/* row 2 */
};
static unsigned char PFnormalize_vars_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_where_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_insert_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_replace_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_rename_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_transform_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_modify_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_transbinds_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_recursion_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_seed_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static unsigned char PFnormalize_xrpc_transition[2][2] = {
{    0,    0}	/* row 0 */,
{    0,    1}	/* row 1 */
};
static struct {
	unsigned int f139:2;
	unsigned int f140:2;
	unsigned int f141:2;
	unsigned int f142:2;
	unsigned int f143:2;
	unsigned int f144:2;
	unsigned int f145:2;
	unsigned int f146:2;
	unsigned int f147:2;
	unsigned int f148:5;
	unsigned int f149:3;
	unsigned int f150:2;
	unsigned int f151:2;
	unsigned int f152:2;
} PFnormalize_plank_0[] = {
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 0 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 1 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 2 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 3 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 4 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 5 */
	{   0,   0,   0,   0,   0,   1,   0,   0,   0,   5,   0,   0,   0,   0,},	/* row 6 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 7 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 8 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 9 */
	{   0,   0,   0,   0,   0,   0,   0,   1,   0,   3,   0,   0,   0,   0,},	/* row 10 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 11 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 12 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 13 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 14 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 15 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 16 */
	{   0,   0,   0,   0,   0,   0,   1,   0,   0,   4,   0,   0,   0,   0,},	/* row 17 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 18 */
	{   0,   0,   0,   0,   1,   0,   0,   0,   0,   6,   0,   0,   0,   0,},	/* row 19 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 20 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 21 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 22 */
	{   1,   0,   0,   0,   0,   0,   0,   0,   0,   9,   0,   0,   0,   0,},	/* row 23 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 24 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   3,   0,   0,   0,},	/* row 25 */
	{   0,   1,   0,   0,   0,   0,   0,   0,   0,   8,   0,   0,   0,   0,},	/* row 26 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 27 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 28 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 29 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 30 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 31 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 32 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 33 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 34 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 35 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   2,   0,   0,   0,   0,},	/* row 36 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 37 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 38 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 39 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 40 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 41 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 42 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 43 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 44 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 45 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 46 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 47 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 48 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   2,   2,   0,   0,   0,   0,},	/* row 49 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,  14,   0,   0,   0,   0,},	/* row 50 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,  14,   0,   0,   0,   0,},	/* row 51 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,  14,   0,   0,   0,   0,},	/* row 52 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,  14,   0,   0,   0,   0,},	/* row 53 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,  14,   0,   0,   0,   0,},	/* row 54 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 55 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 56 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 57 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 58 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 59 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 60 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 61 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 62 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 63 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 64 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 65 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 66 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 67 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 68 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 69 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 70 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 71 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,  16,   0,   1,   0,   2,},	/* row 72 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 73 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 74 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 75 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 76 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 77 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 78 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 79 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   1,},	/* row 80 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 81 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 82 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,  13,   0,   0,   0,   0,},	/* row 83 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 84 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 85 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 86 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 87 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 88 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,},	/* row 89 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 90 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 91 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 92 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,  11,   0,   0,   0,   0,},	/* row 93 */
	{   0,   0,   0,   1,   0,   0,   0,   0,   0,  12,   0,   0,   0,   0,},	/* row 94 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 95 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 96 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 97 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 98 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   7,   0,   0,   0,   0,},	/* row 99 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 100 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 101 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 102 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 103 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 104 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 105 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 106 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 107 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 108 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 109 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 110 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 111 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 112 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 113 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 114 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 115 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 116 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 117 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,  10,   0,   0,   0,   0,},	/* row 118 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 119 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 120 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 121 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 122 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,},	/* row 123 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 124 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 125 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 126 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 127 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 128 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 129 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 130 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 131 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 132 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 133 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 134 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 135 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 136 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 137 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 138 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 139 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 140 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 141 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 142 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 143 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 144 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 145 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 146 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 147 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 148 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 149 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 150 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 151 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 152 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,  15,   0,   0,   0,   0,},	/* row 153 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,  15,   0,   0,   0,   0,},	/* row 154 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 155 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 156 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 157 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 158 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 159 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 160 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 161 */
};
static struct {
	unsigned int f125:2;
	unsigned int f126:2;
	unsigned int f127:2;
	unsigned int f128:4;
	unsigned int f129:3;
	unsigned int f130:2;
	unsigned int f131:2;
	unsigned int f132:2;
	unsigned int f133:2;
	unsigned int f134:3;
	unsigned int f135:2;
	unsigned int f136:2;
	unsigned int f137:2;
	unsigned int f138:2;
} PFnormalize_plank_1[] = {
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 0 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 1 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 2 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 3 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 4 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 5 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 6 */
	{   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 7 */
	{   0,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 8 */
	{   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 9 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 10 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 11 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 12 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 13 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 14 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 15 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 16 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 17 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 18 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 19 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 20 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 21 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 22 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 23 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 24 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 25 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 26 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 27 */
	{   0,   0,   0,  10,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 28 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 29 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 30 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 31 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 32 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 33 */
	{   0,   0,   0,   4,   3,   1,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 34 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 35 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 36 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 37 */
	{   0,   0,   0,   5,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 38 */
	{   0,   0,   0,   5,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 39 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 40 */
	{   0,   0,   0,   4,   2,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 41 */
	{   0,   0,   0,   4,   1,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 42 */
	{   0,   0,   0,   4,   4,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 43 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 44 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 45 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 46 */
	{   0,   0,   2,  11,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 47 */
	{   0,   0,   1,  11,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 48 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 49 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,},	/* row 50 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   0,   0,   0,   0,},	/* row 51 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,},	/* row 52 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   5,   0,   0,   0,   0,},	/* row 53 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   3,   0,   0,   0,   0,},	/* row 54 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 55 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 56 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 57 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 58 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 59 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 60 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 61 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 62 */
	{   0,   0,   0,   3,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 63 */
	{   0,   0,   0,   1,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 64 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 65 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 66 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 67 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 68 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 69 */
	{   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 70 */
	{   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 71 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 72 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 73 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 74 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 75 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 76 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 77 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 78 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 79 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 80 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 81 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 82 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,},	/* row 83 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 84 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 85 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 86 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 87 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 88 */
	{   0,   0,   0,   0,   0,   0,   0,   2,   1,   0,   2,   0,   0,   0,},	/* row 89 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 90 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 91 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 92 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,},	/* row 93 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 94 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 95 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 96 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 97 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 98 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 99 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 100 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 101 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 102 */
	{   0,   0,   0,   0,   0,   0,   0,   1,   2,   0,   0,   0,   0,   0,},	/* row 103 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 104 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 105 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 106 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 107 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 108 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 109 */
	{   0,   0,   0,   2,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 110 */
	{   0,   0,   0,   9,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 111 */
	{   0,   0,   0,   8,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 112 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 113 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 114 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 115 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 116 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 117 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 118 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 119 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,},	/* row 120 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 121 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 122 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,},	/* row 123 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 124 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 125 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 126 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 127 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 128 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 129 */
	{   0,   0,   0,   5,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 130 */
	{   0,   0,   0,   5,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 131 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 132 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 133 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 134 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 135 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 136 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 137 */
	{   0,   0,   0,   7,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 138 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 139 */
	{   0,   0,   0,   6,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 140 */
	{   0,   0,   0,   6,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 141 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 142 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 143 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 144 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 145 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 146 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 147 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 148 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 149 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 150 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 151 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 152 */
	{   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,},	/* row 153 */
	{   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,   0,},	/* row 154 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 155 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 156 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 157 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 158 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 159 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 160 */
	{   0,   0,   0,   4,   3,   2,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 161 */
};
static struct {
	unsigned int f109:2;
	unsigned int f111:3;
	unsigned int f112:2;
	unsigned int f113:2;
	unsigned int f114:2;
	unsigned int f115:2;
	unsigned int f116:2;
	unsigned int f117:3;
	unsigned int f118:2;
	unsigned int f119:2;
	unsigned int f120:2;
	unsigned int f121:2;
	unsigned int f122:2;
	unsigned int f123:2;
	unsigned int f124:2;
} PFnormalize_plank_2[] = {
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 0 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 1 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 2 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 3 */
	{   0,   4,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 4 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 5 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 6 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,},	/* row 7 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 8 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 9 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 10 */
	{   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 11 */
	{   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 12 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 13 */
	{   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 14 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 15 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 16 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 17 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 18 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 19 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 20 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 21 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 22 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 23 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 24 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 25 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 26 */
	{   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 27 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 28 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 29 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 30 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 31 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 32 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 33 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 34 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 35 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 36 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 37 */
	{   0,   0,   0,   0,   0,   0,   0,   3,   0,   0,   0,   2,   0,   0,   0,},	/* row 38 */
	{   0,   0,   0,   0,   0,   0,   0,   4,   0,   0,   0,   2,   0,   0,   0,},	/* row 39 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 40 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 41 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 42 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 43 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 44 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 45 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 46 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 47 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 48 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 49 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 50 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 51 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 52 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 53 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 54 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 55 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 56 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 57 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 58 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 59 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 60 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 61 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 62 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 63 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 64 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 65 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 66 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 67 */
	{   0,   5,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 68 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 69 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,},	/* row 70 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,},	/* row 71 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 72 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 73 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 74 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 75 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 76 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 77 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 78 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 79 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 80 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 81 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 82 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 83 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 84 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 85 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 86 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 87 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 88 */
	{   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   1,   1,   0,   0,},	/* row 89 */
	{   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 90 */
	{   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 91 */
	{   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 92 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 93 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 94 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 95 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 96 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,},	/* row 97 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 98 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 99 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,},	/* row 100 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 101 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 102 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 103 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 104 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 105 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 106 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 107 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 108 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 109 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 110 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 111 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 112 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 113 */
	{   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 114 */
	{   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 115 */
	{   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 116 */
	{   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 117 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 118 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 119 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 120 */
	{   0,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 121 */
	{   0,   3,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 122 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 123 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 124 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 125 */
	{   0,   0,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 126 */
	{   0,   0,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 127 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 128 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 129 */
	{   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   2,   0,   0,   0,},	/* row 130 */
	{   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   2,   0,   0,   0,},	/* row 131 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 132 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 133 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 134 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 135 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 136 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 137 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 138 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 139 */
	{   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 140 */
	{   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 141 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 142 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 143 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 144 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 145 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 146 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 147 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 148 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 149 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 150 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 151 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 152 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 153 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 154 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 155 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 156 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 157 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   2,   0,   0,},	/* row 158 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 159 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 160 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 161 */
};
static struct {
	unsigned int f98:2;
	unsigned int f99:3;
	unsigned int f100:2;
	unsigned int f101:3;
	unsigned int f102:3;
	unsigned int f103:2;
	unsigned int f104:5;
	unsigned int f105:2;
	unsigned int f106:2;
	unsigned int f107:2;
	unsigned int f108:3;
	unsigned int f110:3;
} PFnormalize_plank_3[] = {
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 0 */
	{   2,   3,   2,   5,   2,   1,  12,   2,   1,   0,   0,   0,},	/* row 1 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 2 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 3 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 4 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 5 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 6 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 7 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 8 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 9 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 10 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 11 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 12 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 13 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 14 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 15 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 16 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 17 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 18 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 19 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 20 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 21 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 22 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 23 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 24 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 25 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 26 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 27 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 28 */
	{   2,   3,   2,   2,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 29 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 30 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 31 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 32 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 33 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 34 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 35 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 36 */
	{   2,   3,   2,   5,   2,   1,  11,   1,   1,   0,   0,   0,},	/* row 37 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 38 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 39 */
	{   2,   1,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 40 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 41 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 42 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 43 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 44 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 45 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 46 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 47 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 48 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 49 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 50 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 51 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 52 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 53 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 54 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 55 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 56 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 57 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 58 */
	{   2,   3,   2,   5,   2,   1,   2,   1,   1,   0,   0,   0,},	/* row 59 */
	{   2,   3,   2,   5,   2,   1,  13,   1,   1,   0,   0,   0,},	/* row 60 */
	{   2,   3,   2,   5,   2,   1,   1,   1,   1,   0,   0,   0,},	/* row 61 */
	{   2,   3,   2,   3,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 62 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   1,   0,   0,},	/* row 63 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 64 */
	{   1,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 65 */
	{   2,   2,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 66 */
	{   2,   3,   2,   5,   2,   1,   8,   1,   1,   0,   0,   0,},	/* row 67 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 68 */
	{   2,   3,   2,   5,   2,   1,  14,   1,   1,   0,   0,   0,},	/* row 69 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 70 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 71 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 72 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 73 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 74 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 75 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 76 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 77 */
	{   2,   3,   2,   5,   2,   1,   5,   1,   1,   0,   0,   0,},	/* row 78 */
	{   2,   3,   2,   5,   2,   1,   9,   1,   1,   0,   0,   0,},	/* row 79 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 80 */
	{   2,   3,   2,   5,   1,   1,  12,   1,   1,   0,   0,   0,},	/* row 81 */
	{   2,   3,   2,   4,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 82 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 83 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 84 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 85 */
	{   2,   3,   2,   1,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 86 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 87 */
	{   2,   3,   2,   5,   2,   1,   6,   1,   1,   0,   0,   0,},	/* row 88 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 89 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,},	/* row 90 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   3,},	/* row 91 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,},	/* row 92 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 93 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 94 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   2,   0,   0,   0,},	/* row 95 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 96 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 97 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 98 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 99 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 100 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 101 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 102 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 103 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 104 */
	{   2,   3,   2,   5,   3,   1,  12,   1,   1,   0,   0,   0,},	/* row 105 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 106 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 107 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 108 */
	{   2,   3,   2,   5,   2,   2,  12,   1,   1,   0,   0,   0,},	/* row 109 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 110 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 111 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 112 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 113 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   3,   0,},	/* row 114 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   0,},	/* row 115 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,},	/* row 116 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,},	/* row 117 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 118 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 119 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 120 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 121 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 122 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 123 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 124 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 125 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 126 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 127 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 128 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 129 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 130 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 131 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 132 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 133 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 134 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 135 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 136 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 137 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 138 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 139 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 140 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 141 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 142 */
	{   2,   3,   1,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 143 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 144 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 145 */
	{   2,   3,   2,   5,   2,   1,  10,   1,   1,   0,   0,   0,},	/* row 146 */
	{   2,   3,   2,   5,   2,   1,   4,   1,   1,   0,   0,   0,},	/* row 147 */
	{   2,   3,   2,   5,   2,   1,   3,   1,   1,   0,   0,   0,},	/* row 148 */
	{   2,   3,   2,   5,   2,   1,  16,   1,   1,   0,   0,   0,},	/* row 149 */
	{   2,   3,   2,   5,   2,   1,   7,   1,   1,   0,   0,   0,},	/* row 150 */
	{   2,   3,   2,   5,   2,   1,  15,   1,   1,   0,   0,   0,},	/* row 151 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 152 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 153 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 154 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 155 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 156 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 157 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 158 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 159 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 160 */
	{   2,   3,   2,   5,   2,   1,  12,   1,   1,   0,   0,   0,},	/* row 161 */
};
static struct {
	unsigned int f44:1;
	unsigned int f85:2;
	unsigned int f86:4;
	unsigned int f87:2;
	unsigned int f88:2;
	unsigned int f89:2;
	unsigned int f90:3;
	unsigned int f91:2;
	unsigned int f92:2;
	unsigned int f93:3;
	unsigned int f94:3;
	unsigned int f95:2;
	unsigned int f96:2;
	unsigned int f97:2;
} PFnormalize_plank_4[] = {
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 0 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 1 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 2 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 3 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 4 */
	{   0,   0,   3,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 5 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 6 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 7 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 8 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 9 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 10 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 11 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 12 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 13 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 14 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   2,   2,   1,},	/* row 15 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   1,   1,},	/* row 16 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 17 */
	{   0,   0,   3,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 18 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 19 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 20 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 21 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 22 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 23 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 24 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 25 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 26 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 27 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 28 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 29 */
	{   0,   0,   3,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 30 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 31 */
	{   0,   0,   9,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 32 */
	{   0,   0,   3,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 33 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 34 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 35 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 36 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 37 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 38 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 39 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 40 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 41 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 42 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 43 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   2,   2,   1,   2,   1,},	/* row 44 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   2,   2,   1,   2,   1,},	/* row 45 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 46 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 47 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 48 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 49 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 50 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 51 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 52 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 53 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 54 */
	{   0,   0,   2,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 55 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 56 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 57 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 58 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 59 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 60 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 61 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 62 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 63 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 64 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 65 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 66 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 67 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 68 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 69 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 70 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 71 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 72 */
	{   0,   2,   7,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 73 */
	{   0,   2,   7,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 74 */
	{   0,   2,   7,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 75 */
	{   0,   1,   7,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 76 */
	{   0,   0,   1,   0,   2,   2,   2,   2,   0,   1,   2,   1,   2,   1,},	/* row 77 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 78 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 79 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 80 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 81 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 82 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 83 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 84 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 85 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 86 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 87 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 88 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 89 */
	{   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 90 */
	{   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 91 */
	{   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 92 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 93 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 94 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 95 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 96 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 97 */
	{   0,   0,   4,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 98 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 99 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 100 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 101 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 102 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 103 */
	{   0,   0,   3,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 104 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 105 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 106 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 107 */
	{   0,   0,   1,   0,   1,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 108 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 109 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 110 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 111 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 112 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 113 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 114 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 115 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 116 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 117 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 118 */
	{   0,   0,   1,   0,   2,   2,   1,   1,   0,   1,   2,   1,   2,   1,},	/* row 119 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 120 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 121 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 122 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 123 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 124 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 125 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 126 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 127 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 128 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 129 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 130 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 131 */
	{   0,   0,   1,   1,   2,   1,   3,   2,   0,   1,   2,   1,   2,   1,},	/* row 132 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 133 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 134 */
	{   0,   0,   3,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 135 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 136 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 137 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 138 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   2,},	/* row 139 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 140 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 141 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   1,   1,   2,   1,},	/* row 142 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 143 */
	{   0,   0,   5,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 144 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   3,   1,   2,   1,},	/* row 145 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 146 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 147 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 148 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 149 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 150 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 151 */
	{   0,   0,   1,   0,   2,   2,   1,   2,   1,   3,   2,   1,   2,   1,},	/* row 152 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 153 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 154 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 155 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 156 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 157 */
	{   0,   0,   8,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 158 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 159 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 160 */
	{   0,   0,   6,   0,   2,   2,   1,   2,   0,   1,   2,   1,   2,   1,},	/* row 161 */
};
static struct {
	unsigned int f70:2;
	unsigned int f71:2;
	unsigned int f72:2;
	unsigned int f73:2;
	unsigned int f74:2;
	unsigned int f75:3;
	unsigned int f76:2;
	unsigned int f77:2;
	unsigned int f78:2;
	unsigned int f79:2;
	unsigned int f80:2;
	unsigned int f81:2;
	unsigned int f82:2;
	unsigned int f83:2;
	unsigned int f84:3;
} PFnormalize_plank_5[] = {
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 0 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 1 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   0,},	/* row 2 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   2,   1,   0,   0,   0,   0,   0,},	/* row 3 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 4 */
	{   0,   0,   0,   1,   0,   3,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 5 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 6 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 7 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 8 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 9 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 10 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 11 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 12 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 13 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 14 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 15 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 16 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 17 */
	{   0,   1,   0,   0,   0,   4,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 18 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 19 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 20 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 21 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 22 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 23 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 24 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 25 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 26 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 27 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 28 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 29 */
	{   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 30 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 31 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,},	/* row 32 */
	{   0,   0,   0,   0,   1,   6,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 33 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 34 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 35 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 36 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 37 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 38 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 39 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 40 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 41 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 42 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 43 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 44 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 45 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 46 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 47 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 48 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 49 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 50 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 51 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 52 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 53 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 54 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 55 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 56 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 57 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 58 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 59 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 60 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 61 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 62 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 63 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 64 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 65 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 66 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 67 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 68 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 69 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 70 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 71 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 72 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   3,},	/* row 73 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   1,},	/* row 74 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   2,},	/* row 75 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 76 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 77 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 78 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 79 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 80 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 81 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 82 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 83 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 84 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 85 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 86 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 87 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 88 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,},	/* row 89 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 90 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 91 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 92 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 93 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 94 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 95 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 96 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 97 */
	{   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,},	/* row 98 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 99 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 100 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 101 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 102 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 103 */
	{   1,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 104 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 105 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 106 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 107 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 108 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 109 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 110 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 111 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 112 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 113 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 114 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 115 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 116 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 117 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 118 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 119 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 120 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 121 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 122 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 123 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 124 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 125 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 126 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 127 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 128 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 129 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 130 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 131 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 132 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 133 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 134 */
	{   0,   0,   1,   0,   0,   5,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 135 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 136 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 137 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 138 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 139 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 140 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 141 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 142 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 143 */
	{   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 144 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 145 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 146 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 147 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 148 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 149 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 150 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 151 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 152 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 153 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 154 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 155 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 156 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 157 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 158 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 159 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 160 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 161 */
};
static struct {
	unsigned int f35:1;
	unsigned int f55:2;
	unsigned int f56:2;
	unsigned int f57:2;
	unsigned int f58:2;
	unsigned int f59:2;
	unsigned int f60:2;
	unsigned int f61:2;
	unsigned int f62:2;
	unsigned int f63:2;
	unsigned int f64:2;
	unsigned int f65:2;
	unsigned int f66:2;
	unsigned int f67:3;
	unsigned int f68:2;
	unsigned int f69:2;
} PFnormalize_plank_6[] = {
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 0 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 1 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 2 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 3 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 4 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 5 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 6 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 7 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 8 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 9 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 10 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 11 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 12 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 13 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 14 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 15 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 16 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 17 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 18 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 19 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,},	/* row 20 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,},	/* row 21 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   3,   0,   0,},	/* row 22 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 23 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 24 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 25 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 26 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 27 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   4,   2,   0,},	/* row 28 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 29 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   1,},	/* row 30 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 31 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 32 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 33 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 34 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 35 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 36 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 37 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 38 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 39 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 40 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 41 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 42 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 43 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   4,   2,   0,},	/* row 44 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   4,   2,   0,},	/* row 45 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 46 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 47 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 48 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 49 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 50 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 51 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 52 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 53 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 54 */
	{   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 55 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 56 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 57 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 58 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 59 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 60 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 61 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 62 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 63 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   4,   2,   0,},	/* row 64 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 65 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 66 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 67 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 68 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 69 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 70 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 71 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 72 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 73 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 74 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 75 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 76 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 77 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 78 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 79 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 80 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 81 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 82 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 83 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 84 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 85 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 86 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 87 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 88 */
	{   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,},	/* row 89 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 90 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 91 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 92 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 93 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 94 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 95 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 96 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 97 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 98 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 99 */
	{   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 100 */
	{   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 101 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 102 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 103 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 104 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 105 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 106 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,},	/* row 107 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 108 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 109 */
	{   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 110 */
	{   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 111 */
	{   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 112 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 113 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 114 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 115 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 116 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 117 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 118 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 119 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 120 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 121 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 122 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 123 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 124 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 125 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 126 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 127 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 128 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 129 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 130 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 131 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 132 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 133 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,},	/* row 134 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 135 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 136 */
	{   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 137 */
	{   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 138 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 139 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 140 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 141 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 142 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 143 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 144 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 145 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 146 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 147 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 148 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 149 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 150 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 151 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 152 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 153 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 154 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 155 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 156 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 157 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 158 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 159 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 160 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   2,   0,},	/* row 161 */
};
static struct {
	unsigned int f25:1;
	unsigned int f39:2;
	unsigned int f40:2;
	unsigned int f41:2;
	unsigned int f42:2;
	unsigned int f43:3;
	unsigned int f45:2;
	unsigned int f46:2;
	unsigned int f47:2;
	unsigned int f48:2;
	unsigned int f49:2;
	unsigned int f50:2;
	unsigned int f51:2;
	unsigned int f52:2;
	unsigned int f53:2;
	unsigned int f54:2;
} PFnormalize_plank_7[] = {
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 0 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 1 */
	{   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 2 */
	{   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 3 */
	{   0,   0,   0,   0,   0,   3,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 4 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 5 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 6 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 7 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 8 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 9 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 10 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 11 */
	{   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 12 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 13 */
	{   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 14 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 15 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 16 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 17 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 18 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 19 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 20 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 21 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 22 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 23 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 24 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 25 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 26 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 27 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 28 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 29 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 30 */
	{   0,   0,   0,   0,   0,   5,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 31 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 32 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 33 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 34 */
	{   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 35 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 36 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 37 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 38 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 39 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 40 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 41 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 42 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 43 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 44 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 45 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 46 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 47 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 48 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 49 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 50 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 51 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 52 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 53 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 54 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 55 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 56 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 57 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 58 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 59 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 60 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 61 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 62 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 63 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 64 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 65 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 66 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 67 */
	{   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 68 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 69 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 70 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,},	/* row 71 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 72 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 73 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 74 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 75 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 76 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 77 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 78 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 79 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 80 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 81 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 82 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 83 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 84 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,},	/* row 85 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 86 */
	{   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 87 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 88 */
	{   1,   0,   0,   2,   2,   0,   0,   0,   2,   1,   1,   0,   1,   0,   0,   0,},	/* row 89 */
	{   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 90 */
	{   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 91 */
	{   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 92 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 93 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 94 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 95 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,},	/* row 96 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 97 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 98 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 99 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 100 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 101 */
	{   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 102 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 103 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 104 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 105 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 106 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 107 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 108 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 109 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 110 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 111 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 112 */
	{   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 113 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 114 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 115 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 116 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 117 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 118 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 119 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 120 */
	{   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 121 */
	{   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 122 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 123 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,},	/* row 124 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,},	/* row 125 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,},	/* row 126 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,},	/* row 127 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 128 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 129 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 130 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 131 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 132 */
	{   0,   0,   0,   0,   0,   4,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 133 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 134 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 135 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 136 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,},	/* row 137 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 138 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 139 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 140 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 141 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 142 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 143 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 144 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 145 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 146 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 147 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 148 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 149 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 150 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 151 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 152 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 153 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 154 */
	{   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 155 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 156 */
	{   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 157 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   1,   0,   0,   0,   0,   0,},	/* row 158 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 159 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 160 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 161 */
};
static struct {
	unsigned int f22:2;
	unsigned int f23:3;
	unsigned int f24:2;
	unsigned int f26:2;
	unsigned int f27:3;
	unsigned int f28:2;
	unsigned int f29:2;
	unsigned int f30:2;
	unsigned int f31:2;
	unsigned int f32:2;
	unsigned int f33:2;
	unsigned int f34:2;
	unsigned int f36:2;
	unsigned int f37:2;
	unsigned int f38:2;
} PFnormalize_plank_8[] = {
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 0 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 1 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 2 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 3 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 4 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 5 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 6 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 7 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 8 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 9 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 10 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 11 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 12 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 13 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 14 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 15 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 16 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 17 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 18 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 19 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 20 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 21 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 22 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 23 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 24 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 25 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 26 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 27 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 28 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 29 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 30 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 31 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 32 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 33 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 34 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 35 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 36 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 37 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 38 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 39 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 40 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 41 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 42 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 43 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 44 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 45 */
	{   0,   0,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 46 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 47 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 48 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 49 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 50 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 51 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 52 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 53 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 54 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 55 */
	{   0,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 56 */
	{   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 57 */
	{   0,   3,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 58 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 59 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 60 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 61 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 62 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 63 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 64 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 65 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 66 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 67 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 68 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 69 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 70 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 71 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 72 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 73 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 74 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 75 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   3,   0,   0,   0,   0,},	/* row 76 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 77 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 78 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 79 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 80 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 81 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 82 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 83 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 84 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 85 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 86 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 87 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 88 */
	{   0,   0,   0,   1,   0,   0,   0,   0,   0,   1,   2,   1,   1,   1,   0,},	/* row 89 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 90 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,},	/* row 91 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 92 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 93 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 94 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 95 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 96 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,},	/* row 97 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 98 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 99 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,},	/* row 100 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,},	/* row 101 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 102 */
	{   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 103 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 104 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 105 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 106 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 107 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 108 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 109 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 110 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 111 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 112 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 113 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,},	/* row 114 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,},	/* row 115 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,},	/* row 116 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,},	/* row 117 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 118 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 119 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 120 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 121 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 122 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 123 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 124 */
	{   0,   0,   0,   0,   1,   0,   1,   0,   0,   0,   0,   0,   0,   1,   0,},	/* row 125 */
	{   0,   0,   0,   0,   1,   0,   1,   0,   0,   0,   0,   0,   0,   1,   0,},	/* row 126 */
	{   0,   0,   0,   0,   1,   0,   1,   0,   0,   0,   0,   0,   0,   1,   0,},	/* row 127 */
	{   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 128 */
	{   0,   0,   0,   0,   3,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 129 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 130 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 131 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 132 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 133 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 134 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 135 */
	{   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 136 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 137 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 138 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 139 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 140 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 141 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 142 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 143 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 144 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 145 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 146 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 147 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 148 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 149 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 150 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 151 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 152 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 153 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 154 */
	{   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,   0,},	/* row 155 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 156 */
	{   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,},	/* row 157 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,},	/* row 158 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 159 */
	{   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 160 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 161 */
};
static struct {
	unsigned int f2:1;
	unsigned int f8:2;
	unsigned int f9:2;
	unsigned int f10:2;
	unsigned int f11:3;
	unsigned int f12:2;
	unsigned int f13:3;
	unsigned int f14:2;
	unsigned int f15:2;
	unsigned int f16:2;
	unsigned int f17:2;
	unsigned int f18:2;
	unsigned int f19:3;
	unsigned int f20:2;
	unsigned int f21:2;
} PFnormalize_plank_9[] = {
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 0 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 1 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 2 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 3 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 4 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 5 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 6 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 7 */
	{   0,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,},	/* row 8 */
	{   0,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,},	/* row 9 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 10 */
	{   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 11 */
	{   0,   0,   0,   0,   3,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 12 */
	{   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 13 */
	{   0,   0,   0,   0,   3,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 14 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 15 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 16 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 17 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 18 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 19 */
	{   0,   0,   0,   0,   0,   0,   4,   1,   0,   0,   0,   0,   0,   0,   0,},	/* row 20 */
	{   0,   0,   0,   0,   0,   0,   3,   1,   0,   0,   0,   0,   0,   0,   0,},	/* row 21 */
	{   0,   0,   0,   0,   0,   0,   5,   1,   0,   0,   0,   0,   0,   0,   0,},	/* row 22 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 23 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 24 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,},	/* row 25 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 26 */
	{   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 27 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 28 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 29 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 30 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 31 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 32 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 33 */
	{   0,   0,   0,   0,   0,   0,   2,   2,   0,   0,   0,   0,   2,   0,   1,},	/* row 34 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 35 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 36 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 37 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 38 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 39 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 40 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   5,   0,   1,},	/* row 41 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   3,   0,   1,},	/* row 42 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   4,   0,   1,},	/* row 43 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 44 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 45 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 46 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 47 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 48 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 49 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 50 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 51 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 52 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 53 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 54 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 55 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 56 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 57 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 58 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 59 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 60 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 61 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 62 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 63 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 64 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 65 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 66 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 67 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 68 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 69 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 70 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 71 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 72 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 73 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 74 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 75 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 76 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 77 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 78 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 79 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 80 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 81 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 82 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 83 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 84 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 85 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 86 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 87 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 88 */
	{   1,   1,   0,   0,   1,   0,   0,   0,   2,   1,   1,   0,   0,   1,   2,},	/* row 89 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 90 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 91 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 92 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 93 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 94 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 95 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 96 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 97 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 98 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 99 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 100 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 101 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 102 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 103 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 104 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 105 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 106 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,},	/* row 107 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 108 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 109 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 110 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 111 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 112 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 113 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 114 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 115 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 116 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 117 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 118 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 119 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 120 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 121 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 122 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 123 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 124 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 125 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 126 */
	{   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 127 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 128 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 129 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 130 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 131 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 132 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 133 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 134 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 135 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 136 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 137 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 138 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 139 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 140 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 141 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 142 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 143 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 144 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 145 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 146 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 147 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 148 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 149 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 150 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 151 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 152 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 153 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,},	/* row 154 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 155 */
	{   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 156 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 157 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   1,   0,   1,   0,   1,},	/* row 158 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 159 */
	{   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,},	/* row 160 */
	{   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   0,   1,   0,   1,},	/* row 161 */
};
static struct {
	unsigned int f0:2;
	unsigned int f1:2;
	unsigned int f3:2;
	unsigned int f4:2;
	unsigned int f5:2;
	unsigned int f6:2;
	unsigned int f7:2;
} PFnormalize_plank_10[] = {
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 0 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 1 */
	{   0,   2,   0,   0,   0,   0,   0,},	/* row 2 */
	{   0,   2,   0,   0,   0,   0,   0,},	/* row 3 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 4 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 5 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 6 */
	{   0,   0,   0,   0,   0,   0,   1,},	/* row 7 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 8 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 9 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 10 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 11 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 12 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 13 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 14 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 15 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 16 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 17 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 18 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 19 */
	{   0,   0,   0,   1,   0,   0,   0,},	/* row 20 */
	{   0,   0,   0,   1,   0,   0,   0,},	/* row 21 */
	{   0,   0,   0,   1,   0,   0,   0,},	/* row 22 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 23 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 24 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 25 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 26 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 27 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 28 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 29 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 30 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 31 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 32 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 33 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 34 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 35 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 36 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 37 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 38 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 39 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 40 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 41 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 42 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 43 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 44 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 45 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 46 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 47 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 48 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 49 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 50 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 51 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 52 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 53 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 54 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 55 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 56 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 57 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 58 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 59 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 60 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 61 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 62 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 63 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 64 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 65 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 66 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 67 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 68 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 69 */
	{   0,   0,   0,   0,   0,   0,   1,},	/* row 70 */
	{   0,   0,   0,   0,   0,   0,   1,},	/* row 71 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 72 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 73 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 74 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 75 */
	{   1,   0,   1,   1,   1,   0,   0,},	/* row 76 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 77 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 78 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 79 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 80 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 81 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 82 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 83 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 84 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 85 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 86 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 87 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 88 */
	{   0,   1,   0,   0,   0,   0,   0,},	/* row 89 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 90 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 91 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 92 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 93 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 94 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 95 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 96 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 97 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 98 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 99 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 100 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 101 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 102 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 103 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 104 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 105 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 106 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 107 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 108 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 109 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 110 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 111 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 112 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 113 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 114 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 115 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 116 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 117 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 118 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 119 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 120 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 121 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 122 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 123 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 124 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 125 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 126 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 127 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 128 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 129 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 130 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 131 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 132 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 133 */
	{   0,   0,   1,   0,   0,   0,   0,},	/* row 134 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 135 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 136 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 137 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 138 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 139 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 140 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 141 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 142 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 143 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 144 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 145 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 146 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 147 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 148 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 149 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 150 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 151 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 152 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 153 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 154 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 155 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 156 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 157 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 158 */
	{   0,   0,   0,   0,   0,   1,   0,},	/* row 159 */
	{   0,   0,   0,   0,   0,   0,   0,},	/* row 160 */
	{   1,   0,   1,   1,   0,   0,   0,},	/* row 161 */
};
static short PFnormalize_eruleMap[] = {
    0,  141,  139,  140,    0,  118,  119,    0,  211,    0,	/* 0-9 */
   29,    0,   27,    0,   97,    0,   93,   92,    0,  158,	/* 10-19 */
  159,    0,  157,  156,    0,  213,    0,  136,  126,  131,	/* 20-29 */
  132,  123,  122,  129,  133,  135,  127,  121,  120,  125,	/* 30-39 */
  124,  128,  130,    0,   30,    0,  208,  209,  205,  207,	/* 40-49 */
  206,  204,    0,  196,    0,   34,    0,  192,    0,   20,	/* 50-59 */
   10,   11,   12,   13,   14,   15,   16,   17,  226,   18,	/* 60-69 */
   19,   21,   22,   23,   24,    0,    9,    7,    8,    0,	/* 70-79 */
   94,    0,   28,    0,   25,   26,    0,  234,    0,  215,	/* 80-89 */
    0,  193,    0,  114,  111,  112,  113,    0,  210,    0,	/* 90-99 */
   33,    0,   54,   56,   57,   55,    0,  228,  246,   61,	/* 100-109 */
   62,   59,   60,  232,  231,  230,  229,   58,    0,  222,	/* 110-119 */
  223,    0,   64,   63,    0,  175,  174,    0,  200,  201,	/* 120-129 */
    0,  197,    0,   41,   40,  244,  245,  249,    0,  115,	/* 130-139 */
    0,  233,    0,  153,  152,    0,  191,    0,  151,  150,	/* 140-149 */
  149,    0,  101,  105,  104,  103,  102,    0,   71,    0,	/* 150-159 */
   69,    0,    4,    0,  187,  186,    0,    3,    0,    5,	/* 160-169 */
    0,   37,    0,  143,  144,  145,  146,  142,    0,   35,	/* 170-179 */
    0,  243,    0,  107,  108,  106,    0,  189,  188,  190,	/* 180-189 */
    0,  219,  220,  221,  218,    0,  110,  109,    0,   52,	/* 190-199 */
   53,    0,  199,  198,    0,   39,   38,    0,   81,   80,	/* 200-209 */
    0,   42,   43,    0,   72,   73,    0,   47,   48,    0,	/* 210-219 */
   75,   76,    0,   96,   95,    0,   78,   79,    0,   31,	/* 220-229 */
    0,  116,  117,    0,   83,    0,   84,   85,    0,  202,	/* 230-239 */
    0,   32,    0,  214,    0,   46,    0,   45,   44,    0,	/* 240-249 */
  195,    0,  167,  168,    0,   74,    0,  224,  225,    0,	/* 250-259 */
  179,  181,  182,  183,  184,  185,  177,  178,  180,    0,	/* 260-269 */
    6,    0,   86,   87,   88,   89,    0,    1,    2,    0,	/* 270-279 */
   51,    0,  137,  138,    0,  247,    0,  169,  171,  170,	/* 280-289 */
    0,  236,    0,  235,    0,  227,    0,   36,    0,   99,	/* 290-299 */
   98,    0,  100,    0,  241,    0,  172,  173,    0,  176,	/* 300-309 */
    0,  194,    0,  216,  217,    0,  242,    0,  212,    0,	/* 310-319 */
  240,    0,  238,  239,    0,  237,    0,  154,  155,    0,	/* 320-329 */
   77,    0,   90,   91,    0,  161,  160,  162,    0,  148,	/* 330-339 */
  147,    0,  203,    0,  166,    0,  164,  165,  163,    0,	/* 340-349 */
   68,   67,    0,   66,   65,    0,   70,    0,   49,   50,	/* 350-359 */
    0,   82,    0,  248
};
#define PFnormalize_Query_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_0[state].f152 +276]
#define PFnormalize_MainModule_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_0[state].f151 +166]
#define PFnormalize_LibraryModule_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_0[state].f150 +161]
#define PFnormalize_ModuleDecl_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_8[state].f31 +168]
#define PFnormalize_Prolog_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_9[state].f16 +269]
#define PFnormalize_DeclsImports_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_0[state].f149 +75]
#define PFnormalize_DeclImport_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_0[state].f148 +58]
#define PFnormalize_DefaultNamespaceDecl_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_0[state].f147 +83]
#define PFnormalize_BoundarySpaceDecl_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_0[state].f146 +11]
#define PFnormalize_DefaultCollationDecl_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_0[state].f145 +81]
#define PFnormalize_BaseURIDecl_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_0[state].f144 +9]
#define PFnormalize_ConstructionDecl_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_0[state].f143 +43]
#define PFnormalize_OptionDecl_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_0[state].f142 +228]
#define PFnormalize_OrderingModeDecl_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_0[state].f141 +240]
#define PFnormalize_EmptyOrderingDecl_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_0[state].f140 +99]
#define PFnormalize_CopyNamespacesDecl_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_0[state].f139 +54]
#define PFnormalize_NamespaceDecl_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_1[state].f138 +178]
#define PFnormalize_SchemaImport_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_1[state].f137 +296]
#define PFnormalize_ModuleImport_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_1[state].f136 +170]
#define PFnormalize_OptImportLoc__rule(state)	PFnormalize_eruleMap[PFnormalize_plank_1[state].f135 +204]
#define PFnormalize_FunctionDeclaration_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_1[state].f134 +132]
#define PFnormalize_OptParamList__rule(state)	PFnormalize_eruleMap[PFnormalize_plank_1[state].f133 +210]
#define PFnormalize_ParamList__rule(state)	PFnormalize_eruleMap[PFnormalize_plank_1[state].f132 +246]
#define PFnormalize_Param_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_7[state].f39 +244]
#define PFnormalize_OptSequenceType__rule(state)	PFnormalize_eruleMap[PFnormalize_plank_7[state].f48 +216]
#define PFnormalize_VariableDeclaration_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_1[state].f131 +357]
#define PFnormalize_QueryBody_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_10[state].f0 +279]
#define PFnormalize_OptExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_1[state].f130 +198]
#define PFnormalize_Expr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_1[state].f129 +101]
#define PFnormalize_ExprSingle_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_1[state].f128 +106]
#define PFnormalize_FLWORExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_1[state].f127 +121]
#define PFnormalize_VarBindings_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_1[state].f126 +352]
#define PFnormalize_VarBinding_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_1[state].f125 +349]
#define PFnormalize_LetClause_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_2[state].f123 +159]
#define PFnormalize_VarPosBinding_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_2[state].f124 +355]
#define PFnormalize_LetBinding__rule(state)	PFnormalize_eruleMap[PFnormalize_plank_2[state].f123 +157]
#define PFnormalize_OptPositionalVar__rule(state)	PFnormalize_eruleMap[PFnormalize_plank_2[state].f122 +213]
#define PFnormalize_PositionalVar_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_8[state].f38 +254]
#define PFnormalize_OptTypeDeclaration__rule(state)	PFnormalize_eruleMap[PFnormalize_plank_7[state].f48 +219]
#define PFnormalize_TypeDeclaration_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_8[state].f29 +329]
#define PFnormalize_OptWhereClause__rule(state)	PFnormalize_eruleMap[PFnormalize_plank_2[state].f121 +225]
#define PFnormalize_OptOrderByClause__rule(state)	PFnormalize_eruleMap[PFnormalize_plank_2[state].f120 +207]
#define PFnormalize_WhereClause_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_10[state].f0 +360]
#define PFnormalize_OrderByClause_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_2[state].f119 +233]
#define PFnormalize_OrderSpecList_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_2[state].f118 +235]
#define PFnormalize_QuantifiedExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_2[state].f117 +271]
#define PFnormalize_TypeswitchExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_2[state].f116 +331]
#define PFnormalize_CaseClauses__rule(state)	PFnormalize_eruleMap[PFnormalize_plank_2[state].f115 +15]
#define PFnormalize_DefaultClause__rule(state)	PFnormalize_eruleMap[PFnormalize_plank_2[state].f114 +79]
#define PFnormalize_OptVarName__rule(state)	PFnormalize_eruleMap[PFnormalize_plank_7[state].f47 +222]
#define PFnormalize_CaseClause_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_2[state].f113 +13]
#define PFnormalize_SequenceType_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_2[state].f112 +298]
#define PFnormalize_SingleType_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_9[state].f12 +301]
#define PFnormalize_ItemType_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_2[state].f111 +151]
#define PFnormalize_NodeTest_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_3[state].f110 +182]
#define PFnormalize_OptElemOrAttrType__rule(state)	PFnormalize_eruleMap[PFnormalize_plank_2[state].f109 +195]
#define PFnormalize_ElemOrAttrType_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_3[state].f108 +92]
#define PFnormalize_IfExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_3[state].f107 +138]
#define PFnormalize_OrExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_3[state].f106 +230]
#define PFnormalize_AndExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_3[state].f105 +4]
#define PFnormalize_ComparisonExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_3[state].f104 +26]
#define PFnormalize_RangeExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_3[state].f103 +281]
#define PFnormalize_AdditiveExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_3[state].f102 +0]
#define PFnormalize_MultiplicativeExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_3[state].f101 +172]
#define PFnormalize_UnionExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_3[state].f100 +338]
#define PFnormalize_IntersectExceptExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_3[state].f99 +147]
#define PFnormalize_InstanceofExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_3[state].f98 +142]
#define PFnormalize_TreatExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_4[state].f97 +326]
#define PFnormalize_CastableExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_4[state].f96 +21]
#define PFnormalize_CastExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_4[state].f95 +18]
#define PFnormalize_UnaryExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_4[state].f94 +334]
#define PFnormalize_ValueExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_4[state].f93 +345]
#define PFnormalize_ValidateExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_4[state].f92 +343]
#define PFnormalize_PathExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_4[state].f91 +251]
#define PFnormalize_RelativePathExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_4[state].f90 +286]
#define PFnormalize_StepExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_4[state].f89 +305]
#define PFnormalize_FilterExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_4[state].f88 +124]
#define PFnormalize_Step__rule(state)	PFnormalize_eruleMap[PFnormalize_plank_4[state].f87 +308]
#define PFnormalize_PrimaryExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_4[state].f86 +259]
#define PFnormalize_Literal_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_4[state].f85 +163]
#define PFnormalize_NumericLiteral_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_5[state].f84 +186]
#define PFnormalize_IntegerLiteral_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_5[state].f83 +145]
#define PFnormalize_DecimalLiteral_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_5[state].f82 +56]
#define PFnormalize_DoubleLiteral_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_5[state].f81 +90]
#define PFnormalize_StringLiteral_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_10[state].f5 +310]
#define PFnormalize_ParenthesizedExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_10[state].f0 +249]
#define PFnormalize_ContextItemExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_5[state].f80 +52]
#define PFnormalize_FunctionCall_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_6[state].f55 +130]
#define PFnormalize_OptFuncArgList__rule(state)	PFnormalize_eruleMap[PFnormalize_plank_5[state].f79 +201]
#define PFnormalize_FuncArgList__rule(state)	PFnormalize_eruleMap[PFnormalize_plank_5[state].f78 +127]
#define PFnormalize_OrderedExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_5[state].f77 +238]
#define PFnormalize_UnorderedExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_5[state].f76 +341]
#define PFnormalize_Constructor_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_5[state].f75 +45]
#define PFnormalize_ElementConstructor_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_5[state].f74 +97]
#define PFnormalize_AttributeConstructor_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_5[state].f73 +7]
#define PFnormalize_TextConstructor_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_5[state].f72 +317]
#define PFnormalize_CommentConstructor_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_5[state].f71 +24]
#define PFnormalize_PIConstructor_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_5[state].f70 +242]
#define PFnormalize_DocumentConstructor_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_6[state].f69 +88]
#define PFnormalize_TagName_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_6[state].f68 +312]
#define PFnormalize_OptContentExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_6[state].f67 +190]
#define PFnormalize_ExtensionExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_6[state].f66 +118]
#define PFnormalize_Pragmas_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_6[state].f65 +256]
#define PFnormalize_RevalidationDecl_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_6[state].f64 +294]
#define PFnormalize_InsertExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_6[state].f63 +140]
#define PFnormalize_DeleteExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_6[state].f62 +86]
#define PFnormalize_ReplaceExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_6[state].f61 +292]
#define PFnormalize_RenameExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_6[state].f60 +290]
#define PFnormalize_TransformExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_6[state].f59 +324]
#define PFnormalize_TransformBindings_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_6[state].f58 +321]
#define PFnormalize_TransformBinding_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_7[state].f53 +319]
#define PFnormalize_SourceExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_10[state].f0 +303]
#define PFnormalize_TargetExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_10[state].f0 +315]
#define PFnormalize_NewNameExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_10[state].f0 +180]
#define PFnormalize_RecursiveExpr_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_6[state].f57 +284]
#define PFnormalize_XRPCCall_rule(state)	PFnormalize_eruleMap[PFnormalize_plank_6[state].f56 +362]

#ifdef __STDC__
int PFnormalize_rule(int state, int goalnt) {
#else
int PFnormalize_rule(state, goalnt) int state; int goalnt; {
#endif
	PFnormalize_assert(state >= 0 && state < 162, PFnormalize_PANIC("Bad state %d passed to PFnormalize_rule\n", state));
	switch(goalnt) {
	case 1:
		return PFnormalize_Query_rule(state);
	case 2:
		return PFnormalize_MainModule_rule(state);
	case 3:
		return PFnormalize_LibraryModule_rule(state);
	case 4:
		return PFnormalize_ModuleDecl_rule(state);
	case 5:
		return PFnormalize_Prolog_rule(state);
	case 6:
		return PFnormalize_DeclsImports_rule(state);
	case 7:
		return PFnormalize_DeclImport_rule(state);
	case 8:
		return PFnormalize_DefaultNamespaceDecl_rule(state);
	case 9:
		return PFnormalize_BoundarySpaceDecl_rule(state);
	case 10:
		return PFnormalize_DefaultCollationDecl_rule(state);
	case 11:
		return PFnormalize_BaseURIDecl_rule(state);
	case 12:
		return PFnormalize_ConstructionDecl_rule(state);
	case 13:
		return PFnormalize_OptionDecl_rule(state);
	case 14:
		return PFnormalize_OrderingModeDecl_rule(state);
	case 15:
		return PFnormalize_EmptyOrderingDecl_rule(state);
	case 16:
		return PFnormalize_CopyNamespacesDecl_rule(state);
	case 17:
		return PFnormalize_NamespaceDecl_rule(state);
	case 18:
		return PFnormalize_SchemaImport_rule(state);
	case 19:
		return PFnormalize_ModuleImport_rule(state);
	case 20:
		return PFnormalize_OptImportLoc__rule(state);
	case 21:
		return PFnormalize_FunctionDeclaration_rule(state);
	case 22:
		return PFnormalize_OptParamList__rule(state);
	case 23:
		return PFnormalize_ParamList__rule(state);
	case 24:
		return PFnormalize_Param_rule(state);
	case 25:
		return PFnormalize_OptSequenceType__rule(state);
	case 26:
		return PFnormalize_VariableDeclaration_rule(state);
	case 27:
		return PFnormalize_QueryBody_rule(state);
	case 28:
		return PFnormalize_OptExpr_rule(state);
	case 29:
		return PFnormalize_Expr_rule(state);
	case 30:
		return PFnormalize_ExprSingle_rule(state);
	case 31:
		return PFnormalize_FLWORExpr_rule(state);
	case 32:
		return PFnormalize_VarBindings_rule(state);
	case 33:
		return PFnormalize_VarBinding_rule(state);
	case 34:
		return PFnormalize_LetClause_rule(state);
	case 35:
		return PFnormalize_VarPosBinding_rule(state);
	case 36:
		return PFnormalize_LetBinding__rule(state);
	case 37:
		return PFnormalize_OptPositionalVar__rule(state);
	case 38:
		return PFnormalize_PositionalVar_rule(state);
	case 39:
		return PFnormalize_OptTypeDeclaration__rule(state);
	case 40:
		return PFnormalize_TypeDeclaration_rule(state);
	case 41:
		return PFnormalize_OptWhereClause__rule(state);
	case 42:
		return PFnormalize_OptOrderByClause__rule(state);
	case 43:
		return PFnormalize_WhereClause_rule(state);
	case 44:
		return PFnormalize_OrderByClause_rule(state);
	case 45:
		return PFnormalize_OrderSpecList_rule(state);
	case 46:
		return PFnormalize_QuantifiedExpr_rule(state);
	case 47:
		return PFnormalize_TypeswitchExpr_rule(state);
	case 48:
		return PFnormalize_CaseClauses__rule(state);
	case 49:
		return PFnormalize_DefaultClause__rule(state);
	case 50:
		return PFnormalize_OptVarName__rule(state);
	case 51:
		return PFnormalize_CaseClause_rule(state);
	case 52:
		return PFnormalize_SequenceType_rule(state);
	case 53:
		return PFnormalize_SingleType_rule(state);
	case 54:
		return PFnormalize_ItemType_rule(state);
	case 55:
		return PFnormalize_NodeTest_rule(state);
	case 56:
		return PFnormalize_OptElemOrAttrType__rule(state);
	case 57:
		return PFnormalize_ElemOrAttrType_rule(state);
	case 58:
		return PFnormalize_IfExpr_rule(state);
	case 59:
		return PFnormalize_OrExpr_rule(state);
	case 60:
		return PFnormalize_AndExpr_rule(state);
	case 61:
		return PFnormalize_ComparisonExpr_rule(state);
	case 62:
		return PFnormalize_RangeExpr_rule(state);
	case 63:
		return PFnormalize_AdditiveExpr_rule(state);
	case 64:
		return PFnormalize_MultiplicativeExpr_rule(state);
	case 65:
		return PFnormalize_UnionExpr_rule(state);
	case 66:
		return PFnormalize_IntersectExceptExpr_rule(state);
	case 67:
		return PFnormalize_InstanceofExpr_rule(state);
	case 68:
		return PFnormalize_TreatExpr_rule(state);
	case 69:
		return PFnormalize_CastableExpr_rule(state);
	case 70:
		return PFnormalize_CastExpr_rule(state);
	case 71:
		return PFnormalize_UnaryExpr_rule(state);
	case 72:
		return PFnormalize_ValueExpr_rule(state);
	case 73:
		return PFnormalize_ValidateExpr_rule(state);
	case 74:
		return PFnormalize_PathExpr_rule(state);
	case 75:
		return PFnormalize_RelativePathExpr_rule(state);
	case 76:
		return PFnormalize_StepExpr_rule(state);
	case 77:
		return PFnormalize_FilterExpr_rule(state);
	case 78:
		return PFnormalize_Step__rule(state);
	case 79:
		return PFnormalize_PrimaryExpr_rule(state);
	case 80:
		return PFnormalize_Literal_rule(state);
	case 81:
		return PFnormalize_NumericLiteral_rule(state);
	case 82:
		return PFnormalize_IntegerLiteral_rule(state);
	case 83:
		return PFnormalize_DecimalLiteral_rule(state);
	case 84:
		return PFnormalize_DoubleLiteral_rule(state);
	case 85:
		return PFnormalize_StringLiteral_rule(state);
	case 86:
		return PFnormalize_ParenthesizedExpr_rule(state);
	case 87:
		return PFnormalize_ContextItemExpr_rule(state);
	case 88:
		return PFnormalize_FunctionCall_rule(state);
	case 89:
		return PFnormalize_OptFuncArgList__rule(state);
	case 90:
		return PFnormalize_FuncArgList__rule(state);
	case 91:
		return PFnormalize_OrderedExpr_rule(state);
	case 92:
		return PFnormalize_UnorderedExpr_rule(state);
	case 93:
		return PFnormalize_Constructor_rule(state);
	case 94:
		return PFnormalize_ElementConstructor_rule(state);
	case 95:
		return PFnormalize_AttributeConstructor_rule(state);
	case 96:
		return PFnormalize_TextConstructor_rule(state);
	case 97:
		return PFnormalize_CommentConstructor_rule(state);
	case 98:
		return PFnormalize_PIConstructor_rule(state);
	case 99:
		return PFnormalize_DocumentConstructor_rule(state);
	case 100:
		return PFnormalize_TagName_rule(state);
	case 101:
		return PFnormalize_OptContentExpr_rule(state);
	case 102:
		return PFnormalize_ExtensionExpr_rule(state);
	case 103:
		return PFnormalize_Pragmas_rule(state);
	case 104:
		return PFnormalize_RevalidationDecl_rule(state);
	case 105:
		return PFnormalize_InsertExpr_rule(state);
	case 106:
		return PFnormalize_DeleteExpr_rule(state);
	case 107:
		return PFnormalize_ReplaceExpr_rule(state);
	case 108:
		return PFnormalize_RenameExpr_rule(state);
	case 109:
		return PFnormalize_TransformExpr_rule(state);
	case 110:
		return PFnormalize_TransformBindings_rule(state);
	case 111:
		return PFnormalize_TransformBinding_rule(state);
	case 112:
		return PFnormalize_SourceExpr_rule(state);
	case 113:
		return PFnormalize_TargetExpr_rule(state);
	case 114:
		return PFnormalize_NewNameExpr_rule(state);
	case 115:
		return PFnormalize_RecursiveExpr_rule(state);
	case 116:
		return PFnormalize_XRPCCall_rule(state);
	default:
		PFnormalize_PANIC("Unknown nonterminal %d in PFnormalize_rule;\n", goalnt);
		abort();
		return 0;
	}
}

int PFnormalize_TEMP;
#define PFnormalize_and_state(l,r)	( (PFnormalize_TEMP = PFnormalize_and_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 0 : 0 )
#define PFnormalize_args_state(l,r)	( (PFnormalize_TEMP = PFnormalize_args_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f1]) ? PFnormalize_TEMP + 1 : 0 )
#define PFnormalize_atom_ty_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_9[l].f2) ? PFnormalize_TEMP + 3 : 0 )
#define PFnormalize_attr_state(l,r)	( (PFnormalize_TEMP = PFnormalize_attr_transition[PFnormalize_plank_10[l].f3][PFnormalize_plank_10[r].f4]) ? PFnormalize_TEMP + 4 : 0 )
#define PFnormalize_base_uri_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_10[l].f5) ? PFnormalize_TEMP + 5 : 0 )
#define PFnormalize_bind_state(l,r)	( (PFnormalize_TEMP = PFnormalize_bind_transition[PFnormalize_plank_10[l].f6][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 6 : 0 )
#define PFnormalize_binds_state(l,r)	( (PFnormalize_TEMP = PFnormalize_binds_transition[PFnormalize_plank_10[l].f7][PFnormalize_plank_9[r].f8]) ? PFnormalize_TEMP + 7 : 0 )
#define PFnormalize_case__state(l,r)	( (PFnormalize_TEMP = PFnormalize_case__transition[PFnormalize_plank_9[l].f9][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 10 : 0 )
#define PFnormalize_cases_state(l,r)	( (PFnormalize_TEMP = PFnormalize_cases_transition[PFnormalize_plank_9[l].f10][PFnormalize_plank_9[r].f11]) ? PFnormalize_TEMP + 11 : 0 )
#define PFnormalize_cast_state(l,r)	( (PFnormalize_TEMP = PFnormalize_cast_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_9[r].f12]) ? PFnormalize_TEMP + 14 : 0 )
#define PFnormalize_castable_state(l,r)	( (PFnormalize_TEMP = PFnormalize_castable_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_9[r].f12]) ? PFnormalize_TEMP + 15 : 0 )
#define PFnormalize_coll_decl_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_10[l].f5) ? PFnormalize_TEMP + 16 : 0 )
#define PFnormalize_comment_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_10[l].f4) ? PFnormalize_TEMP + 17 : 0 )
#define PFnormalize_constr_decl_state	19
#define PFnormalize_contseq_state(l,r)	( (PFnormalize_TEMP = PFnormalize_contseq_transition[PFnormalize_plank_9[l].f13][PFnormalize_plank_9[r].f14]) ? PFnormalize_TEMP + 19 : 0 )
#define PFnormalize_decl_imps_state(l,r)	( (PFnormalize_TEMP = PFnormalize_decl_imps_transition[PFnormalize_plank_9[l].f15][PFnormalize_plank_9[r].f16]) ? PFnormalize_TEMP + 23 : 0 )
#define PFnormalize_def_order_state	26
#define PFnormalize_default__state(l,r)	( (PFnormalize_TEMP = PFnormalize_default__transition[PFnormalize_plank_9[l].f17][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 26 : 0 )
#define PFnormalize_div_state(l,r)	( (PFnormalize_TEMP = PFnormalize_div_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 28 : 0 )
#define PFnormalize_doc_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_10[l].f4) ? PFnormalize_TEMP + 29 : 0 )
#define PFnormalize_dot_state	32
#define PFnormalize_elem_state(l,r)	( (PFnormalize_TEMP = PFnormalize_elem_transition[PFnormalize_plank_10[l].f3][PFnormalize_plank_10[r].f4]) ? PFnormalize_TEMP + 32 : 0 )
#define PFnormalize_empty_seq_state	34
#define PFnormalize_empty_ty_state	35
#define PFnormalize_ens_decl_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_10[l].f5) ? PFnormalize_TEMP + 35 : 0 )
#define PFnormalize_eq_state(l,r)	( (PFnormalize_TEMP = PFnormalize_eq_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 36 : 0 )
#define PFnormalize_every_state(l,r)	( (PFnormalize_TEMP = PFnormalize_every_transition[PFnormalize_plank_9[l].f18][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 37 : 0 )
#define PFnormalize_except_state(l,r)	( (PFnormalize_TEMP = PFnormalize_except_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 39 : 0 )
#define PFnormalize_exprseq_state(l,r)	( (PFnormalize_TEMP = PFnormalize_exprseq_transition[PFnormalize_plank_9[l].f19][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 40 : 0 )
#define PFnormalize_external_state	46
#define PFnormalize_ext_expr_state(l,r)	( (PFnormalize_TEMP = PFnormalize_ext_expr_transition[PFnormalize_plank_9[l].f20][PFnormalize_plank_9[r].f21]) ? PFnormalize_TEMP + 43 : 0 )
#define PFnormalize_flwr_state(l,r)	( (PFnormalize_TEMP = PFnormalize_flwr_transition[PFnormalize_plank_9[l].f18][PFnormalize_plank_8[r].f22]) ? PFnormalize_TEMP + 46 : 0 )
#define PFnormalize_fns_decl_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_10[l].f5) ? PFnormalize_TEMP + 48 : 0 )
#define PFnormalize_fun_decl_state(l,r)	( (PFnormalize_TEMP = PFnormalize_fun_decl_transition[PFnormalize_plank_8[l].f23][PFnormalize_plank_8[r].f24]) ? PFnormalize_TEMP + 49 : 0 )
#define PFnormalize_fun_ref_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_7[l].f25) ? PFnormalize_TEMP + 54 : 0 )
#define PFnormalize_fun_sig_state(l,r)	( (PFnormalize_TEMP = PFnormalize_fun_sig_transition[PFnormalize_plank_8[l].f26][PFnormalize_plank_8[r].f27]) ? PFnormalize_TEMP + 55 : 0 )
#define PFnormalize_ge_state(l,r)	( (PFnormalize_TEMP = PFnormalize_ge_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 58 : 0 )
#define PFnormalize_gt_state(l,r)	( (PFnormalize_TEMP = PFnormalize_gt_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 59 : 0 )
#define PFnormalize_gtgt_state(l,r)	( (PFnormalize_TEMP = PFnormalize_gtgt_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 60 : 0 )
#define PFnormalize_idiv_state(l,r)	( (PFnormalize_TEMP = PFnormalize_idiv_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 61 : 0 )
#define PFnormalize_if__state(l,r)	( (PFnormalize_TEMP = PFnormalize_if__transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_8[r].f28]) ? PFnormalize_TEMP + 62 : 0 )
#define PFnormalize_copy_ns_state	23
#define PFnormalize_instof_state(l,r)	( (PFnormalize_TEMP = PFnormalize_instof_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_8[r].f29]) ? PFnormalize_TEMP + 64 : 0 )
#define PFnormalize_intersect_state(l,r)	( (PFnormalize_TEMP = PFnormalize_intersect_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 65 : 0 )
#define PFnormalize_is_state(l,r)	( (PFnormalize_TEMP = PFnormalize_is_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 66 : 0 )
#define PFnormalize_item_ty_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_9[l].f2) ? PFnormalize_TEMP + 67 : 0 )
#define PFnormalize_le_state(l,r)	( (PFnormalize_TEMP = PFnormalize_le_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 68 : 0 )
#define PFnormalize_let_state(l,r)	( (PFnormalize_TEMP = PFnormalize_let_transition[PFnormalize_plank_8[l].f30][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 69 : 0 )
#define PFnormalize_lib_mod_state(l,r)	( (PFnormalize_TEMP = PFnormalize_lib_mod_transition[PFnormalize_plank_8[l].f31][PFnormalize_plank_9[r].f16]) ? PFnormalize_TEMP + 71 : 0 )
#define PFnormalize_lit_dbl_state	73
#define PFnormalize_lit_dec_state	74
#define PFnormalize_lit_int_state	75
#define PFnormalize_lit_str_state	76
#define PFnormalize_locpath_state(l,r)	( (PFnormalize_TEMP = PFnormalize_locpath_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 76 : 0 )
#define PFnormalize_lt_state(l,r)	( (PFnormalize_TEMP = PFnormalize_lt_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 77 : 0 )
#define PFnormalize_ltlt_state(l,r)	( (PFnormalize_TEMP = PFnormalize_ltlt_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 78 : 0 )
#define PFnormalize_main_mod_state(l,r)	( (PFnormalize_TEMP = PFnormalize_main_mod_transition[PFnormalize_plank_9[l].f16][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 79 : 0 )
#define PFnormalize_minus_state(l,r)	( (PFnormalize_TEMP = PFnormalize_minus_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 80 : 0 )
#define PFnormalize_mod_state(l,r)	( (PFnormalize_TEMP = PFnormalize_mod_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 81 : 0 )
#define PFnormalize_mod_imp_state(l,r)	( (PFnormalize_TEMP = PFnormalize_mod_imp_transition[PFnormalize_plank_10[l].f5][PFnormalize_plank_8[r].f32]) ? PFnormalize_TEMP + 82 : 0 )
#define PFnormalize_mod_ns_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_10[l].f5) ? PFnormalize_TEMP + 83 : 0 )
#define PFnormalize_mult_state(l,r)	( (PFnormalize_TEMP = PFnormalize_mult_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 85 : 0 )
#define PFnormalize_named_ty_state	87
#define PFnormalize_ne_state(l,r)	( (PFnormalize_TEMP = PFnormalize_ne_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 87 : 0 )
#define PFnormalize_nil_state	89
#define PFnormalize_node_ty_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_8[l].f33) ? PFnormalize_TEMP + 89 : 0 )
#define PFnormalize_ns_decl_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_10[l].f5) ? PFnormalize_TEMP + 92 : 0 )
#define PFnormalize_option_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_10[l].f5) ? PFnormalize_TEMP + 93 : 0 )
#define PFnormalize_or_state(l,r)	( (PFnormalize_TEMP = PFnormalize_or_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 94 : 0 )
#define PFnormalize_ord_ret_state(l,r)	( (PFnormalize_TEMP = PFnormalize_ord_ret_transition[PFnormalize_plank_8[l].f34][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 95 : 0 )
#define PFnormalize_orderby_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_6[l].f35) ? PFnormalize_TEMP + 96 : 0 )
#define PFnormalize_ordered_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_10[l].f0) ? PFnormalize_TEMP + 97 : 0 )
#define PFnormalize_ordering_mode_state	99
#define PFnormalize_orderspecs_state(l,r)	( (PFnormalize_TEMP = PFnormalize_orderspecs_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_8[r].f36]) ? PFnormalize_TEMP + 99 : 0 )
#define PFnormalize_param_state(l,r)	( (PFnormalize_TEMP = PFnormalize_param_transition[PFnormalize_plank_8[l].f37][PFnormalize_plank_8[r].f38]) ? PFnormalize_TEMP + 101 : 0 )
#define PFnormalize_params_state(l,r)	( (PFnormalize_TEMP = PFnormalize_params_transition[PFnormalize_plank_7[l].f39][PFnormalize_plank_8[r].f26]) ? PFnormalize_TEMP + 102 : 0 )
#define PFnormalize_pi_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_10[l].f4) ? PFnormalize_TEMP + 103 : 0 )
#define PFnormalize_plus_state(l,r)	( (PFnormalize_TEMP = PFnormalize_plus_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 104 : 0 )
#define PFnormalize_pragma_state	106
#define PFnormalize_pragmas_state(l,r)	( (PFnormalize_TEMP = PFnormalize_pragmas_transition[PFnormalize_plank_7[l].f40][PFnormalize_plank_9[r].f20]) ? PFnormalize_TEMP + 106 : 0 )
#define PFnormalize_pred_state(l,r)	( (PFnormalize_TEMP = PFnormalize_pred_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 107 : 0 )
#define PFnormalize_range_state(l,r)	( (PFnormalize_TEMP = PFnormalize_range_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 108 : 0 )
#define PFnormalize_req_name_state	113
#define PFnormalize_req_ty_state(l,r)	( (PFnormalize_TEMP = PFnormalize_req_ty_transition[PFnormalize_plank_7[l].f41][PFnormalize_plank_7[r].f42]) ? PFnormalize_TEMP + 113 : 0 )
#define PFnormalize_root__state	119
#define PFnormalize_schm_ats_state(l,r)	( (PFnormalize_TEMP = PFnormalize_schm_ats_transition[PFnormalize_plank_10[l].f5][PFnormalize_plank_8[r].f32]) ? PFnormalize_TEMP + 119 : 0 )
#define PFnormalize_schm_attr_state	121
#define PFnormalize_schm_elem_state	122
#define PFnormalize_schm_imp_state(l,r)	( (PFnormalize_TEMP = PFnormalize_schm_imp_transition[PFnormalize_plank_10[l].f5][PFnormalize_plank_8[r].f32]) ? PFnormalize_TEMP + 122 : 0 )
#define PFnormalize_seq_ty_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_7[l].f43) ? PFnormalize_TEMP + 124 : 0 )
#define PFnormalize_some_state(l,r)	( (PFnormalize_TEMP = PFnormalize_some_transition[PFnormalize_plank_9[l].f18][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 129 : 0 )
#define PFnormalize_step_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_4[l].f44) ? PFnormalize_TEMP + 131 : 0 )
#define PFnormalize_tag_state	134
#define PFnormalize_text_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_10[l].f4) ? PFnormalize_TEMP + 134 : 0 )
#define PFnormalize_then_else_state(l,r)	( (PFnormalize_TEMP = PFnormalize_then_else_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 135 : 0 )
#define PFnormalize_treat_state(l,r)	( (PFnormalize_TEMP = PFnormalize_treat_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_8[r].f29]) ? PFnormalize_TEMP + 138 : 0 )
#define PFnormalize_typeswitch_state(l,r)	( (PFnormalize_TEMP = PFnormalize_typeswitch_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_7[r].f45]) ? PFnormalize_TEMP + 139 : 0 )
#define PFnormalize_uminus_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_10[l].f0) ? PFnormalize_TEMP + 141 : 0 )
#define PFnormalize_union__state(l,r)	( (PFnormalize_TEMP = PFnormalize_union__transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 142 : 0 )
#define PFnormalize_unordered_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_10[l].f0) ? PFnormalize_TEMP + 143 : 0 )
#define PFnormalize_uplus_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_10[l].f0) ? PFnormalize_TEMP + 144 : 0 )
#define PFnormalize_val_eq_state(l,r)	( (PFnormalize_TEMP = PFnormalize_val_eq_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 145 : 0 )
#define PFnormalize_val_ge_state(l,r)	( (PFnormalize_TEMP = PFnormalize_val_ge_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 146 : 0 )
#define PFnormalize_val_gt_state(l,r)	( (PFnormalize_TEMP = PFnormalize_val_gt_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 147 : 0 )
#define PFnormalize_val_le_state(l,r)	( (PFnormalize_TEMP = PFnormalize_val_le_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 148 : 0 )
#define PFnormalize_val_lt_state(l,r)	( (PFnormalize_TEMP = PFnormalize_val_lt_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 149 : 0 )
#define PFnormalize_val_ne_state(l,r)	( (PFnormalize_TEMP = PFnormalize_val_ne_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 150 : 0 )
#define PFnormalize_validate_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_10[l].f0) ? PFnormalize_TEMP + 151 : 0 )
#define PFnormalize_var_decl_state(l,r)	( (PFnormalize_TEMP = PFnormalize_var_decl_transition[PFnormalize_plank_7[l].f46][PFnormalize_plank_8[r].f24]) ? PFnormalize_TEMP + 152 : 0 )
#define PFnormalize_var_type_state(l,r)	( (PFnormalize_TEMP = PFnormalize_var_type_transition[PFnormalize_plank_7[l].f47][PFnormalize_plank_7[r].f48]) ? PFnormalize_TEMP + 154 : 0 )
#define PFnormalize_varref_state	158
#define PFnormalize_vars_state(l,r)	( (PFnormalize_TEMP = PFnormalize_vars_transition[PFnormalize_plank_7[l].f46][PFnormalize_plank_9[r].f17]) ? PFnormalize_TEMP + 158 : 0 )
#define PFnormalize_where_state(l,r)	( (PFnormalize_TEMP = PFnormalize_where_transition[PFnormalize_plank_7[l].f49][PFnormalize_plank_7[r].f50]) ? PFnormalize_TEMP + 159 : 0 )
#define PFnormalize_boundspc_decl_state	10
#define PFnormalize_revalid_state	118
#define PFnormalize_insert_state(l,r)	( (PFnormalize_TEMP = PFnormalize_insert_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 63 : 0 )
#define PFnormalize_delete_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_10[l].f0) ? PFnormalize_TEMP + 27 : 0 )
#define PFnormalize_replace_state(l,r)	( (PFnormalize_TEMP = PFnormalize_replace_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 111 : 0 )
#define PFnormalize_rename_state(l,r)	( (PFnormalize_TEMP = PFnormalize_rename_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 110 : 0 )
#define PFnormalize_transform_state(l,r)	( (PFnormalize_TEMP = PFnormalize_transform_transition[PFnormalize_plank_7[l].f51][PFnormalize_plank_7[r].f52]) ? PFnormalize_TEMP + 137 : 0 )
#define PFnormalize_modify_state(l,r)	( (PFnormalize_TEMP = PFnormalize_modify_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 84 : 0 )
#define PFnormalize_transbinds_state(l,r)	( (PFnormalize_TEMP = PFnormalize_transbinds_transition[PFnormalize_plank_7[l].f53][PFnormalize_plank_7[r].f51]) ? PFnormalize_TEMP + 136 : 0 )
#define PFnormalize_stmt_ty_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_9[l].f2) ? PFnormalize_TEMP + 132 : 0 )
#define PFnormalize_recursion_state(l,r)	( (PFnormalize_TEMP = PFnormalize_recursion_transition[PFnormalize_plank_7[l].f46][PFnormalize_plank_7[r].f54]) ? PFnormalize_TEMP + 109 : 0 )
#define PFnormalize_seed_state(l,r)	( (PFnormalize_TEMP = PFnormalize_seed_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_10[r].f0]) ? PFnormalize_TEMP + 123 : 0 )
#define PFnormalize_xrpc_state(l,r)	( (PFnormalize_TEMP = PFnormalize_xrpc_transition[PFnormalize_plank_10[l].f0][PFnormalize_plank_6[r].f55]) ? PFnormalize_TEMP + 160 : 0 )
#define PFnormalize_docmgmt_ty_state(l)	( (PFnormalize_TEMP = PFnormalize_plank_9[l].f2) ? PFnormalize_TEMP + 30 : 0 )

#ifdef __STDC__
int PFnormalize_state(int op, int l, int r) {
#else
int PFnormalize_state(op, l, r) int op; int l; int r; {
#endif
	register int PFnormalize_TEMP;
#ifndef NDEBUG
	switch (op) {
	case 1:
	case 3:
	case 5:
	case 7:
	case 8:
	case 9:
	case 10:
	case 11:
	case 12:
	case 16:
	case 17:
	case 19:
	case 20:
	case 23:
	case 27:
	case 28:
	case 29:
	case 30:
	case 32:
	case 33:
	case 36:
	case 38:
	case 39:
	case 40:
	case 41:
	case 42:
	case 43:
	case 45:
	case 46:
	case 47:
	case 49:
	case 50:
	case 51:
	case 56:
	case 57:
	case 58:
	case 59:
	case 60:
	case 61:
	case 62:
	case 64:
	case 66:
	case 71:
	case 72:
	case 76:
	case 77:
	case 78:
	case 80:
	case 82:
	case 83:
	case 84:
	case 86:
	case 88:
	case 91:
	case 93:
	case 97:
	case 98:
	case 99:
	case 101:
	case 104:
	case 105:
	case 106:
	case 107:
	case 108:
	case 109:
	case 112:
	case 113:
	case 115:
	case 116:
	case 119:
	case 121:
	case 122:
	case 123:
	case 124:
	case 125:
	case 127:
	case 128:
	case 129:
		PFnormalize_assert(r >= 0 && r < 162, PFnormalize_PANIC("Bad state %d passed to PFnormalize_state\n", r));
		/*FALLTHROUGH*/
	case 4:
	case 6:
	case 13:
	case 14:
	case 21:
	case 26:
	case 34:
	case 37:
	case 48:
	case 63:
	case 68:
	case 69:
	case 70:
	case 73:
	case 74:
	case 79:
	case 92:
	case 94:
	case 96:
	case 100:
	case 102:
	case 103:
	case 110:
	case 120:
	case 126:
	case 130:
		PFnormalize_assert(l >= 0 && l < 162, PFnormalize_PANIC("Bad state %d passed to PFnormalize_state\n", l));
		/*FALLTHROUGH*/
	case 15:
	case 18:
	case 22:
	case 24:
	case 25:
	case 31:
	case 44:
	case 52:
	case 53:
	case 54:
	case 55:
	case 65:
	case 67:
	case 75:
	case 81:
	case 85:
	case 87:
	case 89:
	case 90:
	case 95:
	case 114:
	case 117:
	case 118:
		break;
	}
#endif
	switch (op) {
	default: PFnormalize_PANIC("Unknown op %d in PFnormalize_state\n", op); abort(); return 0;
	case 1:
		return PFnormalize_and_state(l,r);
	case 3:
		return PFnormalize_args_state(l,r);
	case 4:
		return PFnormalize_atom_ty_state(l);
	case 5:
		return PFnormalize_attr_state(l,r);
	case 6:
		return PFnormalize_base_uri_state(l);
	case 7:
		return PFnormalize_bind_state(l,r);
	case 8:
		return PFnormalize_binds_state(l,r);
	case 9:
		return PFnormalize_case__state(l,r);
	case 10:
		return PFnormalize_cases_state(l,r);
	case 11:
		return PFnormalize_cast_state(l,r);
	case 12:
		return PFnormalize_castable_state(l,r);
	case 13:
		return PFnormalize_coll_decl_state(l);
	case 14:
		return PFnormalize_comment_state(l);
	case 15:
		return PFnormalize_constr_decl_state;
	case 16:
		return PFnormalize_contseq_state(l,r);
	case 17:
		return PFnormalize_decl_imps_state(l,r);
	case 18:
		return PFnormalize_def_order_state;
	case 19:
		return PFnormalize_default__state(l,r);
	case 20:
		return PFnormalize_div_state(l,r);
	case 21:
		return PFnormalize_doc_state(l);
	case 22:
		return PFnormalize_dot_state;
	case 23:
		return PFnormalize_elem_state(l,r);
	case 24:
		return PFnormalize_empty_seq_state;
	case 25:
		return PFnormalize_empty_ty_state;
	case 26:
		return PFnormalize_ens_decl_state(l);
	case 27:
		return PFnormalize_eq_state(l,r);
	case 28:
		return PFnormalize_every_state(l,r);
	case 29:
		return PFnormalize_except_state(l,r);
	case 30:
		return PFnormalize_exprseq_state(l,r);
	case 31:
		return PFnormalize_external_state;
	case 32:
		return PFnormalize_ext_expr_state(l,r);
	case 33:
		return PFnormalize_flwr_state(l,r);
	case 34:
		return PFnormalize_fns_decl_state(l);
	case 36:
		return PFnormalize_fun_decl_state(l,r);
	case 37:
		return PFnormalize_fun_ref_state(l);
	case 38:
		return PFnormalize_fun_sig_state(l,r);
	case 39:
		return PFnormalize_ge_state(l,r);
	case 40:
		return PFnormalize_gt_state(l,r);
	case 41:
		return PFnormalize_gtgt_state(l,r);
	case 42:
		return PFnormalize_idiv_state(l,r);
	case 43:
		return PFnormalize_if__state(l,r);
	case 44:
		return PFnormalize_copy_ns_state;
	case 45:
		return PFnormalize_instof_state(l,r);
	case 46:
		return PFnormalize_intersect_state(l,r);
	case 47:
		return PFnormalize_is_state(l,r);
	case 48:
		return PFnormalize_item_ty_state(l);
	case 49:
		return PFnormalize_le_state(l,r);
	case 50:
		return PFnormalize_let_state(l,r);
	case 51:
		return PFnormalize_lib_mod_state(l,r);
	case 52:
		return PFnormalize_lit_dbl_state;
	case 53:
		return PFnormalize_lit_dec_state;
	case 54:
		return PFnormalize_lit_int_state;
	case 55:
		return PFnormalize_lit_str_state;
	case 56:
		return PFnormalize_locpath_state(l,r);
	case 57:
		return PFnormalize_lt_state(l,r);
	case 58:
		return PFnormalize_ltlt_state(l,r);
	case 59:
		return PFnormalize_main_mod_state(l,r);
	case 60:
		return PFnormalize_minus_state(l,r);
	case 61:
		return PFnormalize_mod_state(l,r);
	case 62:
		return PFnormalize_mod_imp_state(l,r);
	case 63:
		return PFnormalize_mod_ns_state(l);
	case 64:
		return PFnormalize_mult_state(l,r);
	case 65:
		return PFnormalize_named_ty_state;
	case 66:
		return PFnormalize_ne_state(l,r);
	case 67:
		return PFnormalize_nil_state;
	case 68:
		return PFnormalize_node_ty_state(l);
	case 69:
		return PFnormalize_ns_decl_state(l);
	case 70:
		return PFnormalize_option_state(l);
	case 71:
		return PFnormalize_or_state(l,r);
	case 72:
		return PFnormalize_ord_ret_state(l,r);
	case 73:
		return PFnormalize_orderby_state(l);
	case 74:
		return PFnormalize_ordered_state(l);
	case 75:
		return PFnormalize_ordering_mode_state;
	case 76:
		return PFnormalize_orderspecs_state(l,r);
	case 77:
		return PFnormalize_param_state(l,r);
	case 78:
		return PFnormalize_params_state(l,r);
	case 79:
		return PFnormalize_pi_state(l);
	case 80:
		return PFnormalize_plus_state(l,r);
	case 81:
		return PFnormalize_pragma_state;
	case 82:
		return PFnormalize_pragmas_state(l,r);
	case 83:
		return PFnormalize_pred_state(l,r);
	case 84:
		return PFnormalize_range_state(l,r);
	case 85:
		return PFnormalize_req_name_state;
	case 86:
		return PFnormalize_req_ty_state(l,r);
	case 87:
		return PFnormalize_root__state;
	case 88:
		return PFnormalize_schm_ats_state(l,r);
	case 89:
		return PFnormalize_schm_attr_state;
	case 90:
		return PFnormalize_schm_elem_state;
	case 91:
		return PFnormalize_schm_imp_state(l,r);
	case 92:
		return PFnormalize_seq_ty_state(l);
	case 93:
		return PFnormalize_some_state(l,r);
	case 94:
		return PFnormalize_step_state(l);
	case 95:
		return PFnormalize_tag_state;
	case 96:
		return PFnormalize_text_state(l);
	case 97:
		return PFnormalize_then_else_state(l,r);
	case 98:
		return PFnormalize_treat_state(l,r);
	case 99:
		return PFnormalize_typeswitch_state(l,r);
	case 100:
		return PFnormalize_uminus_state(l);
	case 101:
		return PFnormalize_union__state(l,r);
	case 102:
		return PFnormalize_unordered_state(l);
	case 103:
		return PFnormalize_uplus_state(l);
	case 104:
		return PFnormalize_val_eq_state(l,r);
	case 105:
		return PFnormalize_val_ge_state(l,r);
	case 106:
		return PFnormalize_val_gt_state(l,r);
	case 107:
		return PFnormalize_val_le_state(l,r);
	case 108:
		return PFnormalize_val_lt_state(l,r);
	case 109:
		return PFnormalize_val_ne_state(l,r);
	case 110:
		return PFnormalize_validate_state(l);
	case 112:
		return PFnormalize_var_decl_state(l,r);
	case 113:
		return PFnormalize_var_type_state(l,r);
	case 114:
		return PFnormalize_varref_state;
	case 115:
		return PFnormalize_vars_state(l,r);
	case 116:
		return PFnormalize_where_state(l,r);
	case 117:
		return PFnormalize_boundspc_decl_state;
	case 118:
		return PFnormalize_revalid_state;
	case 119:
		return PFnormalize_insert_state(l,r);
	case 120:
		return PFnormalize_delete_state(l);
	case 121:
		return PFnormalize_replace_state(l,r);
	case 122:
		return PFnormalize_rename_state(l,r);
	case 123:
		return PFnormalize_transform_state(l,r);
	case 124:
		return PFnormalize_modify_state(l,r);
	case 125:
		return PFnormalize_transbinds_state(l,r);
	case 126:
		return PFnormalize_stmt_ty_state(l);
	case 127:
		return PFnormalize_recursion_state(l,r);
	case 128:
		return PFnormalize_seed_state(l,r);
	case 129:
		return PFnormalize_xrpc_state(l,r);
	case 130:
		return PFnormalize_docmgmt_ty_state(l);
	}
}
#ifdef PFnormalize_STATE_LABEL
#define PFnormalize_INCLUDE_EXTRA
#else
#ifdef STATE_LABEL
#define PFnormalize_INCLUDE_EXTRA
#define PFnormalize_STATE_LABEL 	STATE_LABEL
#define PFnormalize_NODEPTR_TYPE	NODEPTR_TYPE
#define PFnormalize_LEFT_CHILD  	LEFT_CHILD
#define PFnormalize_OP_LABEL    	OP_LABEL
#define PFnormalize_RIGHT_CHILD 	RIGHT_CHILD
#endif /* STATE_LABEL */
#endif /* PFnormalize_STATE_LABEL */

#ifdef PFnormalize_INCLUDE_EXTRA

#ifdef __STDC__
int PFnormalize_label(PFnormalize_NODEPTR_TYPE n) {
#else
int PFnormalize_label(n) PFnormalize_NODEPTR_TYPE n; {
#endif
	PFnormalize_assert(n, PFnormalize_PANIC("NULL pointer passed to PFnormalize_label\n"));
	switch (PFnormalize_OP_LABEL(n)) {
	default: PFnormalize_PANIC("Bad op %d in PFnormalize_label\n", PFnormalize_OP_LABEL(n)); abort(); return 0;
	case 15:
	case 18:
	case 22:
	case 24:
	case 25:
	case 31:
	case 44:
	case 52:
	case 53:
	case 54:
	case 55:
	case 65:
	case 67:
	case 75:
	case 81:
	case 85:
	case 87:
	case 89:
	case 90:
	case 95:
	case 114:
	case 117:
	case 118:
		return PFnormalize_STATE_LABEL(n) = PFnormalize_state(PFnormalize_OP_LABEL(n), 0, 0);
	case 4:
	case 6:
	case 13:
	case 14:
	case 21:
	case 26:
	case 34:
	case 37:
	case 48:
	case 63:
	case 68:
	case 69:
	case 70:
	case 73:
	case 74:
	case 79:
	case 92:
	case 94:
	case 96:
	case 100:
	case 102:
	case 103:
	case 110:
	case 120:
	case 126:
	case 130:
		return PFnormalize_STATE_LABEL(n) = PFnormalize_state(PFnormalize_OP_LABEL(n), PFnormalize_label(PFnormalize_LEFT_CHILD(n)), 0);
	case 1:
	case 3:
	case 5:
	case 7:
	case 8:
	case 9:
	case 10:
	case 11:
	case 12:
	case 16:
	case 17:
	case 19:
	case 20:
	case 23:
	case 27:
	case 28:
	case 29:
	case 30:
	case 32:
	case 33:
	case 36:
	case 38:
	case 39:
	case 40:
	case 41:
	case 42:
	case 43:
	case 45:
	case 46:
	case 47:
	case 49:
	case 50:
	case 51:
	case 56:
	case 57:
	case 58:
	case 59:
	case 60:
	case 61:
	case 62:
	case 64:
	case 66:
	case 71:
	case 72:
	case 76:
	case 77:
	case 78:
	case 80:
	case 82:
	case 83:
	case 84:
	case 86:
	case 88:
	case 91:
	case 93:
	case 97:
	case 98:
	case 99:
	case 101:
	case 104:
	case 105:
	case 106:
	case 107:
	case 108:
	case 109:
	case 112:
	case 113:
	case 115:
	case 116:
	case 119:
	case 121:
	case 122:
	case 123:
	case 124:
	case 125:
	case 127:
	case 128:
	case 129:
		return PFnormalize_STATE_LABEL(n) = PFnormalize_state(PFnormalize_OP_LABEL(n), PFnormalize_label(PFnormalize_LEFT_CHILD(n)), PFnormalize_label(PFnormalize_RIGHT_CHILD(n)));
	}
}
#ifdef __STDC__
PFnormalize_NODEPTR_TYPE * PFnormalize_kids(PFnormalize_NODEPTR_TYPE p, int rulenumber, PFnormalize_NODEPTR_TYPE *kids) {
#else
PFnormalize_NODEPTR_TYPE * PFnormalize_kids(p, rulenumber, kids) PFnormalize_NODEPTR_TYPE p; int rulenumber; PFnormalize_NODEPTR_TYPE *kids; {
#endif
	PFnormalize_assert(p, PFnormalize_PANIC("NULL node pointer passed to PFnormalize_kids\n"));
	PFnormalize_assert(kids, PFnormalize_PANIC("NULL kids pointer passed to PFnormalize_kids\n"));
	switch (rulenumber) {
	default:
		PFnormalize_PANIC("Unknown Rule %d in PFnormalize_kids;\n", rulenumber);
		abort();
		/* NOTREACHED */
	case 41:
		kids[0] = PFnormalize_LEFT_CHILD(PFnormalize_LEFT_CHILD(p));
		kids[1] = PFnormalize_RIGHT_CHILD(PFnormalize_LEFT_CHILD(p));
		break;
	case 50:
		kids[0] = PFnormalize_RIGHT_CHILD(PFnormalize_LEFT_CHILD(p));
		break;
	case 63:
		kids[0] = PFnormalize_LEFT_CHILD(PFnormalize_LEFT_CHILD(p));
		kids[1] = PFnormalize_LEFT_CHILD(PFnormalize_RIGHT_CHILD(p));
		kids[2] = PFnormalize_LEFT_CHILD(PFnormalize_RIGHT_CHILD(PFnormalize_RIGHT_CHILD(p)));
		kids[3] = PFnormalize_RIGHT_CHILD(PFnormalize_RIGHT_CHILD(PFnormalize_RIGHT_CHILD(p)));
		break;
	case 64:
		kids[0] = PFnormalize_LEFT_CHILD(PFnormalize_LEFT_CHILD(p));
		kids[1] = PFnormalize_RIGHT_CHILD(PFnormalize_LEFT_CHILD(p));
		kids[2] = PFnormalize_LEFT_CHILD(PFnormalize_RIGHT_CHILD(p));
		kids[3] = PFnormalize_LEFT_CHILD(PFnormalize_RIGHT_CHILD(PFnormalize_RIGHT_CHILD(p)));
		kids[4] = PFnormalize_RIGHT_CHILD(PFnormalize_RIGHT_CHILD(PFnormalize_RIGHT_CHILD(p)));
		break;
	case 70:
		kids[0] = PFnormalize_RIGHT_CHILD(PFnormalize_LEFT_CHILD(PFnormalize_LEFT_CHILD(p)));
		kids[1] = PFnormalize_RIGHT_CHILD(PFnormalize_LEFT_CHILD(p));
		kids[2] = PFnormalize_RIGHT_CHILD(p);
		break;
	case 49:
	case 71:
		kids[0] = PFnormalize_RIGHT_CHILD(PFnormalize_LEFT_CHILD(p));
		kids[1] = PFnormalize_RIGHT_CHILD(p);
		break;
	case 90:
		kids[0] = PFnormalize_LEFT_CHILD(p);
		kids[1] = PFnormalize_LEFT_CHILD(PFnormalize_RIGHT_CHILD(p));
		kids[2] = PFnormalize_LEFT_CHILD(PFnormalize_RIGHT_CHILD(PFnormalize_RIGHT_CHILD(p)));
		break;
	case 92:
		kids[0] = PFnormalize_LEFT_CHILD(p);
		kids[1] = PFnormalize_LEFT_CHILD(PFnormalize_RIGHT_CHILD(p));
		break;
	case 40:
	case 56:
	case 87:
	case 89:
	case 97:
	case 221:
		kids[0] = PFnormalize_LEFT_CHILD(PFnormalize_LEFT_CHILD(p));
		kids[1] = PFnormalize_RIGHT_CHILD(PFnormalize_LEFT_CHILD(p));
		kids[2] = PFnormalize_RIGHT_CHILD(p);
		break;
	case 46:
	case 65:
	case 83:
	case 84:
	case 98:
	case 106:
	case 161:
	case 162:
	case 166:
	case 170:
	case 176:
	case 197:
	case 200:
	case 202:
	case 203:
	case 212:
	case 213:
	case 214:
	case 215:
	case 222:
	case 234:
		kids[0] = PFnormalize_LEFT_CHILD(p);
		break;
	case 91:
	case 115:
	case 237:
		kids[0] = PFnormalize_LEFT_CHILD(p);
		kids[1] = PFnormalize_LEFT_CHILD(PFnormalize_RIGHT_CHILD(p));
		kids[2] = PFnormalize_RIGHT_CHILD(PFnormalize_RIGHT_CHILD(p));
		break;
	case 5:
	case 9:
	case 25:
	case 26:
	case 27:
	case 28:
	case 29:
	case 30:
	case 31:
	case 32:
	case 33:
	case 34:
	case 35:
	case 38:
	case 42:
	case 44:
	case 47:
	case 52:
	case 72:
	case 74:
	case 75:
	case 78:
	case 80:
	case 95:
	case 96:
	case 99:
	case 100:
	case 102:
	case 103:
	case 104:
	case 105:
	case 107:
	case 109:
	case 111:
	case 112:
	case 113:
	case 114:
	case 167:
	case 178:
	case 191:
	case 192:
	case 193:
	case 194:
	case 196:
	case 198:
	case 216:
	case 220:
	case 224:
	case 227:
	case 238:
		break;
	case 8:
	case 36:
	case 37:
	case 39:
	case 55:
	case 225:
	case 240:
		kids[0] = PFnormalize_RIGHT_CHILD(p);
		break;
	case 108:
	case 245:
		kids[0] = PFnormalize_LEFT_CHILD(PFnormalize_LEFT_CHILD(p));
		break;
	case 1:
	case 2:
	case 6:
	case 10:
	case 11:
	case 12:
	case 13:
	case 14:
	case 15:
	case 16:
	case 17:
	case 18:
	case 19:
	case 20:
	case 21:
	case 22:
	case 23:
	case 24:
	case 43:
	case 48:
	case 51:
	case 53:
	case 57:
	case 58:
	case 59:
	case 60:
	case 61:
	case 62:
	case 67:
	case 68:
	case 69:
	case 73:
	case 76:
	case 77:
	case 79:
	case 81:
	case 82:
	case 101:
	case 110:
	case 116:
	case 118:
	case 120:
	case 137:
	case 139:
	case 142:
	case 147:
	case 149:
	case 152:
	case 154:
	case 156:
	case 158:
	case 160:
	case 163:
	case 164:
	case 165:
	case 168:
	case 169:
	case 172:
	case 173:
	case 174:
	case 177:
	case 179:
	case 180:
	case 181:
	case 182:
	case 183:
	case 184:
	case 185:
	case 186:
	case 187:
	case 188:
	case 189:
	case 190:
	case 195:
	case 199:
	case 204:
	case 205:
	case 206:
	case 207:
	case 208:
	case 209:
	case 217:
	case 218:
	case 226:
	case 228:
	case 229:
	case 230:
	case 231:
	case 232:
	case 241:
	case 242:
	case 243:
	case 246:
		kids[0] = p;
		break;
	case 247:
		kids[0] = PFnormalize_RIGHT_CHILD(PFnormalize_LEFT_CHILD(p));
		kids[1] = PFnormalize_LEFT_CHILD(PFnormalize_RIGHT_CHILD(p));
		kids[2] = PFnormalize_RIGHT_CHILD(PFnormalize_RIGHT_CHILD(p));
		break;
	case 3:
	case 4:
	case 7:
	case 45:
	case 54:
	case 66:
	case 85:
	case 86:
	case 88:
	case 93:
	case 94:
	case 117:
	case 119:
	case 121:
	case 122:
	case 123:
	case 124:
	case 125:
	case 126:
	case 127:
	case 128:
	case 129:
	case 130:
	case 131:
	case 132:
	case 133:
	case 135:
	case 136:
	case 138:
	case 140:
	case 141:
	case 143:
	case 144:
	case 145:
	case 146:
	case 148:
	case 150:
	case 151:
	case 153:
	case 155:
	case 157:
	case 159:
	case 171:
	case 175:
	case 201:
	case 210:
	case 211:
	case 219:
	case 223:
	case 233:
	case 235:
	case 236:
	case 239:
	case 248:
		kids[0] = PFnormalize_LEFT_CHILD(p);
		kids[1] = PFnormalize_RIGHT_CHILD(p);
		break;
	case 244:
	case 249:
		kids[0] = PFnormalize_LEFT_CHILD(PFnormalize_LEFT_CHILD(p));
		kids[1] = PFnormalize_RIGHT_CHILD(p);
		break;
	}
	return kids;
}
#ifdef __STDC__
PFnormalize_NODEPTR_TYPE PFnormalize_child(PFnormalize_NODEPTR_TYPE p, int index) {
#else
PFnormalize_NODEPTR_TYPE PFnormalize_child(p, index) PFnormalize_NODEPTR_TYPE p; int index; {
#endif
	PFnormalize_assert(p, PFnormalize_PANIC("NULL pointer passed to PFnormalize_child\n"));
	switch (index) {
	case 0:
		return PFnormalize_LEFT_CHILD(p);
	case 1:
		return PFnormalize_RIGHT_CHILD(p);
	}
	PFnormalize_PANIC("Bad index %d in PFnormalize_child;\n", index);
	abort();
	return 0;
}
#ifdef __STDC__
int PFnormalize_op_label(PFnormalize_NODEPTR_TYPE p) {
#else
int PFnormalize_op_label(p) PFnormalize_NODEPTR_TYPE p; {
#endif
	PFnormalize_assert(p, PFnormalize_PANIC("NULL pointer passed to PFnormalize_op_label\n"));
	return PFnormalize_OP_LABEL(p);
}
#ifdef __STDC__
int PFnormalize_state_label(PFnormalize_NODEPTR_TYPE p) {
#else
int PFnormalize_state_label(p) PFnormalize_NODEPTR_TYPE p; {
#endif
	PFnormalize_assert(p, PFnormalize_PANIC("NULL pointer passed to PFnormalize_state_label\n"));
	return PFnormalize_STATE_LABEL(p);
}
#endif /* PFnormalize_INCLUDE_EXTRA */
#define PFnormalize_XRPCCall_NT 116
#define PFnormalize_RecursiveExpr_NT 115
#define PFnormalize_NewNameExpr_NT 114
#define PFnormalize_TargetExpr_NT 113
#define PFnormalize_SourceExpr_NT 112
#define PFnormalize_TransformBinding_NT 111
#define PFnormalize_TransformBindings_NT 110
#define PFnormalize_TransformExpr_NT 109
#define PFnormalize_RenameExpr_NT 108
#define PFnormalize_ReplaceExpr_NT 107
#define PFnormalize_DeleteExpr_NT 106
#define PFnormalize_InsertExpr_NT 105
#define PFnormalize_RevalidationDecl_NT 104
#define PFnormalize_Pragmas_NT 103
#define PFnormalize_ExtensionExpr_NT 102
#define PFnormalize_OptContentExpr_NT 101
#define PFnormalize_TagName_NT 100
#define PFnormalize_DocumentConstructor_NT 99
#define PFnormalize_PIConstructor_NT 98
#define PFnormalize_CommentConstructor_NT 97
#define PFnormalize_TextConstructor_NT 96
#define PFnormalize_AttributeConstructor_NT 95
#define PFnormalize_ElementConstructor_NT 94
#define PFnormalize_Constructor_NT 93
#define PFnormalize_UnorderedExpr_NT 92
#define PFnormalize_OrderedExpr_NT 91
#define PFnormalize_FuncArgList__NT 90
#define PFnormalize_OptFuncArgList__NT 89
#define PFnormalize_FunctionCall_NT 88
#define PFnormalize_ContextItemExpr_NT 87
#define PFnormalize_ParenthesizedExpr_NT 86
#define PFnormalize_StringLiteral_NT 85
#define PFnormalize_DoubleLiteral_NT 84
#define PFnormalize_DecimalLiteral_NT 83
#define PFnormalize_IntegerLiteral_NT 82
#define PFnormalize_NumericLiteral_NT 81
#define PFnormalize_Literal_NT 80
#define PFnormalize_PrimaryExpr_NT 79
#define PFnormalize_Step__NT 78
#define PFnormalize_FilterExpr_NT 77
#define PFnormalize_StepExpr_NT 76
#define PFnormalize_RelativePathExpr_NT 75
#define PFnormalize_PathExpr_NT 74
#define PFnormalize_ValidateExpr_NT 73
#define PFnormalize_ValueExpr_NT 72
#define PFnormalize_UnaryExpr_NT 71
#define PFnormalize_CastExpr_NT 70
#define PFnormalize_CastableExpr_NT 69
#define PFnormalize_TreatExpr_NT 68
#define PFnormalize_InstanceofExpr_NT 67
#define PFnormalize_IntersectExceptExpr_NT 66
#define PFnormalize_UnionExpr_NT 65
#define PFnormalize_MultiplicativeExpr_NT 64
#define PFnormalize_AdditiveExpr_NT 63
#define PFnormalize_RangeExpr_NT 62
#define PFnormalize_ComparisonExpr_NT 61
#define PFnormalize_AndExpr_NT 60
#define PFnormalize_OrExpr_NT 59
#define PFnormalize_IfExpr_NT 58
#define PFnormalize_ElemOrAttrType_NT 57
#define PFnormalize_OptElemOrAttrType__NT 56
#define PFnormalize_NodeTest_NT 55
#define PFnormalize_ItemType_NT 54
#define PFnormalize_SingleType_NT 53
#define PFnormalize_SequenceType_NT 52
#define PFnormalize_CaseClause_NT 51
#define PFnormalize_OptVarName__NT 50
#define PFnormalize_DefaultClause__NT 49
#define PFnormalize_CaseClauses__NT 48
#define PFnormalize_TypeswitchExpr_NT 47
#define PFnormalize_QuantifiedExpr_NT 46
#define PFnormalize_OrderSpecList_NT 45
#define PFnormalize_OrderByClause_NT 44
#define PFnormalize_WhereClause_NT 43
#define PFnormalize_OptOrderByClause__NT 42
#define PFnormalize_OptWhereClause__NT 41
#define PFnormalize_TypeDeclaration_NT 40
#define PFnormalize_OptTypeDeclaration__NT 39
#define PFnormalize_PositionalVar_NT 38
#define PFnormalize_OptPositionalVar__NT 37
#define PFnormalize_LetBinding__NT 36
#define PFnormalize_VarPosBinding_NT 35
#define PFnormalize_LetClause_NT 34
#define PFnormalize_VarBinding_NT 33
#define PFnormalize_VarBindings_NT 32
#define PFnormalize_FLWORExpr_NT 31
#define PFnormalize_ExprSingle_NT 30
#define PFnormalize_Expr_NT 29
#define PFnormalize_OptExpr_NT 28
#define PFnormalize_QueryBody_NT 27
#define PFnormalize_VariableDeclaration_NT 26
#define PFnormalize_OptSequenceType__NT 25
#define PFnormalize_Param_NT 24
#define PFnormalize_ParamList__NT 23
#define PFnormalize_OptParamList__NT 22
#define PFnormalize_FunctionDeclaration_NT 21
#define PFnormalize_OptImportLoc__NT 20
#define PFnormalize_ModuleImport_NT 19
#define PFnormalize_SchemaImport_NT 18
#define PFnormalize_NamespaceDecl_NT 17
#define PFnormalize_CopyNamespacesDecl_NT 16
#define PFnormalize_EmptyOrderingDecl_NT 15
#define PFnormalize_OrderingModeDecl_NT 14
#define PFnormalize_OptionDecl_NT 13
#define PFnormalize_ConstructionDecl_NT 12
#define PFnormalize_BaseURIDecl_NT 11
#define PFnormalize_DefaultCollationDecl_NT 10
#define PFnormalize_BoundarySpaceDecl_NT 9
#define PFnormalize_DefaultNamespaceDecl_NT 8
#define PFnormalize_DeclImport_NT 7
#define PFnormalize_DeclsImports_NT 6
#define PFnormalize_Prolog_NT 5
#define PFnormalize_ModuleDecl_NT 4
#define PFnormalize_LibraryModule_NT 3
#define PFnormalize_MainModule_NT 2
#define PFnormalize_Query_NT 1
#define PFnormalize_NT 116
char * PFnormalize_opname[] = {
	0, /* 0 */
	"and", /* 1 */
	0, /* 2 */
	"args", /* 3 */
	"atom_ty", /* 4 */
	"attr", /* 5 */
	"base_uri", /* 6 */
	"bind", /* 7 */
	"binds", /* 8 */
	"case_", /* 9 */
	"cases", /* 10 */
	"cast", /* 11 */
	"castable", /* 12 */
	"coll_decl", /* 13 */
	"comment", /* 14 */
	"constr_decl", /* 15 */
	"contseq", /* 16 */
	"decl_imps", /* 17 */
	"def_order", /* 18 */
	"default_", /* 19 */
	"div", /* 20 */
	"doc", /* 21 */
	"dot", /* 22 */
	"elem", /* 23 */
	"empty_seq", /* 24 */
	"empty_ty", /* 25 */
	"ens_decl", /* 26 */
	"eq", /* 27 */
	"every", /* 28 */
	"except", /* 29 */
	"exprseq", /* 30 */
	"external", /* 31 */
	"ext_expr", /* 32 */
	"flwr", /* 33 */
	"fns_decl", /* 34 */
	0, /* 35 */
	"fun_decl", /* 36 */
	"fun_ref", /* 37 */
	"fun_sig", /* 38 */
	"ge", /* 39 */
	"gt", /* 40 */
	"gtgt", /* 41 */
	"idiv", /* 42 */
	"if_", /* 43 */
	"copy_ns", /* 44 */
	"instof", /* 45 */
	"intersect", /* 46 */
	"is", /* 47 */
	"item_ty", /* 48 */
	"le", /* 49 */
	"let", /* 50 */
	"lib_mod", /* 51 */
	"lit_dbl", /* 52 */
	"lit_dec", /* 53 */
	"lit_int", /* 54 */
	"lit_str", /* 55 */
	"locpath", /* 56 */
	"lt", /* 57 */
	"ltlt", /* 58 */
	"main_mod", /* 59 */
	"minus", /* 60 */
	"mod", /* 61 */
	"mod_imp", /* 62 */
	"mod_ns", /* 63 */
	"mult", /* 64 */
	"named_ty", /* 65 */
	"ne", /* 66 */
	"nil", /* 67 */
	"node_ty", /* 68 */
	"ns_decl", /* 69 */
	"option", /* 70 */
	"or", /* 71 */
	"ord_ret", /* 72 */
	"orderby", /* 73 */
	"ordered", /* 74 */
	"ordering_mode", /* 75 */
	"orderspecs", /* 76 */
	"param", /* 77 */
	"params", /* 78 */
	"pi", /* 79 */
	"plus", /* 80 */
	"pragma", /* 81 */
	"pragmas", /* 82 */
	"pred", /* 83 */
	"range", /* 84 */
	"req_name", /* 85 */
	"req_ty", /* 86 */
	"root_", /* 87 */
	"schm_ats", /* 88 */
	"schm_attr", /* 89 */
	"schm_elem", /* 90 */
	"schm_imp", /* 91 */
	"seq_ty", /* 92 */
	"some", /* 93 */
	"step", /* 94 */
	"tag", /* 95 */
	"text", /* 96 */
	"then_else", /* 97 */
	"treat", /* 98 */
	"typeswitch", /* 99 */
	"uminus", /* 100 */
	"union_", /* 101 */
	"unordered", /* 102 */
	"uplus", /* 103 */
	"val_eq", /* 104 */
	"val_ge", /* 105 */
	"val_gt", /* 106 */
	"val_le", /* 107 */
	"val_lt", /* 108 */
	"val_ne", /* 109 */
	"validate", /* 110 */
	0, /* 111 */
	"var_decl", /* 112 */
	"var_type", /* 113 */
	"varref", /* 114 */
	"vars", /* 115 */
	"where", /* 116 */
	"boundspc_decl", /* 117 */
	"revalid", /* 118 */
	"insert", /* 119 */
	"delete", /* 120 */
	"replace", /* 121 */
	"rename", /* 122 */
	"transform", /* 123 */
	"modify", /* 124 */
	"transbinds", /* 125 */
	"stmt_ty", /* 126 */
	"recursion", /* 127 */
	"seed", /* 128 */
	"xrpc", /* 129 */
	"docmgmt_ty"
};
char PFnormalize_arity[] = {
	-1, /* 0 */
	2, /* 1 */
	-1, /* 2 */
	2, /* 3 */
	1, /* 4 */
	2, /* 5 */
	1, /* 6 */
	2, /* 7 */
	2, /* 8 */
	2, /* 9 */
	2, /* 10 */
	2, /* 11 */
	2, /* 12 */
	1, /* 13 */
	1, /* 14 */
	0, /* 15 */
	2, /* 16 */
	2, /* 17 */
	0, /* 18 */
	2, /* 19 */
	2, /* 20 */
	1, /* 21 */
	0, /* 22 */
	2, /* 23 */
	0, /* 24 */
	0, /* 25 */
	1, /* 26 */
	2, /* 27 */
	2, /* 28 */
	2, /* 29 */
	2, /* 30 */
	0, /* 31 */
	2, /* 32 */
	2, /* 33 */
	1, /* 34 */
	-1, /* 35 */
	2, /* 36 */
	1, /* 37 */
	2, /* 38 */
	2, /* 39 */
	2, /* 40 */
	2, /* 41 */
	2, /* 42 */
	2, /* 43 */
	0, /* 44 */
	2, /* 45 */
	2, /* 46 */
	2, /* 47 */
	1, /* 48 */
	2, /* 49 */
	2, /* 50 */
	2, /* 51 */
	0, /* 52 */
	0, /* 53 */
	0, /* 54 */
	0, /* 55 */
	2, /* 56 */
	2, /* 57 */
	2, /* 58 */
	2, /* 59 */
	2, /* 60 */
	2, /* 61 */
	2, /* 62 */
	1, /* 63 */
	2, /* 64 */
	0, /* 65 */
	2, /* 66 */
	0, /* 67 */
	1, /* 68 */
	1, /* 69 */
	1, /* 70 */
	2, /* 71 */
	2, /* 72 */
	1, /* 73 */
	1, /* 74 */
	0, /* 75 */
	2, /* 76 */
	2, /* 77 */
	2, /* 78 */
	1, /* 79 */
	2, /* 80 */
	0, /* 81 */
	2, /* 82 */
	2, /* 83 */
	2, /* 84 */
	0, /* 85 */
	2, /* 86 */
	0, /* 87 */
	2, /* 88 */
	0, /* 89 */
	0, /* 90 */
	2, /* 91 */
	1, /* 92 */
	2, /* 93 */
	1, /* 94 */
	0, /* 95 */
	1, /* 96 */
	2, /* 97 */
	2, /* 98 */
	2, /* 99 */
	1, /* 100 */
	2, /* 101 */
	1, /* 102 */
	1, /* 103 */
	2, /* 104 */
	2, /* 105 */
	2, /* 106 */
	2, /* 107 */
	2, /* 108 */
	2, /* 109 */
	1, /* 110 */
	-1, /* 111 */
	2, /* 112 */
	2, /* 113 */
	0, /* 114 */
	2, /* 115 */
	2, /* 116 */
	0, /* 117 */
	0, /* 118 */
	2, /* 119 */
	1, /* 120 */
	2, /* 121 */
	2, /* 122 */
	2, /* 123 */
	2, /* 124 */
	2, /* 125 */
	1, /* 126 */
	2, /* 127 */
	2, /* 128 */
	2, /* 129 */
	1
};
int PFnormalize_max_op = 130;
int PFnormalize_max_state = 161;
#define PFnormalize_Max_state 161
char *PFnormalize_string[] = {
	0,
	"Query: MainModule",
	"Query: LibraryModule",
	"MainModule: main_mod(Prolog, QueryBody)",
	"LibraryModule: lib_mod(ModuleDecl, Prolog)",
	"ModuleDecl: mod_ns(lit_str)",
	"Prolog: DeclsImports",
	"DeclsImports: decl_imps(DeclImport, DeclsImports)",
	"DeclsImports: decl_imps(nil, DeclsImports)",
	"DeclsImports: nil",
	"DeclImport: DefaultNamespaceDecl",
	"DeclImport: BoundarySpaceDecl",
	"DeclImport: DefaultCollationDecl",
	"DeclImport: BaseURIDecl",
	"DeclImport: ConstructionDecl",
	"DeclImport: OrderingModeDecl",
	"DeclImport: EmptyOrderingDecl",
	"DeclImport: CopyNamespacesDecl",
	"DeclImport: NamespaceDecl",
	"DeclImport: OptionDecl",
	"DeclImport: SchemaImport",
	"DeclImport: ModuleImport",
	"DeclImport: FunctionDeclaration",
	"DeclImport: VariableDeclaration",
	"DeclImport: LibraryModule",
	"DefaultNamespaceDecl: ens_decl(lit_str)",
	"DefaultNamespaceDecl: fns_decl(lit_str)",
	"BoundarySpaceDecl: boundspc_decl",
	"DefaultCollationDecl: coll_decl(lit_str)",
	"BaseURIDecl: base_uri(lit_str)",
	"ConstructionDecl: constr_decl",
	"OptionDecl: option(lit_str)",
	"OrderingModeDecl: ordering_mode",
	"EmptyOrderingDecl: def_order",
	"CopyNamespacesDecl: copy_ns",
	"NamespaceDecl: ns_decl(lit_str)",
	"SchemaImport: schm_imp(lit_str, OptImportLoc_)",
	"ModuleImport: mod_imp(lit_str, OptImportLoc_)",
	"OptImportLoc_: nil",
	"OptImportLoc_: schm_ats(lit_str, OptImportLoc_)",
	"FunctionDeclaration: fun_decl(fun_sig(OptParamList_, SequenceType), OptExpr)",
	"FunctionDeclaration: fun_decl(fun_sig(OptParamList_, SequenceType), external)",
	"OptParamList_: nil",
	"OptParamList_: ParamList_",
	"ParamList_: nil",
	"ParamList_: params(Param, ParamList_)",
	"Param: param(OptSequenceType_, varref)",
	"OptSequenceType_: nil",
	"OptSequenceType_: SequenceType",
	"VariableDeclaration: var_decl(var_type(varref, OptSequenceType_), OptExpr)",
	"VariableDeclaration: var_decl(var_type(varref, OptSequenceType_), external)",
	"QueryBody: OptExpr",
	"OptExpr: empty_seq",
	"OptExpr: Expr",
	"Expr: exprseq(Expr, OptExpr)",
	"Expr: exprseq(empty_seq, OptExpr)",
	"Expr: exprseq(exprseq(Expr, OptExpr), OptExpr)",
	"Expr: ExprSingle",
	"ExprSingle: FLWORExpr",
	"ExprSingle: QuantifiedExpr",
	"ExprSingle: TypeswitchExpr",
	"ExprSingle: IfExpr",
	"ExprSingle: OrExpr",
	"FLWORExpr: flwr(binds(VarBinding, nil), where(OptWhereClause_, ord_ret(OptOrderByClause_, OptExpr)))",
	"FLWORExpr: flwr(binds(VarBinding, VarBindings), where(OptWhereClause_, ord_ret(OptOrderByClause_, OptExpr)))",
	"VarBindings: binds(VarBinding, nil)",
	"VarBindings: binds(VarBinding, VarBindings)",
	"VarBinding: VarPosBinding",
	"VarBinding: LetClause",
	"LetClause: LetBinding_",
	"VarPosBinding: bind(vars(var_type(varref, OptTypeDeclaration_), OptPositionalVar_), OptExpr)",
	"LetBinding_: let(var_type(varref, OptTypeDeclaration_), OptExpr)",
	"OptPositionalVar_: nil",
	"OptPositionalVar_: PositionalVar",
	"PositionalVar: varref",
	"OptTypeDeclaration_: nil",
	"OptTypeDeclaration_: TypeDeclaration",
	"TypeDeclaration: SequenceType",
	"OptWhereClause_: nil",
	"OptWhereClause_: WhereClause",
	"OptOrderByClause_: nil",
	"OptOrderByClause_: OrderByClause",
	"WhereClause: OptExpr",
	"OrderByClause: orderby(OrderSpecList)",
	"OrderSpecList: orderspecs(OptExpr, nil)",
	"OrderSpecList: orderspecs(OptExpr, OrderSpecList)",
	"QuantifiedExpr: some(VarBindings, OptExpr)",
	"QuantifiedExpr: some(binds(VarBinding, VarBindings), OptExpr)",
	"QuantifiedExpr: every(VarBindings, OptExpr)",
	"QuantifiedExpr: every(binds(VarBinding, VarBindings), OptExpr)",
	"TypeswitchExpr: typeswitch(OptExpr, cases(CaseClause, cases(DefaultClause_, nil)))",
	"TypeswitchExpr: typeswitch(OptExpr, cases(CaseClause, CaseClauses_))",
	"CaseClauses_: cases(CaseClause, cases(DefaultClause_, nil))",
	"CaseClauses_: cases(CaseClause, CaseClauses_)",
	"DefaultClause_: default_(OptVarName_, OptExpr)",
	"OptVarName_: nil",
	"OptVarName_: varref",
	"CaseClause: case_(var_type(OptVarName_, SequenceType), OptExpr)",
	"SequenceType: seq_ty(ItemType)",
	"SequenceType: seq_ty(empty_ty)",
	"SingleType: seq_ty(atom_ty(nil))",
	"ItemType: NodeTest",
	"ItemType: item_ty(nil)",
	"ItemType: atom_ty(nil)",
	"ItemType: schm_elem",
	"ItemType: schm_attr",
	"NodeTest: node_ty(OptElemOrAttrType_)",
	"NodeTest: node_ty(lit_str)",
	"NodeTest: node_ty(node_ty(OptElemOrAttrType_))",
	"OptElemOrAttrType_: nil",
	"OptElemOrAttrType_: ElemOrAttrType",
	"ElemOrAttrType: req_ty(nil, nil)",
	"ElemOrAttrType: req_ty(nil, named_ty)",
	"ElemOrAttrType: req_ty(req_name, nil)",
	"ElemOrAttrType: req_ty(req_name, named_ty)",
	"IfExpr: if_(OptExpr, then_else(OptExpr, OptExpr))",
	"OrExpr: AndExpr",
	"OrExpr: or(AndExpr, OrExpr)",
	"AndExpr: ComparisonExpr",
	"AndExpr: and(ComparisonExpr, AndExpr)",
	"ComparisonExpr: RangeExpr",
	"ComparisonExpr: eq(RangeExpr, RangeExpr)",
	"ComparisonExpr: ne(RangeExpr, RangeExpr)",
	"ComparisonExpr: lt(RangeExpr, RangeExpr)",
	"ComparisonExpr: le(RangeExpr, RangeExpr)",
	"ComparisonExpr: gt(RangeExpr, RangeExpr)",
	"ComparisonExpr: ge(RangeExpr, RangeExpr)",
	"ComparisonExpr: val_eq(RangeExpr, RangeExpr)",
	"ComparisonExpr: val_ne(RangeExpr, RangeExpr)",
	"ComparisonExpr: val_lt(RangeExpr, RangeExpr)",
	"ComparisonExpr: val_le(RangeExpr, RangeExpr)",
	"ComparisonExpr: val_gt(RangeExpr, RangeExpr)",
	"ComparisonExpr: val_ge(RangeExpr, RangeExpr)",
	"ComparisonExpr: is(RangeExpr, RangeExpr)",
	0,
	"ComparisonExpr: ltlt(RangeExpr, RangeExpr)",
	"ComparisonExpr: gtgt(RangeExpr, RangeExpr)",
	"RangeExpr: AdditiveExpr",
	"RangeExpr: range(RangeExpr, RangeExpr)",
	"AdditiveExpr: MultiplicativeExpr",
	"AdditiveExpr: plus(AdditiveExpr, MultiplicativeExpr)",
	"AdditiveExpr: minus(AdditiveExpr, MultiplicativeExpr)",
	"MultiplicativeExpr: UnionExpr",
	"MultiplicativeExpr: mult(UnionExpr, MultiplicativeExpr)",
	"MultiplicativeExpr: div(UnionExpr, MultiplicativeExpr)",
	"MultiplicativeExpr: idiv(UnionExpr, MultiplicativeExpr)",
	"MultiplicativeExpr: mod(UnionExpr, MultiplicativeExpr)",
	"UnionExpr: IntersectExceptExpr",
	"UnionExpr: union_(IntersectExceptExpr, UnionExpr)",
	"IntersectExceptExpr: InstanceofExpr",
	"IntersectExceptExpr: intersect(InstanceofExpr, IntersectExceptExpr)",
	"IntersectExceptExpr: except(InstanceofExpr, IntersectExceptExpr)",
	"InstanceofExpr: TreatExpr",
	"InstanceofExpr: instof(TreatExpr, SequenceType)",
	"TreatExpr: CastableExpr",
	"TreatExpr: treat(CastableExpr, SequenceType)",
	"CastableExpr: CastExpr",
	"CastableExpr: castable(CastExpr, SingleType)",
	"CastExpr: UnaryExpr",
	"CastExpr: cast(UnaryExpr, SingleType)",
	"UnaryExpr: ValueExpr",
	"UnaryExpr: uminus(ValueExpr)",
	"UnaryExpr: uplus(ValueExpr)",
	"ValueExpr: ValidateExpr",
	"ValueExpr: PathExpr",
	"ValueExpr: ExtensionExpr",
	"ValidateExpr: validate(OptExpr)",
	"PathExpr: root_",
	"PathExpr: RelativePathExpr",
	"RelativePathExpr: StepExpr",
	"RelativePathExpr: step(NodeTest)",
	"RelativePathExpr: locpath(StepExpr, RelativePathExpr)",
	"StepExpr: Step_",
	"StepExpr: FilterExpr",
	"FilterExpr: PrimaryExpr",
	"FilterExpr: pred(RelativePathExpr, OptExpr)",
	"Step_: step(NodeTest)",
	"PrimaryExpr: Literal",
	"PrimaryExpr: varref",
	"PrimaryExpr: ParenthesizedExpr",
	"PrimaryExpr: ContextItemExpr",
	"PrimaryExpr: FunctionCall",
	"PrimaryExpr: Constructor",
	"PrimaryExpr: OrderedExpr",
	"PrimaryExpr: UnorderedExpr",
	"PrimaryExpr: XRPCCall",
	"Literal: NumericLiteral",
	"Literal: StringLiteral",
	"NumericLiteral: IntegerLiteral",
	"NumericLiteral: DecimalLiteral",
	"NumericLiteral: DoubleLiteral",
	"IntegerLiteral: lit_int",
	"DecimalLiteral: lit_dec",
	"DoubleLiteral: lit_dbl",
	"StringLiteral: lit_str",
	"ParenthesizedExpr: OptExpr",
	"ContextItemExpr: dot",
	"FunctionCall: fun_ref(OptFuncArgList_)",
	"OptFuncArgList_: nil",
	"OptFuncArgList_: FuncArgList_",
	"FuncArgList_: args(OptExpr, nil)",
	"FuncArgList_: args(OptExpr, FuncArgList_)",
	"OrderedExpr: ordered(OptExpr)",
	"UnorderedExpr: unordered(OptExpr)",
	"Constructor: ElementConstructor",
	"Constructor: AttributeConstructor",
	"Constructor: TextConstructor",
	"Constructor: CommentConstructor",
	"Constructor: PIConstructor",
	"Constructor: DocumentConstructor",
	"ElementConstructor: elem(TagName, OptContentExpr)",
	"AttributeConstructor: attr(TagName, OptContentExpr)",
	"TextConstructor: text(OptContentExpr)",
	"CommentConstructor: comment(OptContentExpr)",
	"PIConstructor: pi(OptContentExpr)",
	"DocumentConstructor: doc(OptContentExpr)",
	"TagName: tag",
	"TagName: OptExpr",
	"OptContentExpr: OptExpr",
	"OptContentExpr: contseq(OptContentExpr, OptContentExpr)",
	"OptContentExpr: contseq(empty_seq, empty_seq)",
	"OptContentExpr: contseq(contseq(OptContentExpr, OptContentExpr), OptContentExpr)",
	"ExtensionExpr: ext_expr(Pragmas, nil)",
	"ExtensionExpr: ext_expr(Pragmas, Expr)",
	"Pragmas: nil",
	"Pragmas: pragmas(pragma, Pragmas)",
	"DeclImport: RevalidationDecl",
	"RevalidationDecl: revalid",
	"ExprSingle: InsertExpr",
	"ExprSingle: DeleteExpr",
	"ExprSingle: RenameExpr",
	"ExprSingle: ReplaceExpr",
	"ExprSingle: TransformExpr",
	"InsertExpr: insert(SourceExpr, TargetExpr)",
	"DeleteExpr: delete(SourceExpr)",
	"ReplaceExpr: replace(SourceExpr, TargetExpr)",
	"RenameExpr: rename(SourceExpr, NewNameExpr)",
	"TransformExpr: transform(TransformBindings, modify(ExprSingle, ExprSingle))",
	"TransformBindings: nil",
	"TransformBindings: transbinds(TransformBinding, TransformBindings)",
	"TransformBinding: let(var_type(varref, nil), OptExpr)",
	"SourceExpr: ExprSingle",
	"TargetExpr: ExprSingle",
	"NewNameExpr: ExprSingle",
	"FunctionDeclaration: fun_decl(fun_sig(OptParamList_, seq_ty(stmt_ty(nil))), OptExpr)",
	"FunctionDeclaration: fun_decl(fun_sig(OptParamList_, seq_ty(stmt_ty(nil))), external)",
	"ExprSingle: RecursiveExpr",
	"RecursiveExpr: recursion(var_type(varref, OptTypeDeclaration_), seed(ExprSingle, ExprSingle))",
	"XRPCCall: xrpc(Expr, FunctionCall)",
	"FunctionDeclaration: fun_decl(fun_sig(OptParamList_, seq_ty(docmgmt_ty(nil))), OptExpr)",
};
int PFnormalize_max_rule = 249;
#define PFnormalize_Max_rule 249
short PFnormalize_rule_descriptor_0[] = { 0, 0 };
short PFnormalize_rule_descriptor_1[] = {   -1,   -2, };
short PFnormalize_rule_descriptor_2[] = {   -1,   -3, };
short PFnormalize_rule_descriptor_3[] = {   -2,   59,   -5,  -27, };
short PFnormalize_rule_descriptor_4[] = {   -3,   51,   -4,   -5, };
short PFnormalize_rule_descriptor_5[] = {   -4,   63,   55, };
short PFnormalize_rule_descriptor_6[] = {   -5,   -6, };
short PFnormalize_rule_descriptor_7[] = {   -6,   17,   -7,   -6, };
short PFnormalize_rule_descriptor_8[] = {   -6,   17,   67,   -6, };
short PFnormalize_rule_descriptor_9[] = {   -6,   67, };
short PFnormalize_rule_descriptor_10[] = {   -7,   -8, };
short PFnormalize_rule_descriptor_11[] = {   -7,   -9, };
short PFnormalize_rule_descriptor_12[] = {   -7,  -10, };
short PFnormalize_rule_descriptor_13[] = {   -7,  -11, };
short PFnormalize_rule_descriptor_14[] = {   -7,  -12, };
short PFnormalize_rule_descriptor_15[] = {   -7,  -14, };
short PFnormalize_rule_descriptor_16[] = {   -7,  -15, };
short PFnormalize_rule_descriptor_17[] = {   -7,  -16, };
short PFnormalize_rule_descriptor_18[] = {   -7,  -17, };
short PFnormalize_rule_descriptor_19[] = {   -7,  -13, };
short PFnormalize_rule_descriptor_20[] = {   -7,  -18, };
short PFnormalize_rule_descriptor_21[] = {   -7,  -19, };
short PFnormalize_rule_descriptor_22[] = {   -7,  -21, };
short PFnormalize_rule_descriptor_23[] = {   -7,  -26, };
short PFnormalize_rule_descriptor_24[] = {   -7,   -3, };
short PFnormalize_rule_descriptor_25[] = {   -8,   26,   55, };
short PFnormalize_rule_descriptor_26[] = {   -8,   34,   55, };
short PFnormalize_rule_descriptor_27[] = {   -9,  117, };
short PFnormalize_rule_descriptor_28[] = {  -10,   13,   55, };
short PFnormalize_rule_descriptor_29[] = {  -11,    6,   55, };
short PFnormalize_rule_descriptor_30[] = {  -12,   15, };
short PFnormalize_rule_descriptor_31[] = {  -13,   70,   55, };
short PFnormalize_rule_descriptor_32[] = {  -14,   75, };
short PFnormalize_rule_descriptor_33[] = {  -15,   18, };
short PFnormalize_rule_descriptor_34[] = {  -16,   44, };
short PFnormalize_rule_descriptor_35[] = {  -17,   69,   55, };
short PFnormalize_rule_descriptor_36[] = {  -18,   91,   55,  -20, };
short PFnormalize_rule_descriptor_37[] = {  -19,   62,   55,  -20, };
short PFnormalize_rule_descriptor_38[] = {  -20,   67, };
short PFnormalize_rule_descriptor_39[] = {  -20,   88,   55,  -20, };
short PFnormalize_rule_descriptor_40[] = {  -21,   36,   38,  -22,  -52,  -28, };
short PFnormalize_rule_descriptor_41[] = {  -21,   36,   38,  -22,  -52,   31, };
short PFnormalize_rule_descriptor_42[] = {  -22,   67, };
short PFnormalize_rule_descriptor_43[] = {  -22,  -23, };
short PFnormalize_rule_descriptor_44[] = {  -23,   67, };
short PFnormalize_rule_descriptor_45[] = {  -23,   78,  -24,  -23, };
short PFnormalize_rule_descriptor_46[] = {  -24,   77,  -25,  114, };
short PFnormalize_rule_descriptor_47[] = {  -25,   67, };
short PFnormalize_rule_descriptor_48[] = {  -25,  -52, };
short PFnormalize_rule_descriptor_49[] = {  -26,  112,  113,  114,  -25,  -28, };
short PFnormalize_rule_descriptor_50[] = {  -26,  112,  113,  114,  -25,   31, };
short PFnormalize_rule_descriptor_51[] = {  -27,  -28, };
short PFnormalize_rule_descriptor_52[] = {  -28,   24, };
short PFnormalize_rule_descriptor_53[] = {  -28,  -29, };
short PFnormalize_rule_descriptor_54[] = {  -29,   30,  -29,  -28, };
short PFnormalize_rule_descriptor_55[] = {  -29,   30,   24,  -28, };
short PFnormalize_rule_descriptor_56[] = {  -29,   30,   30,  -29,  -28,  -28, };
short PFnormalize_rule_descriptor_57[] = {  -29,  -30, };
short PFnormalize_rule_descriptor_58[] = {  -30,  -31, };
short PFnormalize_rule_descriptor_59[] = {  -30,  -46, };
short PFnormalize_rule_descriptor_60[] = {  -30,  -47, };
short PFnormalize_rule_descriptor_61[] = {  -30,  -58, };
short PFnormalize_rule_descriptor_62[] = {  -30,  -59, };
short PFnormalize_rule_descriptor_63[] = {  -31,   33,    8,  -33,   67,  116,  -41,   72,  -42,  -28, };
short PFnormalize_rule_descriptor_64[] = {  -31,   33,    8,  -33,  -32,  116,  -41,   72,  -42,  -28, };
short PFnormalize_rule_descriptor_65[] = {  -32,    8,  -33,   67, };
short PFnormalize_rule_descriptor_66[] = {  -32,    8,  -33,  -32, };
short PFnormalize_rule_descriptor_67[] = {  -33,  -35, };
short PFnormalize_rule_descriptor_68[] = {  -33,  -34, };
short PFnormalize_rule_descriptor_69[] = {  -34,  -36, };
short PFnormalize_rule_descriptor_70[] = {  -35,    7,  115,  113,  114,  -39,  -37,  -28, };
short PFnormalize_rule_descriptor_71[] = {  -36,   50,  113,  114,  -39,  -28, };
short PFnormalize_rule_descriptor_72[] = {  -37,   67, };
short PFnormalize_rule_descriptor_73[] = {  -37,  -38, };
short PFnormalize_rule_descriptor_74[] = {  -38,  114, };
short PFnormalize_rule_descriptor_75[] = {  -39,   67, };
short PFnormalize_rule_descriptor_76[] = {  -39,  -40, };
short PFnormalize_rule_descriptor_77[] = {  -40,  -52, };
short PFnormalize_rule_descriptor_78[] = {  -41,   67, };
short PFnormalize_rule_descriptor_79[] = {  -41,  -43, };
short PFnormalize_rule_descriptor_80[] = {  -42,   67, };
short PFnormalize_rule_descriptor_81[] = {  -42,  -44, };
short PFnormalize_rule_descriptor_82[] = {  -43,  -28, };
short PFnormalize_rule_descriptor_83[] = {  -44,   73,  -45, };
short PFnormalize_rule_descriptor_84[] = {  -45,   76,  -28,   67, };
short PFnormalize_rule_descriptor_85[] = {  -45,   76,  -28,  -45, };
short PFnormalize_rule_descriptor_86[] = {  -46,   93,  -32,  -28, };
short PFnormalize_rule_descriptor_87[] = {  -46,   93,    8,  -33,  -32,  -28, };
short PFnormalize_rule_descriptor_88[] = {  -46,   28,  -32,  -28, };
short PFnormalize_rule_descriptor_89[] = {  -46,   28,    8,  -33,  -32,  -28, };
short PFnormalize_rule_descriptor_90[] = {  -47,   99,  -28,   10,  -51,   10,  -49,   67, };
short PFnormalize_rule_descriptor_91[] = {  -47,   99,  -28,   10,  -51,  -48, };
short PFnormalize_rule_descriptor_92[] = {  -48,   10,  -51,   10,  -49,   67, };
short PFnormalize_rule_descriptor_93[] = {  -48,   10,  -51,  -48, };
short PFnormalize_rule_descriptor_94[] = {  -49,   19,  -50,  -28, };
short PFnormalize_rule_descriptor_95[] = {  -50,   67, };
short PFnormalize_rule_descriptor_96[] = {  -50,  114, };
short PFnormalize_rule_descriptor_97[] = {  -51,    9,  113,  -50,  -52,  -28, };
short PFnormalize_rule_descriptor_98[] = {  -52,   92,  -54, };
short PFnormalize_rule_descriptor_99[] = {  -52,   92,   25, };
short PFnormalize_rule_descriptor_100[] = {  -53,   92,    4,   67, };
short PFnormalize_rule_descriptor_101[] = {  -54,  -55, };
short PFnormalize_rule_descriptor_102[] = {  -54,   48,   67, };
short PFnormalize_rule_descriptor_103[] = {  -54,    4,   67, };
short PFnormalize_rule_descriptor_104[] = {  -54,   90, };
short PFnormalize_rule_descriptor_105[] = {  -54,   89, };
short PFnormalize_rule_descriptor_106[] = {  -55,   68,  -56, };
short PFnormalize_rule_descriptor_107[] = {  -55,   68,   55, };
short PFnormalize_rule_descriptor_108[] = {  -55,   68,   68,  -56, };
short PFnormalize_rule_descriptor_109[] = {  -56,   67, };
short PFnormalize_rule_descriptor_110[] = {  -56,  -57, };
short PFnormalize_rule_descriptor_111[] = {  -57,   86,   67,   67, };
short PFnormalize_rule_descriptor_112[] = {  -57,   86,   67,   65, };
short PFnormalize_rule_descriptor_113[] = {  -57,   86,   85,   67, };
short PFnormalize_rule_descriptor_114[] = {  -57,   86,   85,   65, };
short PFnormalize_rule_descriptor_115[] = {  -58,   43,  -28,   97,  -28,  -28, };
short PFnormalize_rule_descriptor_116[] = {  -59,  -60, };
short PFnormalize_rule_descriptor_117[] = {  -59,   71,  -60,  -59, };
short PFnormalize_rule_descriptor_118[] = {  -60,  -61, };
short PFnormalize_rule_descriptor_119[] = {  -60,    1,  -61,  -60, };
short PFnormalize_rule_descriptor_120[] = {  -61,  -62, };
short PFnormalize_rule_descriptor_121[] = {  -61,   27,  -62,  -62, };
short PFnormalize_rule_descriptor_122[] = {  -61,   66,  -62,  -62, };
short PFnormalize_rule_descriptor_123[] = {  -61,   57,  -62,  -62, };
short PFnormalize_rule_descriptor_124[] = {  -61,   49,  -62,  -62, };
short PFnormalize_rule_descriptor_125[] = {  -61,   40,  -62,  -62, };
short PFnormalize_rule_descriptor_126[] = {  -61,   39,  -62,  -62, };
short PFnormalize_rule_descriptor_127[] = {  -61,  104,  -62,  -62, };
short PFnormalize_rule_descriptor_128[] = {  -61,  109,  -62,  -62, };
short PFnormalize_rule_descriptor_129[] = {  -61,  108,  -62,  -62, };
short PFnormalize_rule_descriptor_130[] = {  -61,  107,  -62,  -62, };
short PFnormalize_rule_descriptor_131[] = {  -61,  106,  -62,  -62, };
short PFnormalize_rule_descriptor_132[] = {  -61,  105,  -62,  -62, };
short PFnormalize_rule_descriptor_133[] = {  -61,   47,  -62,  -62, };
short PFnormalize_rule_descriptor_135[] = {  -61,   58,  -62,  -62, };
short PFnormalize_rule_descriptor_136[] = {  -61,   41,  -62,  -62, };
short PFnormalize_rule_descriptor_137[] = {  -62,  -63, };
short PFnormalize_rule_descriptor_138[] = {  -62,   84,  -62,  -62, };
short PFnormalize_rule_descriptor_139[] = {  -63,  -64, };
short PFnormalize_rule_descriptor_140[] = {  -63,   80,  -63,  -64, };
short PFnormalize_rule_descriptor_141[] = {  -63,   60,  -63,  -64, };
short PFnormalize_rule_descriptor_142[] = {  -64,  -65, };
short PFnormalize_rule_descriptor_143[] = {  -64,   64,  -65,  -64, };
short PFnormalize_rule_descriptor_144[] = {  -64,   20,  -65,  -64, };
short PFnormalize_rule_descriptor_145[] = {  -64,   42,  -65,  -64, };
short PFnormalize_rule_descriptor_146[] = {  -64,   61,  -65,  -64, };
short PFnormalize_rule_descriptor_147[] = {  -65,  -66, };
short PFnormalize_rule_descriptor_148[] = {  -65,  101,  -66,  -65, };
short PFnormalize_rule_descriptor_149[] = {  -66,  -67, };
short PFnormalize_rule_descriptor_150[] = {  -66,   46,  -67,  -66, };
short PFnormalize_rule_descriptor_151[] = {  -66,   29,  -67,  -66, };
short PFnormalize_rule_descriptor_152[] = {  -67,  -68, };
short PFnormalize_rule_descriptor_153[] = {  -67,   45,  -68,  -52, };
short PFnormalize_rule_descriptor_154[] = {  -68,  -69, };
short PFnormalize_rule_descriptor_155[] = {  -68,   98,  -69,  -52, };
short PFnormalize_rule_descriptor_156[] = {  -69,  -70, };
short PFnormalize_rule_descriptor_157[] = {  -69,   12,  -70,  -53, };
short PFnormalize_rule_descriptor_158[] = {  -70,  -71, };
short PFnormalize_rule_descriptor_159[] = {  -70,   11,  -71,  -53, };
short PFnormalize_rule_descriptor_160[] = {  -71,  -72, };
short PFnormalize_rule_descriptor_161[] = {  -71,  100,  -72, };
short PFnormalize_rule_descriptor_162[] = {  -71,  103,  -72, };
short PFnormalize_rule_descriptor_163[] = {  -72,  -73, };
short PFnormalize_rule_descriptor_164[] = {  -72,  -74, };
short PFnormalize_rule_descriptor_165[] = {  -72, -102, };
short PFnormalize_rule_descriptor_166[] = {  -73,  110,  -28, };
short PFnormalize_rule_descriptor_167[] = {  -74,   87, };
short PFnormalize_rule_descriptor_168[] = {  -74,  -75, };
short PFnormalize_rule_descriptor_169[] = {  -75,  -76, };
short PFnormalize_rule_descriptor_170[] = {  -75,   94,  -55, };
short PFnormalize_rule_descriptor_171[] = {  -75,   56,  -76,  -75, };
short PFnormalize_rule_descriptor_172[] = {  -76,  -78, };
short PFnormalize_rule_descriptor_173[] = {  -76,  -77, };
short PFnormalize_rule_descriptor_174[] = {  -77,  -79, };
short PFnormalize_rule_descriptor_175[] = {  -77,   83,  -75,  -28, };
short PFnormalize_rule_descriptor_176[] = {  -78,   94,  -55, };
short PFnormalize_rule_descriptor_177[] = {  -79,  -80, };
short PFnormalize_rule_descriptor_178[] = {  -79,  114, };
short PFnormalize_rule_descriptor_179[] = {  -79,  -86, };
short PFnormalize_rule_descriptor_180[] = {  -79,  -87, };
short PFnormalize_rule_descriptor_181[] = {  -79,  -88, };
short PFnormalize_rule_descriptor_182[] = {  -79,  -93, };
short PFnormalize_rule_descriptor_183[] = {  -79,  -91, };
short PFnormalize_rule_descriptor_184[] = {  -79,  -92, };
short PFnormalize_rule_descriptor_185[] = {  -79, -116, };
short PFnormalize_rule_descriptor_186[] = {  -80,  -81, };
short PFnormalize_rule_descriptor_187[] = {  -80,  -85, };
short PFnormalize_rule_descriptor_188[] = {  -81,  -82, };
short PFnormalize_rule_descriptor_189[] = {  -81,  -83, };
short PFnormalize_rule_descriptor_190[] = {  -81,  -84, };
short PFnormalize_rule_descriptor_191[] = {  -82,   54, };
short PFnormalize_rule_descriptor_192[] = {  -83,   53, };
short PFnormalize_rule_descriptor_193[] = {  -84,   52, };
short PFnormalize_rule_descriptor_194[] = {  -85,   55, };
short PFnormalize_rule_descriptor_195[] = {  -86,  -28, };
short PFnormalize_rule_descriptor_196[] = {  -87,   22, };
short PFnormalize_rule_descriptor_197[] = {  -88,   37,  -89, };
short PFnormalize_rule_descriptor_198[] = {  -89,   67, };
short PFnormalize_rule_descriptor_199[] = {  -89,  -90, };
short PFnormalize_rule_descriptor_200[] = {  -90,    3,  -28,   67, };
short PFnormalize_rule_descriptor_201[] = {  -90,    3,  -28,  -90, };
short PFnormalize_rule_descriptor_202[] = {  -91,   74,  -28, };
short PFnormalize_rule_descriptor_203[] = {  -92,  102,  -28, };
short PFnormalize_rule_descriptor_204[] = {  -93,  -94, };
short PFnormalize_rule_descriptor_205[] = {  -93,  -95, };
short PFnormalize_rule_descriptor_206[] = {  -93,  -96, };
short PFnormalize_rule_descriptor_207[] = {  -93,  -97, };
short PFnormalize_rule_descriptor_208[] = {  -93,  -98, };
short PFnormalize_rule_descriptor_209[] = {  -93,  -99, };
short PFnormalize_rule_descriptor_210[] = {  -94,   23, -100, -101, };
short PFnormalize_rule_descriptor_211[] = {  -95,    5, -100, -101, };
short PFnormalize_rule_descriptor_212[] = {  -96,   96, -101, };
short PFnormalize_rule_descriptor_213[] = {  -97,   14, -101, };
short PFnormalize_rule_descriptor_214[] = {  -98,   79, -101, };
short PFnormalize_rule_descriptor_215[] = {  -99,   21, -101, };
short PFnormalize_rule_descriptor_216[] = { -100,   95, };
short PFnormalize_rule_descriptor_217[] = { -100,  -28, };
short PFnormalize_rule_descriptor_218[] = { -101,  -28, };
short PFnormalize_rule_descriptor_219[] = { -101,   16, -101, -101, };
short PFnormalize_rule_descriptor_220[] = { -101,   16,   24,   24, };
short PFnormalize_rule_descriptor_221[] = { -101,   16,   16, -101, -101, -101, };
short PFnormalize_rule_descriptor_222[] = { -102,   32, -103,   67, };
short PFnormalize_rule_descriptor_223[] = { -102,   32, -103,  -29, };
short PFnormalize_rule_descriptor_224[] = { -103,   67, };
short PFnormalize_rule_descriptor_225[] = { -103,   82,   81, -103, };
short PFnormalize_rule_descriptor_226[] = {   -7, -104, };
short PFnormalize_rule_descriptor_227[] = { -104,  118, };
short PFnormalize_rule_descriptor_228[] = {  -30, -105, };
short PFnormalize_rule_descriptor_229[] = {  -30, -106, };
short PFnormalize_rule_descriptor_230[] = {  -30, -108, };
short PFnormalize_rule_descriptor_231[] = {  -30, -107, };
short PFnormalize_rule_descriptor_232[] = {  -30, -109, };
short PFnormalize_rule_descriptor_233[] = { -105,  119, -112, -113, };
short PFnormalize_rule_descriptor_234[] = { -106,  120, -112, };
short PFnormalize_rule_descriptor_235[] = { -107,  121, -112, -113, };
short PFnormalize_rule_descriptor_236[] = { -108,  122, -112, -114, };
short PFnormalize_rule_descriptor_237[] = { -109,  123, -110,  124,  -30,  -30, };
short PFnormalize_rule_descriptor_238[] = { -110,   67, };
short PFnormalize_rule_descriptor_239[] = { -110,  125, -111, -110, };
short PFnormalize_rule_descriptor_240[] = { -111,   50,  113,  114,   67,  -28, };
short PFnormalize_rule_descriptor_241[] = { -112,  -30, };
short PFnormalize_rule_descriptor_242[] = { -113,  -30, };
short PFnormalize_rule_descriptor_243[] = { -114,  -30, };
short PFnormalize_rule_descriptor_244[] = {  -21,   36,   38,  -22,   92,  126,   67,  -28, };
short PFnormalize_rule_descriptor_245[] = {  -21,   36,   38,  -22,   92,  126,   67,   31, };
short PFnormalize_rule_descriptor_246[] = {  -30, -115, };
short PFnormalize_rule_descriptor_247[] = { -115,  127,  113,  114,  -39,  128,  -30,  -30, };
short PFnormalize_rule_descriptor_248[] = { -116,  129,  -29,  -88, };
short PFnormalize_rule_descriptor_249[] = {  -21,   36,   38,  -22,   92,  130,   67,  -28, };
/* PFnormalize_rule_descriptors[0][1] = 1 iff grammar is normal form. */
short * PFnormalize_rule_descriptors[] = {
	PFnormalize_rule_descriptor_0,
	PFnormalize_rule_descriptor_1,
	PFnormalize_rule_descriptor_2,
	PFnormalize_rule_descriptor_3,
	PFnormalize_rule_descriptor_4,
	PFnormalize_rule_descriptor_5,
	PFnormalize_rule_descriptor_6,
	PFnormalize_rule_descriptor_7,
	PFnormalize_rule_descriptor_8,
	PFnormalize_rule_descriptor_9,
	PFnormalize_rule_descriptor_10,
	PFnormalize_rule_descriptor_11,
	PFnormalize_rule_descriptor_12,
	PFnormalize_rule_descriptor_13,
	PFnormalize_rule_descriptor_14,
	PFnormalize_rule_descriptor_15,
	PFnormalize_rule_descriptor_16,
	PFnormalize_rule_descriptor_17,
	PFnormalize_rule_descriptor_18,
	PFnormalize_rule_descriptor_19,
	PFnormalize_rule_descriptor_20,
	PFnormalize_rule_descriptor_21,
	PFnormalize_rule_descriptor_22,
	PFnormalize_rule_descriptor_23,
	PFnormalize_rule_descriptor_24,
	PFnormalize_rule_descriptor_25,
	PFnormalize_rule_descriptor_26,
	PFnormalize_rule_descriptor_27,
	PFnormalize_rule_descriptor_28,
	PFnormalize_rule_descriptor_29,
	PFnormalize_rule_descriptor_30,
	PFnormalize_rule_descriptor_31,
	PFnormalize_rule_descriptor_32,
	PFnormalize_rule_descriptor_33,
	PFnormalize_rule_descriptor_34,
	PFnormalize_rule_descriptor_35,
	PFnormalize_rule_descriptor_36,
	PFnormalize_rule_descriptor_37,
	PFnormalize_rule_descriptor_38,
	PFnormalize_rule_descriptor_39,
	PFnormalize_rule_descriptor_40,
	PFnormalize_rule_descriptor_41,
	PFnormalize_rule_descriptor_42,
	PFnormalize_rule_descriptor_43,
	PFnormalize_rule_descriptor_44,
	PFnormalize_rule_descriptor_45,
	PFnormalize_rule_descriptor_46,
	PFnormalize_rule_descriptor_47,
	PFnormalize_rule_descriptor_48,
	PFnormalize_rule_descriptor_49,
	PFnormalize_rule_descriptor_50,
	PFnormalize_rule_descriptor_51,
	PFnormalize_rule_descriptor_52,
	PFnormalize_rule_descriptor_53,
	PFnormalize_rule_descriptor_54,
	PFnormalize_rule_descriptor_55,
	PFnormalize_rule_descriptor_56,
	PFnormalize_rule_descriptor_57,
	PFnormalize_rule_descriptor_58,
	PFnormalize_rule_descriptor_59,
	PFnormalize_rule_descriptor_60,
	PFnormalize_rule_descriptor_61,
	PFnormalize_rule_descriptor_62,
	PFnormalize_rule_descriptor_63,
	PFnormalize_rule_descriptor_64,
	PFnormalize_rule_descriptor_65,
	PFnormalize_rule_descriptor_66,
	PFnormalize_rule_descriptor_67,
	PFnormalize_rule_descriptor_68,
	PFnormalize_rule_descriptor_69,
	PFnormalize_rule_descriptor_70,
	PFnormalize_rule_descriptor_71,
	PFnormalize_rule_descriptor_72,
	PFnormalize_rule_descriptor_73,
	PFnormalize_rule_descriptor_74,
	PFnormalize_rule_descriptor_75,
	PFnormalize_rule_descriptor_76,
	PFnormalize_rule_descriptor_77,
	PFnormalize_rule_descriptor_78,
	PFnormalize_rule_descriptor_79,
	PFnormalize_rule_descriptor_80,
	PFnormalize_rule_descriptor_81,
	PFnormalize_rule_descriptor_82,
	PFnormalize_rule_descriptor_83,
	PFnormalize_rule_descriptor_84,
	PFnormalize_rule_descriptor_85,
	PFnormalize_rule_descriptor_86,
	PFnormalize_rule_descriptor_87,
	PFnormalize_rule_descriptor_88,
	PFnormalize_rule_descriptor_89,
	PFnormalize_rule_descriptor_90,
	PFnormalize_rule_descriptor_91,
	PFnormalize_rule_descriptor_92,
	PFnormalize_rule_descriptor_93,
	PFnormalize_rule_descriptor_94,
	PFnormalize_rule_descriptor_95,
	PFnormalize_rule_descriptor_96,
	PFnormalize_rule_descriptor_97,
	PFnormalize_rule_descriptor_98,
	PFnormalize_rule_descriptor_99,
	PFnormalize_rule_descriptor_100,
	PFnormalize_rule_descriptor_101,
	PFnormalize_rule_descriptor_102,
	PFnormalize_rule_descriptor_103,
	PFnormalize_rule_descriptor_104,
	PFnormalize_rule_descriptor_105,
	PFnormalize_rule_descriptor_106,
	PFnormalize_rule_descriptor_107,
	PFnormalize_rule_descriptor_108,
	PFnormalize_rule_descriptor_109,
	PFnormalize_rule_descriptor_110,
	PFnormalize_rule_descriptor_111,
	PFnormalize_rule_descriptor_112,
	PFnormalize_rule_descriptor_113,
	PFnormalize_rule_descriptor_114,
	PFnormalize_rule_descriptor_115,
	PFnormalize_rule_descriptor_116,
	PFnormalize_rule_descriptor_117,
	PFnormalize_rule_descriptor_118,
	PFnormalize_rule_descriptor_119,
	PFnormalize_rule_descriptor_120,
	PFnormalize_rule_descriptor_121,
	PFnormalize_rule_descriptor_122,
	PFnormalize_rule_descriptor_123,
	PFnormalize_rule_descriptor_124,
	PFnormalize_rule_descriptor_125,
	PFnormalize_rule_descriptor_126,
	PFnormalize_rule_descriptor_127,
	PFnormalize_rule_descriptor_128,
	PFnormalize_rule_descriptor_129,
	PFnormalize_rule_descriptor_130,
	PFnormalize_rule_descriptor_131,
	PFnormalize_rule_descriptor_132,
	PFnormalize_rule_descriptor_133,
	PFnormalize_rule_descriptor_0,
	PFnormalize_rule_descriptor_135,
	PFnormalize_rule_descriptor_136,
	PFnormalize_rule_descriptor_137,
	PFnormalize_rule_descriptor_138,
	PFnormalize_rule_descriptor_139,
	PFnormalize_rule_descriptor_140,
	PFnormalize_rule_descriptor_141,
	PFnormalize_rule_descriptor_142,
	PFnormalize_rule_descriptor_143,
	PFnormalize_rule_descriptor_144,
	PFnormalize_rule_descriptor_145,
	PFnormalize_rule_descriptor_146,
	PFnormalize_rule_descriptor_147,
	PFnormalize_rule_descriptor_148,
	PFnormalize_rule_descriptor_149,
	PFnormalize_rule_descriptor_150,
	PFnormalize_rule_descriptor_151,
	PFnormalize_rule_descriptor_152,
	PFnormalize_rule_descriptor_153,
	PFnormalize_rule_descriptor_154,
	PFnormalize_rule_descriptor_155,
	PFnormalize_rule_descriptor_156,
	PFnormalize_rule_descriptor_157,
	PFnormalize_rule_descriptor_158,
	PFnormalize_rule_descriptor_159,
	PFnormalize_rule_descriptor_160,
	PFnormalize_rule_descriptor_161,
	PFnormalize_rule_descriptor_162,
	PFnormalize_rule_descriptor_163,
	PFnormalize_rule_descriptor_164,
	PFnormalize_rule_descriptor_165,
	PFnormalize_rule_descriptor_166,
	PFnormalize_rule_descriptor_167,
	PFnormalize_rule_descriptor_168,
	PFnormalize_rule_descriptor_169,
	PFnormalize_rule_descriptor_170,
	PFnormalize_rule_descriptor_171,
	PFnormalize_rule_descriptor_172,
	PFnormalize_rule_descriptor_173,
	PFnormalize_rule_descriptor_174,
	PFnormalize_rule_descriptor_175,
	PFnormalize_rule_descriptor_176,
	PFnormalize_rule_descriptor_177,
	PFnormalize_rule_descriptor_178,
	PFnormalize_rule_descriptor_179,
	PFnormalize_rule_descriptor_180,
	PFnormalize_rule_descriptor_181,
	PFnormalize_rule_descriptor_182,
	PFnormalize_rule_descriptor_183,
	PFnormalize_rule_descriptor_184,
	PFnormalize_rule_descriptor_185,
	PFnormalize_rule_descriptor_186,
	PFnormalize_rule_descriptor_187,
	PFnormalize_rule_descriptor_188,
	PFnormalize_rule_descriptor_189,
	PFnormalize_rule_descriptor_190,
	PFnormalize_rule_descriptor_191,
	PFnormalize_rule_descriptor_192,
	PFnormalize_rule_descriptor_193,
	PFnormalize_rule_descriptor_194,
	PFnormalize_rule_descriptor_195,
	PFnormalize_rule_descriptor_196,
	PFnormalize_rule_descriptor_197,
	PFnormalize_rule_descriptor_198,
	PFnormalize_rule_descriptor_199,
	PFnormalize_rule_descriptor_200,
	PFnormalize_rule_descriptor_201,
	PFnormalize_rule_descriptor_202,
	PFnormalize_rule_descriptor_203,
	PFnormalize_rule_descriptor_204,
	PFnormalize_rule_descriptor_205,
	PFnormalize_rule_descriptor_206,
	PFnormalize_rule_descriptor_207,
	PFnormalize_rule_descriptor_208,
	PFnormalize_rule_descriptor_209,
	PFnormalize_rule_descriptor_210,
	PFnormalize_rule_descriptor_211,
	PFnormalize_rule_descriptor_212,
	PFnormalize_rule_descriptor_213,
	PFnormalize_rule_descriptor_214,
	PFnormalize_rule_descriptor_215,
	PFnormalize_rule_descriptor_216,
	PFnormalize_rule_descriptor_217,
	PFnormalize_rule_descriptor_218,
	PFnormalize_rule_descriptor_219,
	PFnormalize_rule_descriptor_220,
	PFnormalize_rule_descriptor_221,
	PFnormalize_rule_descriptor_222,
	PFnormalize_rule_descriptor_223,
	PFnormalize_rule_descriptor_224,
	PFnormalize_rule_descriptor_225,
	PFnormalize_rule_descriptor_226,
	PFnormalize_rule_descriptor_227,
	PFnormalize_rule_descriptor_228,
	PFnormalize_rule_descriptor_229,
	PFnormalize_rule_descriptor_230,
	PFnormalize_rule_descriptor_231,
	PFnormalize_rule_descriptor_232,
	PFnormalize_rule_descriptor_233,
	PFnormalize_rule_descriptor_234,
	PFnormalize_rule_descriptor_235,
	PFnormalize_rule_descriptor_236,
	PFnormalize_rule_descriptor_237,
	PFnormalize_rule_descriptor_238,
	PFnormalize_rule_descriptor_239,
	PFnormalize_rule_descriptor_240,
	PFnormalize_rule_descriptor_241,
	PFnormalize_rule_descriptor_242,
	PFnormalize_rule_descriptor_243,
	PFnormalize_rule_descriptor_244,
	PFnormalize_rule_descriptor_245,
	PFnormalize_rule_descriptor_246,
	PFnormalize_rule_descriptor_247,
	PFnormalize_rule_descriptor_248,
	PFnormalize_rule_descriptor_249,
};
short PFnormalize_cost[][4] = {
	{    0,    0,    0,    0}, /* (none) = 0 */
	{   10,    0,    0,    0}, /* Query: MainModule = 1 */
	{   10,    0,    0,    0}, /* Query: LibraryModule = 2 */
	{   10,    0,    0,    0}, /* MainModule: main_mod(Prolog, QueryBody) = 3 */
	{   10,    0,    0,    0}, /* LibraryModule: lib_mod(ModuleDecl, Prolog) = 4 */
	{   10,    0,    0,    0}, /* ModuleDecl: mod_ns(lit_str) = 5 */
	{   10,    0,    0,    0}, /* Prolog: DeclsImports = 6 */
	{   10,    0,    0,    0}, /* DeclsImports: decl_imps(DeclImport, DeclsImports) = 7 */
	{    1,    0,    0,    0}, /* DeclsImports: decl_imps(nil, DeclsImports) = 8 */
	{   10,    0,    0,    0}, /* DeclsImports: nil = 9 */
	{   10,    0,    0,    0}, /* DeclImport: DefaultNamespaceDecl = 10 */
	{   10,    0,    0,    0}, /* DeclImport: BoundarySpaceDecl = 11 */
	{   10,    0,    0,    0}, /* DeclImport: DefaultCollationDecl = 12 */
	{   10,    0,    0,    0}, /* DeclImport: BaseURIDecl = 13 */
	{   10,    0,    0,    0}, /* DeclImport: ConstructionDecl = 14 */
	{   10,    0,    0,    0}, /* DeclImport: OrderingModeDecl = 15 */
	{   10,    0,    0,    0}, /* DeclImport: EmptyOrderingDecl = 16 */
	{   10,    0,    0,    0}, /* DeclImport: CopyNamespacesDecl = 17 */
	{   10,    0,    0,    0}, /* DeclImport: NamespaceDecl = 18 */
	{   10,    0,    0,    0}, /* DeclImport: OptionDecl = 19 */
	{   10,    0,    0,    0}, /* DeclImport: SchemaImport = 20 */
	{   10,    0,    0,    0}, /* DeclImport: ModuleImport = 21 */
	{   10,    0,    0,    0}, /* DeclImport: FunctionDeclaration = 22 */
	{   10,    0,    0,    0}, /* DeclImport: VariableDeclaration = 23 */
	{   10,    0,    0,    0}, /* DeclImport: LibraryModule = 24 */
	{   10,    0,    0,    0}, /* DefaultNamespaceDecl: ens_decl(lit_str) = 25 */
	{   10,    0,    0,    0}, /* DefaultNamespaceDecl: fns_decl(lit_str) = 26 */
	{   10,    0,    0,    0}, /* BoundarySpaceDecl: boundspc_decl = 27 */
	{   10,    0,    0,    0}, /* DefaultCollationDecl: coll_decl(lit_str) = 28 */
	{   10,    0,    0,    0}, /* BaseURIDecl: base_uri(lit_str) = 29 */
	{   10,    0,    0,    0}, /* ConstructionDecl: constr_decl = 30 */
	{   10,    0,    0,    0}, /* OptionDecl: option(lit_str) = 31 */
	{   10,    0,    0,    0}, /* OrderingModeDecl: ordering_mode = 32 */
	{   10,    0,    0,    0}, /* EmptyOrderingDecl: def_order = 33 */
	{   10,    0,    0,    0}, /* CopyNamespacesDecl: copy_ns = 34 */
	{   10,    0,    0,    0}, /* NamespaceDecl: ns_decl(lit_str) = 35 */
	{   10,    0,    0,    0}, /* SchemaImport: schm_imp(lit_str, OptImportLoc_) = 36 */
	{   10,    0,    0,    0}, /* ModuleImport: mod_imp(lit_str, OptImportLoc_) = 37 */
	{   10,    0,    0,    0}, /* OptImportLoc_: nil = 38 */
	{   10,    0,    0,    0}, /* OptImportLoc_: schm_ats(lit_str, OptImportLoc_) = 39 */
	{   10,    0,    0,    0}, /* FunctionDeclaration: fun_decl(fun_sig(OptParamList_, SequenceType), OptExpr) = 40 */
	{   10,    0,    0,    0}, /* FunctionDeclaration: fun_decl(fun_sig(OptParamList_, SequenceType), external) = 41 */
	{   10,    0,    0,    0}, /* OptParamList_: nil = 42 */
	{   10,    0,    0,    0}, /* OptParamList_: ParamList_ = 43 */
	{   10,    0,    0,    0}, /* ParamList_: nil = 44 */
	{   10,    0,    0,    0}, /* ParamList_: params(Param, ParamList_) = 45 */
	{   10,    0,    0,    0}, /* Param: param(OptSequenceType_, varref) = 46 */
	{   10,    0,    0,    0}, /* OptSequenceType_: nil = 47 */
	{   10,    0,    0,    0}, /* OptSequenceType_: SequenceType = 48 */
	{   10,    0,    0,    0}, /* VariableDeclaration: var_decl(var_type(varref, OptSequenceType_), OptExpr) = 49 */
	{   10,    0,    0,    0}, /* VariableDeclaration: var_decl(var_type(varref, OptSequenceType_), external) = 50 */
	{   10,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   10,    0,    0,    0}, /* OptExpr: empty_seq = 52 */
	{   10,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   10,    0,    0,    0}, /* Expr: exprseq(Expr, OptExpr) = 54 */
	{   10,    0,    0,    0}, /* Expr: exprseq(empty_seq, OptExpr) = 55 */
	{    1,    0,    0,    0}, /* Expr: exprseq(exprseq(Expr, OptExpr), OptExpr) = 56 */
	{   10,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   10,    0,    0,    0}, /* ExprSingle: FLWORExpr = 58 */
	{   10,    0,    0,    0}, /* ExprSingle: QuantifiedExpr = 59 */
	{   10,    0,    0,    0}, /* ExprSingle: TypeswitchExpr = 60 */
	{   10,    0,    0,    0}, /* ExprSingle: IfExpr = 61 */
	{   10,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{   10,    0,    0,    0}, /* FLWORExpr: flwr(binds(VarBinding, nil), where(OptWhereClause_, ord_ret(OptOrderByClause_, OptExpr))) = 63 */
	{   10,    0,    0,    0}, /* FLWORExpr: flwr(binds(VarBinding, VarBindings), where(OptWhereClause_, ord_ret(OptOrderByClause_, OptExpr))) = 64 */
	{   10,    0,    0,    0}, /* VarBindings: binds(VarBinding, nil) = 65 */
	{   10,    0,    0,    0}, /* VarBindings: binds(VarBinding, VarBindings) = 66 */
	{   10,    0,    0,    0}, /* VarBinding: VarPosBinding = 67 */
	{   10,    0,    0,    0}, /* VarBinding: LetClause = 68 */
	{   10,    0,    0,    0}, /* LetClause: LetBinding_ = 69 */
	{   10,    0,    0,    0}, /* VarPosBinding: bind(vars(var_type(varref, OptTypeDeclaration_), OptPositionalVar_), OptExpr) = 70 */
	{   10,    0,    0,    0}, /* LetBinding_: let(var_type(varref, OptTypeDeclaration_), OptExpr) = 71 */
	{   10,    0,    0,    0}, /* OptPositionalVar_: nil = 72 */
	{   10,    0,    0,    0}, /* OptPositionalVar_: PositionalVar = 73 */
	{   10,    0,    0,    0}, /* PositionalVar: varref = 74 */
	{   10,    0,    0,    0}, /* OptTypeDeclaration_: nil = 75 */
	{   10,    0,    0,    0}, /* OptTypeDeclaration_: TypeDeclaration = 76 */
	{   10,    0,    0,    0}, /* TypeDeclaration: SequenceType = 77 */
	{   10,    0,    0,    0}, /* OptWhereClause_: nil = 78 */
	{   10,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{   10,    0,    0,    0}, /* OptOrderByClause_: nil = 80 */
	{   10,    0,    0,    0}, /* OptOrderByClause_: OrderByClause = 81 */
	{   10,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{   10,    0,    0,    0}, /* OrderByClause: orderby(OrderSpecList) = 83 */
	{   10,    0,    0,    0}, /* OrderSpecList: orderspecs(OptExpr, nil) = 84 */
	{   10,    0,    0,    0}, /* OrderSpecList: orderspecs(OptExpr, OrderSpecList) = 85 */
	{   10,    0,    0,    0}, /* QuantifiedExpr: some(VarBindings, OptExpr) = 86 */
	{   10,    0,    0,    0}, /* QuantifiedExpr: some(binds(VarBinding, VarBindings), OptExpr) = 87 */
	{   10,    0,    0,    0}, /* QuantifiedExpr: every(VarBindings, OptExpr) = 88 */
	{   10,    0,    0,    0}, /* QuantifiedExpr: every(binds(VarBinding, VarBindings), OptExpr) = 89 */
	{   10,    0,    0,    0}, /* TypeswitchExpr: typeswitch(OptExpr, cases(CaseClause, cases(DefaultClause_, nil))) = 90 */
	{   10,    0,    0,    0}, /* TypeswitchExpr: typeswitch(OptExpr, cases(CaseClause, CaseClauses_)) = 91 */
	{   10,    0,    0,    0}, /* CaseClauses_: cases(CaseClause, cases(DefaultClause_, nil)) = 92 */
	{   10,    0,    0,    0}, /* CaseClauses_: cases(CaseClause, CaseClauses_) = 93 */
	{   10,    0,    0,    0}, /* DefaultClause_: default_(OptVarName_, OptExpr) = 94 */
	{   10,    0,    0,    0}, /* OptVarName_: nil = 95 */
	{   10,    0,    0,    0}, /* OptVarName_: varref = 96 */
	{   10,    0,    0,    0}, /* CaseClause: case_(var_type(OptVarName_, SequenceType), OptExpr) = 97 */
	{   10,    0,    0,    0}, /* SequenceType: seq_ty(ItemType) = 98 */
	{   10,    0,    0,    0}, /* SequenceType: seq_ty(empty_ty) = 99 */
	{   10,    0,    0,    0}, /* SingleType: seq_ty(atom_ty(nil)) = 100 */
	{   10,    0,    0,    0}, /* ItemType: NodeTest = 101 */
	{   10,    0,    0,    0}, /* ItemType: item_ty(nil) = 102 */
	{   10,    0,    0,    0}, /* ItemType: atom_ty(nil) = 103 */
	{   10,    0,    0,    0}, /* ItemType: schm_elem = 104 */
	{   10,    0,    0,    0}, /* ItemType: schm_attr = 105 */
	{   10,    0,    0,    0}, /* NodeTest: node_ty(OptElemOrAttrType_) = 106 */
	{   10,    0,    0,    0}, /* NodeTest: node_ty(lit_str) = 107 */
	{   10,    0,    0,    0}, /* NodeTest: node_ty(node_ty(OptElemOrAttrType_)) = 108 */
	{   10,    0,    0,    0}, /* OptElemOrAttrType_: nil = 109 */
	{   10,    0,    0,    0}, /* OptElemOrAttrType_: ElemOrAttrType = 110 */
	{   10,    0,    0,    0}, /* ElemOrAttrType: req_ty(nil, nil) = 111 */
	{   10,    0,    0,    0}, /* ElemOrAttrType: req_ty(nil, named_ty) = 112 */
	{   10,    0,    0,    0}, /* ElemOrAttrType: req_ty(req_name, nil) = 113 */
	{   10,    0,    0,    0}, /* ElemOrAttrType: req_ty(req_name, named_ty) = 114 */
	{   10,    0,    0,    0}, /* IfExpr: if_(OptExpr, then_else(OptExpr, OptExpr)) = 115 */
	{   10,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   10,    0,    0,    0}, /* OrExpr: or(AndExpr, OrExpr) = 117 */
	{   10,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{   10,    0,    0,    0}, /* AndExpr: and(ComparisonExpr, AndExpr) = 119 */
	{   10,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{   10,    0,    0,    0}, /* ComparisonExpr: eq(RangeExpr, RangeExpr) = 121 */
	{   10,    0,    0,    0}, /* ComparisonExpr: ne(RangeExpr, RangeExpr) = 122 */
	{   10,    0,    0,    0}, /* ComparisonExpr: lt(RangeExpr, RangeExpr) = 123 */
	{   10,    0,    0,    0}, /* ComparisonExpr: le(RangeExpr, RangeExpr) = 124 */
	{   10,    0,    0,    0}, /* ComparisonExpr: gt(RangeExpr, RangeExpr) = 125 */
	{   10,    0,    0,    0}, /* ComparisonExpr: ge(RangeExpr, RangeExpr) = 126 */
	{   10,    0,    0,    0}, /* ComparisonExpr: val_eq(RangeExpr, RangeExpr) = 127 */
	{   10,    0,    0,    0}, /* ComparisonExpr: val_ne(RangeExpr, RangeExpr) = 128 */
	{   10,    0,    0,    0}, /* ComparisonExpr: val_lt(RangeExpr, RangeExpr) = 129 */
	{   10,    0,    0,    0}, /* ComparisonExpr: val_le(RangeExpr, RangeExpr) = 130 */
	{   10,    0,    0,    0}, /* ComparisonExpr: val_gt(RangeExpr, RangeExpr) = 131 */
	{   10,    0,    0,    0}, /* ComparisonExpr: val_ge(RangeExpr, RangeExpr) = 132 */
	{   10,    0,    0,    0}, /* ComparisonExpr: is(RangeExpr, RangeExpr) = 133 */
	{    0,    0,    0,    0}, /* (none) = 134 */
	{   10,    0,    0,    0}, /* ComparisonExpr: ltlt(RangeExpr, RangeExpr) = 135 */
	{   10,    0,    0,    0}, /* ComparisonExpr: gtgt(RangeExpr, RangeExpr) = 136 */
	{   10,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{   10,    0,    0,    0}, /* RangeExpr: range(RangeExpr, RangeExpr) = 138 */
	{   10,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{   10,    0,    0,    0}, /* AdditiveExpr: plus(AdditiveExpr, MultiplicativeExpr) = 140 */
	{   10,    0,    0,    0}, /* AdditiveExpr: minus(AdditiveExpr, MultiplicativeExpr) = 141 */
	{   10,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{   10,    0,    0,    0}, /* MultiplicativeExpr: mult(UnionExpr, MultiplicativeExpr) = 143 */
	{   10,    0,    0,    0}, /* MultiplicativeExpr: div(UnionExpr, MultiplicativeExpr) = 144 */
	{   10,    0,    0,    0}, /* MultiplicativeExpr: idiv(UnionExpr, MultiplicativeExpr) = 145 */
	{   10,    0,    0,    0}, /* MultiplicativeExpr: mod(UnionExpr, MultiplicativeExpr) = 146 */
	{   10,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{   10,    0,    0,    0}, /* UnionExpr: union_(IntersectExceptExpr, UnionExpr) = 148 */
	{   10,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{   10,    0,    0,    0}, /* IntersectExceptExpr: intersect(InstanceofExpr, IntersectExceptExpr) = 150 */
	{   10,    0,    0,    0}, /* IntersectExceptExpr: except(InstanceofExpr, IntersectExceptExpr) = 151 */
	{   10,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{   10,    0,    0,    0}, /* InstanceofExpr: instof(TreatExpr, SequenceType) = 153 */
	{   10,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{   10,    0,    0,    0}, /* TreatExpr: treat(CastableExpr, SequenceType) = 155 */
	{   10,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   10,    0,    0,    0}, /* CastableExpr: castable(CastExpr, SingleType) = 157 */
	{   10,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   10,    0,    0,    0}, /* CastExpr: cast(UnaryExpr, SingleType) = 159 */
	{   10,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   10,    0,    0,    0}, /* UnaryExpr: uminus(ValueExpr) = 161 */
	{   10,    0,    0,    0}, /* UnaryExpr: uplus(ValueExpr) = 162 */
	{   10,    0,    0,    0}, /* ValueExpr: ValidateExpr = 163 */
	{   10,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{   10,    0,    0,    0}, /* ValueExpr: ExtensionExpr = 165 */
	{   10,    0,    0,    0}, /* ValidateExpr: validate(OptExpr) = 166 */
	{   10,    0,    0,    0}, /* PathExpr: root_ = 167 */
	{   10,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   10,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{    1,    0,    0,    0}, /* RelativePathExpr: step(NodeTest) = 170 */
	{   10,    0,    0,    0}, /* RelativePathExpr: locpath(StepExpr, RelativePathExpr) = 171 */
	{   10,    0,    0,    0}, /* StepExpr: Step_ = 172 */
	{   10,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   10,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{   10,    0,    0,    0}, /* FilterExpr: pred(RelativePathExpr, OptExpr) = 175 */
	{   10,    0,    0,    0}, /* Step_: step(NodeTest) = 176 */
	{   10,    0,    0,    0}, /* PrimaryExpr: Literal = 177 */
	{   10,    0,    0,    0}, /* PrimaryExpr: varref = 178 */
	{   10,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{   10,    0,    0,    0}, /* PrimaryExpr: ContextItemExpr = 180 */
	{   10,    0,    0,    0}, /* PrimaryExpr: FunctionCall = 181 */
	{   10,    0,    0,    0}, /* PrimaryExpr: Constructor = 182 */
	{   10,    0,    0,    0}, /* PrimaryExpr: OrderedExpr = 183 */
	{   10,    0,    0,    0}, /* PrimaryExpr: UnorderedExpr = 184 */
	{   10,    0,    0,    0}, /* PrimaryExpr: XRPCCall = 185 */
	{   10,    0,    0,    0}, /* Literal: NumericLiteral = 186 */
	{   10,    0,    0,    0}, /* Literal: StringLiteral = 187 */
	{   10,    0,    0,    0}, /* NumericLiteral: IntegerLiteral = 188 */
	{   10,    0,    0,    0}, /* NumericLiteral: DecimalLiteral = 189 */
	{   10,    0,    0,    0}, /* NumericLiteral: DoubleLiteral = 190 */
	{   10,    0,    0,    0}, /* IntegerLiteral: lit_int = 191 */
	{   10,    0,    0,    0}, /* DecimalLiteral: lit_dec = 192 */
	{   10,    0,    0,    0}, /* DoubleLiteral: lit_dbl = 193 */
	{   10,    0,    0,    0}, /* StringLiteral: lit_str = 194 */
	{   10,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{   10,    0,    0,    0}, /* ContextItemExpr: dot = 196 */
	{   10,    0,    0,    0}, /* FunctionCall: fun_ref(OptFuncArgList_) = 197 */
	{   10,    0,    0,    0}, /* OptFuncArgList_: nil = 198 */
	{   10,    0,    0,    0}, /* OptFuncArgList_: FuncArgList_ = 199 */
	{   10,    0,    0,    0}, /* FuncArgList_: args(OptExpr, nil) = 200 */
	{   10,    0,    0,    0}, /* FuncArgList_: args(OptExpr, FuncArgList_) = 201 */
	{   10,    0,    0,    0}, /* OrderedExpr: ordered(OptExpr) = 202 */
	{   10,    0,    0,    0}, /* UnorderedExpr: unordered(OptExpr) = 203 */
	{   10,    0,    0,    0}, /* Constructor: ElementConstructor = 204 */
	{   10,    0,    0,    0}, /* Constructor: AttributeConstructor = 205 */
	{   10,    0,    0,    0}, /* Constructor: TextConstructor = 206 */
	{   10,    0,    0,    0}, /* Constructor: CommentConstructor = 207 */
	{   10,    0,    0,    0}, /* Constructor: PIConstructor = 208 */
	{   10,    0,    0,    0}, /* Constructor: DocumentConstructor = 209 */
	{   10,    0,    0,    0}, /* ElementConstructor: elem(TagName, OptContentExpr) = 210 */
	{   10,    0,    0,    0}, /* AttributeConstructor: attr(TagName, OptContentExpr) = 211 */
	{   10,    0,    0,    0}, /* TextConstructor: text(OptContentExpr) = 212 */
	{   10,    0,    0,    0}, /* CommentConstructor: comment(OptContentExpr) = 213 */
	{   10,    0,    0,    0}, /* PIConstructor: pi(OptContentExpr) = 214 */
	{   10,    0,    0,    0}, /* DocumentConstructor: doc(OptContentExpr) = 215 */
	{   10,    0,    0,    0}, /* TagName: tag = 216 */
	{   10,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   10,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{   10,    0,    0,    0}, /* OptContentExpr: contseq(OptContentExpr, OptContentExpr) = 219 */
	{    1,    0,    0,    0}, /* OptContentExpr: contseq(empty_seq, empty_seq) = 220 */
	{    1,    0,    0,    0}, /* OptContentExpr: contseq(contseq(OptContentExpr, OptContentExpr), OptContentExpr) = 221 */
	{   10,    0,    0,    0}, /* ExtensionExpr: ext_expr(Pragmas, nil) = 222 */
	{   10,    0,    0,    0}, /* ExtensionExpr: ext_expr(Pragmas, Expr) = 223 */
	{   10,    0,    0,    0}, /* Pragmas: nil = 224 */
	{   10,    0,    0,    0}, /* Pragmas: pragmas(pragma, Pragmas) = 225 */
	{   10,    0,    0,    0}, /* DeclImport: RevalidationDecl = 226 */
	{   10,    0,    0,    0}, /* RevalidationDecl: revalid = 227 */
	{   10,    0,    0,    0}, /* ExprSingle: InsertExpr = 228 */
	{   10,    0,    0,    0}, /* ExprSingle: DeleteExpr = 229 */
	{   10,    0,    0,    0}, /* ExprSingle: RenameExpr = 230 */
	{   10,    0,    0,    0}, /* ExprSingle: ReplaceExpr = 231 */
	{   10,    0,    0,    0}, /* ExprSingle: TransformExpr = 232 */
	{   10,    0,    0,    0}, /* InsertExpr: insert(SourceExpr, TargetExpr) = 233 */
	{   10,    0,    0,    0}, /* DeleteExpr: delete(SourceExpr) = 234 */
	{   10,    0,    0,    0}, /* ReplaceExpr: replace(SourceExpr, TargetExpr) = 235 */
	{   10,    0,    0,    0}, /* RenameExpr: rename(SourceExpr, NewNameExpr) = 236 */
	{   10,    0,    0,    0}, /* TransformExpr: transform(TransformBindings, modify(ExprSingle, ExprSingle)) = 237 */
	{   10,    0,    0,    0}, /* TransformBindings: nil = 238 */
	{   10,    0,    0,    0}, /* TransformBindings: transbinds(TransformBinding, TransformBindings) = 239 */
	{   10,    0,    0,    0}, /* TransformBinding: let(var_type(varref, nil), OptExpr) = 240 */
	{   10,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   10,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   10,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{   10,    0,    0,    0}, /* FunctionDeclaration: fun_decl(fun_sig(OptParamList_, seq_ty(stmt_ty(nil))), OptExpr) = 244 */
	{   10,    0,    0,    0}, /* FunctionDeclaration: fun_decl(fun_sig(OptParamList_, seq_ty(stmt_ty(nil))), external) = 245 */
	{   10,    0,    0,    0}, /* ExprSingle: RecursiveExpr = 246 */
	{   10,    0,    0,    0}, /* RecursiveExpr: recursion(var_type(varref, OptTypeDeclaration_), seed(ExprSingle, ExprSingle)) = 247 */
	{   10,    0,    0,    0}, /* XRPCCall: xrpc(Expr, FunctionCall) = 248 */
	{   10,    0,    0,    0}, /* FunctionDeclaration: fun_decl(fun_sig(OptParamList_, seq_ty(docmgmt_ty(nil))), OptExpr) = 249 */
};

short PFnormalize_delta_cost[162][117][4] = {
{{0}}, /* state 0 */
{ /* state #1: and(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   40,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   30,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   20,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{    0,    0,    0,    0}, /* AndExpr: and(ComparisonExpr, AndExpr) = 119 */
	{  220,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  210,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  200,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  190,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  180,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  170,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  160,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  150,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  140,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  130,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  120,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  110,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  100,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   90,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   80,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   70,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   50,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   30,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   30,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   30,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #2: args(dot, nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OptFuncArgList_: FuncArgList_ = 199 */
	{    0,    0,    0,    0}, /* FuncArgList_: args(OptExpr, nil) = 200 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #3: args(dot, args(dot, nil)) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OptFuncArgList_: FuncArgList_ = 199 */
	{    0,    0,    0,    0}, /* FuncArgList_: args(OptExpr, FuncArgList_) = 201 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #4: atom_ty(nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* ItemType: atom_ty(nil) = 103 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #5: attr(tag, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  230,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  220,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  210,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  250,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  200,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  190,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  180,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  170,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  160,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  150,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  140,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  130,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  120,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  110,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  100,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   90,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   80,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   70,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   50,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   40,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   30,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* PrimaryExpr: Constructor = 182 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* Constructor: AttributeConstructor = 205 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* AttributeConstructor: attr(TagName, OptContentExpr) = 211 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  240,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  220,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  220,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  220,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #6: base_uri(lit_str) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: BaseURIDecl = 13 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* BaseURIDecl: base_uri(lit_str) = 29 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #7: bind(vars(var_type(varref, nil), varref), dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* VarBinding: VarPosBinding = 67 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* VarPosBinding: bind(vars(var_type(varref, OptTypeDeclaration_), OptPositionalVar_), OptExpr) = 70 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #8: binds(let(var_type(varref, nil), dot), nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* VarBindings: binds(VarBinding, nil) = 65 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #9: binds(let(var_type(varref, nil), dot), binds(let(var_type(varref, nil), dot), nil)) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* VarBindings: binds(VarBinding, VarBindings) = 66 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #10: boundspc_decl */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: BoundarySpaceDecl = 11 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* BoundarySpaceDecl: boundspc_decl = 27 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #11: case_(var_type(varref, seq_ty(schm_attr)), dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* CaseClause: case_(var_type(OptVarName_, SequenceType), OptExpr) = 97 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #12: cases(case_(var_type(varref, seq_ty(schm_attr)), dot), cases(case_(var_type(varref, seq_ty(schm_attr)), dot), cases(default_(varref, dot), nil))) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* CaseClauses_: cases(CaseClause, CaseClauses_) = 93 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #13: cases(default_(varref, dot), nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #14: cases(case_(var_type(varref, seq_ty(schm_attr)), dot), cases(default_(varref, dot), nil)) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* CaseClauses_: cases(CaseClause, cases(DefaultClause_, nil)) = 92 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #15: cast(dot, seq_ty(atom_ty(nil))) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  150,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  140,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  130,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  120,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  160,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  150,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  100,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{   90,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{   80,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{   70,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{   60,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{   50,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{   40,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{   30,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{   20,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{   10,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{    0,    0,    0,    0}, /* CastExpr: cast(UnaryExpr, SingleType) = 159 */
	{  220,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  210,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  200,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  190,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{  180,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{  170,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{  160,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  150,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  150,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  150,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  130,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  130,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  130,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #16: castable(dot, seq_ty(atom_ty(nil))) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  140,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  130,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  120,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  110,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  150,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  140,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  100,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   90,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{   80,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{   70,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{   60,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{   50,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{   40,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{   30,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{   20,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{   10,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{    0,    0,    0,    0}, /* CastableExpr: castable(CastExpr, SingleType) = 157 */
	{  220,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  210,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  200,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  180,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{  170,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{  160,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{  150,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  140,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  140,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  140,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  120,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  120,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  120,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #17: coll_decl(lit_str) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: DefaultCollationDecl = 12 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* DefaultCollationDecl: coll_decl(lit_str) = 28 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #18: comment(dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  230,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  220,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  210,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  250,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  200,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  190,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  180,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  170,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  160,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  150,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  140,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  130,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  120,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  110,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  100,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   90,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   80,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   70,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   50,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   40,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   30,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* PrimaryExpr: Constructor = 182 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* Constructor: CommentConstructor = 207 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* CommentConstructor: comment(OptContentExpr) = 213 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  240,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  220,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  220,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  220,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #19: constr_decl */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: ConstructionDecl = 14 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* ConstructionDecl: constr_decl = 30 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #20: contseq(empty_seq, empty_seq) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* OptContentExpr: contseq(empty_seq, empty_seq) = 220 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #21: contseq(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OptContentExpr: contseq(OptContentExpr, OptContentExpr) = 219 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #22: contseq(contseq(dot, dot), dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* OptContentExpr: contseq(contseq(OptContentExpr, OptContentExpr), OptContentExpr) = 221 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #23: copy_ns */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: CopyNamespacesDecl = 17 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* CopyNamespacesDecl: copy_ns = 34 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #24: decl_imps(revalid, nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* Prolog: DeclsImports = 6 */
	{    0,    0,    0,    0}, /* DeclsImports: decl_imps(DeclImport, DeclsImports) = 7 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #25: decl_imps(nil, nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* Prolog: DeclsImports = 6 */
	{    0,    0,    0,    0}, /* DeclsImports: decl_imps(nil, DeclsImports) = 8 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #26: def_order */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: EmptyOrderingDecl = 16 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* EmptyOrderingDecl: def_order = 33 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #27: default_(varref, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* DefaultClause_: default_(OptVarName_, OptExpr) = 94 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #28: delete(dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   30,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   20,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   10,    0,    0,    0}, /* ExprSingle: DeleteExpr = 229 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  220,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  210,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  200,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  190,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  180,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  170,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  160,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  150,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  140,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  130,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  120,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  110,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  100,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   80,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   70,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   60,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   40,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* DeleteExpr: delete(SourceExpr) = 234 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   20,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   20,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #29: div(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   80,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   70,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   60,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  100,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   40,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{   30,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{   20,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{   10,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{    0,    0,    0,    0}, /* MultiplicativeExpr: div(UnionExpr, MultiplicativeExpr) = 144 */
	{  220,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  210,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  200,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  190,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  180,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  170,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  160,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  150,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  140,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  130,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{  120,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{  110,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{  100,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   90,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   70,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   70,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #30: doc(dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  230,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  220,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  210,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  250,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  200,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  190,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  180,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  170,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  160,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  150,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  140,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  130,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  120,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  110,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  100,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   90,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   80,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   70,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   50,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   40,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   30,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* PrimaryExpr: Constructor = 182 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* Constructor: DocumentConstructor = 209 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* DocumentConstructor: doc(OptContentExpr) = 215 */
	{  240,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  240,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  220,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  220,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  220,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #31: docmgmt_ty(nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #32: dot */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  220,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  210,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  200,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  180,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  170,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  160,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  150,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  140,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  130,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  120,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  110,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  100,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{   90,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   80,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   70,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   60,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   40,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   30,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   20,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* PrimaryExpr: ContextItemExpr = 180 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* ContextItemExpr: dot = 196 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  230,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  210,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  210,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  210,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #33: elem(tag, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  230,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  220,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  210,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  250,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  200,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  190,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  180,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  170,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  160,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  150,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  140,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  130,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  120,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  110,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  100,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   90,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   80,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   70,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   50,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   40,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   30,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* PrimaryExpr: Constructor = 182 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* Constructor: ElementConstructor = 204 */
	{    0,    0,    0,    0}, /* ElementConstructor: elem(TagName, OptContentExpr) = 210 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  240,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  220,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  220,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  220,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #34: empty_seq */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   10,    0,    0,    0}, /* OptExpr: empty_seq = 52 */
	{  230,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  220,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   30,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  210,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  200,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  190,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  180,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  170,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  160,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  150,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  140,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  130,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  120,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  110,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  100,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   90,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   80,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   60,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   50,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   40,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   30,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   20,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  230,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  230,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #35: empty_ty */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #36: ens_decl(lit_str) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: DefaultNamespaceDecl = 10 */
	{    0,    0,    0,    0}, /* DefaultNamespaceDecl: ens_decl(lit_str) = 25 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #37: eq(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   50,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   40,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   30,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   10,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{    0,    0,    0,    0}, /* ComparisonExpr: eq(RangeExpr, RangeExpr) = 121 */
	{  220,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  210,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  200,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  190,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  180,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  170,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  160,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  150,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  140,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  130,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  120,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  100,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   90,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   80,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   60,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   40,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   40,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #38: every(binds(let(var_type(varref, nil), dot), nil), dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   30,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   20,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   10,    0,    0,    0}, /* ExprSingle: QuantifiedExpr = 59 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* QuantifiedExpr: every(VarBindings, OptExpr) = 88 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  220,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  210,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  200,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  190,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  180,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  170,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  160,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  150,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  140,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  130,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  120,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  110,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  100,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   80,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   70,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   60,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   40,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   20,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   20,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #39: every(binds(let(var_type(varref, nil), dot), binds(let(var_type(varref, nil), dot), nil)), dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   30,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   20,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   10,    0,    0,    0}, /* ExprSingle: QuantifiedExpr = 59 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* QuantifiedExpr: every(binds(VarBinding, VarBindings), OptExpr) = 89 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  220,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  210,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  200,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  190,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  180,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  170,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  160,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  150,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  140,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  130,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  120,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  110,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  100,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   80,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   70,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   60,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   40,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   20,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   20,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #40: except(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  100,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   90,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   80,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  120,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   60,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{   50,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{   40,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{   30,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{   20,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{   10,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{    0,    0,    0,    0}, /* IntersectExceptExpr: except(InstanceofExpr, IntersectExceptExpr) = 151 */
	{  220,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  210,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  200,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  190,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  180,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  170,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  160,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  150,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{  140,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{  130,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{  120,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  110,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   90,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   90,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #41: exprseq(exprseq(dot, dot), dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   10,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{    0,    0,    0,    0}, /* Expr: exprseq(exprseq(Expr, OptExpr), OptExpr) = 56 */
	{  220,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   30,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  210,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  200,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  190,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  180,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  170,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  160,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  150,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  140,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  130,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  120,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  110,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  100,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   90,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   80,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   60,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   50,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   40,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   30,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   20,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  230,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  230,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #42: exprseq(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   30,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   20,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   10,    0,    0,    0}, /* Expr: exprseq(Expr, OptExpr) = 54 */
	{  230,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   30,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  220,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  210,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  200,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  190,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  180,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  170,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  160,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  150,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  140,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  130,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  120,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  110,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  100,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   90,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   80,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   70,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   60,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   50,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   30,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   30,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   30,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  240,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  240,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #43: exprseq(empty_seq, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   10,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{    0,    0,    0,    0}, /* Expr: exprseq(empty_seq, OptExpr) = 55 */
	{  220,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   30,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  210,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  200,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  190,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  180,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  170,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  160,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  150,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  140,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  130,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  120,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  110,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  100,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   90,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   80,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   60,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   50,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   40,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   30,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   20,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  230,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  230,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #44: ext_expr(nil, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  180,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  170,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  160,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  150,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  180,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  140,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  130,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  120,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  110,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  100,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{   90,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{   80,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{   70,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{   60,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{   50,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{   40,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   30,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   20,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   10,    0,    0,    0}, /* ValueExpr: ExtensionExpr = 165 */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  220,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{  210,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{  200,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  180,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  180,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  180,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* ExtensionExpr: ext_expr(Pragmas, Expr) = 223 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  160,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  160,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  160,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #45: ext_expr(nil, nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  180,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  170,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  160,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  150,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  180,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  140,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  130,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  120,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  110,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  100,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{   90,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{   80,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{   70,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{   60,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{   50,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{   40,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   30,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   20,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   10,    0,    0,    0}, /* ValueExpr: ExtensionExpr = 165 */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  220,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{  210,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{  200,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  180,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  180,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  180,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* ExtensionExpr: ext_expr(Pragmas, nil) = 222 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  160,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  160,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  160,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #46: external */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #47: flwr(binds(let(var_type(varref, nil), dot), nil), where(dot, ord_ret(nil, dot))) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   30,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   20,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   10,    0,    0,    0}, /* ExprSingle: FLWORExpr = 58 */
	{    0,    0,    0,    0}, /* FLWORExpr: flwr(binds(VarBinding, nil), where(OptWhereClause_, ord_ret(OptOrderByClause_, OptExpr))) = 63 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  220,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  210,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  200,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  190,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  180,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  170,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  160,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  150,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  140,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  130,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  120,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  110,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  100,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   80,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   70,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   60,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   40,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   20,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   20,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #48: flwr(binds(let(var_type(varref, nil), dot), binds(let(var_type(varref, nil), dot), nil)), where(dot, ord_ret(nil, dot))) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   30,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   20,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   10,    0,    0,    0}, /* ExprSingle: FLWORExpr = 58 */
	{    0,    0,    0,    0}, /* FLWORExpr: flwr(binds(VarBinding, VarBindings), where(OptWhereClause_, ord_ret(OptOrderByClause_, OptExpr))) = 64 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  220,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  210,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  200,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  190,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  180,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  170,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  160,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  150,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  140,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  130,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  120,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  110,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  100,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   80,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   70,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   60,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   40,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   20,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   20,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #49: fns_decl(lit_str) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: DefaultNamespaceDecl = 10 */
	{    0,    0,    0,    0}, /* DefaultNamespaceDecl: fns_decl(lit_str) = 26 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #50: fun_decl(fun_sig(nil, seq_ty(schm_attr)), dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: FunctionDeclaration = 22 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* FunctionDeclaration: fun_decl(fun_sig(OptParamList_, SequenceType), OptExpr) = 40 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #51: fun_decl(fun_sig(nil, seq_ty(stmt_ty(nil))), external) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: FunctionDeclaration = 22 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* FunctionDeclaration: fun_decl(fun_sig(OptParamList_, seq_ty(stmt_ty(nil))), external) = 245 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #52: fun_decl(fun_sig(nil, seq_ty(schm_attr)), external) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: FunctionDeclaration = 22 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* FunctionDeclaration: fun_decl(fun_sig(OptParamList_, SequenceType), external) = 41 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #53: fun_decl(fun_sig(nil, seq_ty(docmgmt_ty(nil))), dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: FunctionDeclaration = 22 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* FunctionDeclaration: fun_decl(fun_sig(OptParamList_, seq_ty(docmgmt_ty(nil))), OptExpr) = 249 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #54: fun_decl(fun_sig(nil, seq_ty(stmt_ty(nil))), dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: FunctionDeclaration = 22 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* FunctionDeclaration: fun_decl(fun_sig(OptParamList_, seq_ty(stmt_ty(nil))), OptExpr) = 244 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #55: fun_ref(nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  220,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  210,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  200,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  180,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  170,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  160,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  150,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  140,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  130,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  120,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  110,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  100,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{   90,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   80,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   70,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   60,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   40,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   30,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   20,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* PrimaryExpr: FunctionCall = 181 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* FunctionCall: fun_ref(OptFuncArgList_) = 197 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  230,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  210,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  210,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  210,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #56: fun_sig(nil, seq_ty(stmt_ty(nil))) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #57: fun_sig(nil, seq_ty(schm_attr)) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #58: fun_sig(nil, seq_ty(docmgmt_ty(nil))) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #59: ge(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   50,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   40,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   30,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   10,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{    0,    0,    0,    0}, /* ComparisonExpr: ge(RangeExpr, RangeExpr) = 126 */
	{  220,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  210,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  200,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  190,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  180,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  170,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  160,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  150,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  140,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  130,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  120,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  100,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   90,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   80,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   60,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   40,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   40,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #60: gt(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   50,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   40,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   30,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   10,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{    0,    0,    0,    0}, /* ComparisonExpr: gt(RangeExpr, RangeExpr) = 125 */
	{  220,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  210,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  200,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  190,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  180,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  170,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  160,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  150,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  140,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  130,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  120,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  100,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   90,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   80,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   60,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   40,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   40,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #61: gtgt(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   50,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   40,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   30,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   10,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{    0,    0,    0,    0}, /* ComparisonExpr: gtgt(RangeExpr, RangeExpr) = 136 */
	{  220,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  210,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  200,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  190,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  180,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  170,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  160,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  150,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  140,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  130,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  120,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  100,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   90,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   80,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   60,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   40,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   40,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #62: idiv(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   80,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   70,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   60,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  100,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   40,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{   30,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{   20,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{   10,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{    0,    0,    0,    0}, /* MultiplicativeExpr: idiv(UnionExpr, MultiplicativeExpr) = 145 */
	{  220,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  210,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  200,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  190,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  180,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  170,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  160,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  150,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  140,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  130,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{  120,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{  110,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{  100,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   90,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   70,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   70,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #63: if_(dot, then_else(dot, dot)) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   30,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   20,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   10,    0,    0,    0}, /* ExprSingle: IfExpr = 61 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* IfExpr: if_(OptExpr, then_else(OptExpr, OptExpr)) = 115 */
	{  230,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  220,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  210,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  200,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  190,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  180,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  170,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  160,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  150,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  140,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  130,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  120,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  110,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  100,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   80,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   70,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   60,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   40,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   20,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   20,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #64: insert(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   30,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   20,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   10,    0,    0,    0}, /* ExprSingle: InsertExpr = 228 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  220,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  210,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  200,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  190,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  180,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  170,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  160,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  150,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  140,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  130,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  120,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  110,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  100,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   80,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   70,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   60,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   40,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* InsertExpr: insert(SourceExpr, TargetExpr) = 233 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   20,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   20,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #65: instof(dot, seq_ty(schm_attr)) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  120,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  110,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  100,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   90,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  130,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  120,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   80,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   70,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{   60,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{   50,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{   40,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{   30,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{   20,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{   10,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{    0,    0,    0,    0}, /* InstanceofExpr: instof(TreatExpr, SequenceType) = 153 */
	{  220,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  210,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  200,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  190,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  180,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  170,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  160,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{  150,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{  140,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{  130,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  120,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  120,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  120,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  100,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  100,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  100,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #66: intersect(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  100,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   90,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   80,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  120,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   60,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{   50,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{   40,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{   30,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{   20,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{   10,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{    0,    0,    0,    0}, /* IntersectExceptExpr: intersect(InstanceofExpr, IntersectExceptExpr) = 150 */
	{  220,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  210,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  200,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  190,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  180,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  170,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  160,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  150,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{  140,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{  130,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{  120,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  110,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   90,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   90,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #67: is(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   50,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   40,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   30,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   10,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{    0,    0,    0,    0}, /* ComparisonExpr: is(RangeExpr, RangeExpr) = 133 */
	{  220,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  210,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  200,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  190,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  180,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  170,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  160,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  150,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  140,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  130,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  120,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  100,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   90,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   80,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   60,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   40,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   40,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #68: item_ty(nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* ItemType: item_ty(nil) = 102 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #69: le(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   50,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   40,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   30,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   10,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{    0,    0,    0,    0}, /* ComparisonExpr: le(RangeExpr, RangeExpr) = 124 */
	{  220,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  210,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  200,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  190,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  180,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  170,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  160,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  150,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  140,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  130,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  120,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  100,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   90,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   80,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   60,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   40,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   40,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #70: let(var_type(varref, seq_ty(schm_attr)), dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* VarBinding: LetClause = 68 */
	{   10,    0,    0,    0}, /* LetClause: LetBinding_ = 69 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* LetBinding_: let(var_type(varref, OptTypeDeclaration_), OptExpr) = 71 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #71: let(var_type(varref, nil), dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   30,    0,    0,    0}, /* VarBinding: LetClause = 68 */
	{   20,    0,    0,    0}, /* LetClause: LetBinding_ = 69 */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* LetBinding_: let(var_type(varref, OptTypeDeclaration_), OptExpr) = 71 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* TransformBinding: let(var_type(varref, nil), OptExpr) = 240 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #72: lib_mod(mod_ns(lit_str), nil) */
	{0},
	{   10,    0,    0,    0}, /* Query: LibraryModule = 2 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* LibraryModule: lib_mod(ModuleDecl, Prolog) = 4 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: LibraryModule = 24 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #73: lit_dbl */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  250,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  240,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  230,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  220,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  260,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  250,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  210,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  200,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  190,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  180,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  170,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  160,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  150,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  140,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  130,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  120,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  110,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  100,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   90,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   80,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   60,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   50,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   40,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   30,    0,    0,    0}, /* PrimaryExpr: Literal = 177 */
	{   20,    0,    0,    0}, /* Literal: NumericLiteral = 186 */
	{   10,    0,    0,    0}, /* NumericLiteral: DoubleLiteral = 190 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* DoubleLiteral: lit_dbl = 193 */
	{    0,    0,    0,    0}, /* (none) */
	{  250,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  250,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  250,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  230,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  230,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #74: lit_dec */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  250,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  240,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  230,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  220,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  260,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  250,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  210,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  200,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  190,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  180,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  170,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  160,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  150,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  140,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  130,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  120,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  110,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  100,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   90,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   80,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   60,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   50,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   40,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   30,    0,    0,    0}, /* PrimaryExpr: Literal = 177 */
	{   20,    0,    0,    0}, /* Literal: NumericLiteral = 186 */
	{   10,    0,    0,    0}, /* NumericLiteral: DecimalLiteral = 189 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* DecimalLiteral: lit_dec = 192 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  250,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  250,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  250,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  230,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  230,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #75: lit_int */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  250,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  240,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  230,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  220,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  260,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  250,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  210,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  200,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  190,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  180,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  170,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  160,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  150,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  140,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  130,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  120,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  110,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  100,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   90,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   80,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   60,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   50,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   40,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   30,    0,    0,    0}, /* PrimaryExpr: Literal = 177 */
	{   20,    0,    0,    0}, /* Literal: NumericLiteral = 186 */
	{   10,    0,    0,    0}, /* NumericLiteral: IntegerLiteral = 188 */
	{    0,    0,    0,    0}, /* IntegerLiteral: lit_int = 191 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  250,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  250,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  250,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  230,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  230,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #76: lit_str */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  250,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  240,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  230,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  220,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  260,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  250,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  210,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  200,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  190,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  180,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  170,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  160,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  150,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  140,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  130,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  120,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  110,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  100,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   90,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   80,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   60,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   50,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   40,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   30,    0,    0,    0}, /* PrimaryExpr: Literal = 177 */
	{   20,    0,    0,    0}, /* Literal: StringLiteral = 187 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* StringLiteral: lit_str = 194 */
	{  250,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  250,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  250,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  230,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  230,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #77: locpath(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  180,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  170,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  160,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  200,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  150,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  140,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  130,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  120,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  110,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  100,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{   90,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{   80,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{   70,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{   60,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{   50,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   40,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   30,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   20,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{    0,    0,    0,    0}, /* RelativePathExpr: locpath(StepExpr, RelativePathExpr) = 171 */
	{  220,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{  210,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{  200,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  190,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  170,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  170,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  170,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #78: lt(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   50,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   40,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   30,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   10,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{    0,    0,    0,    0}, /* ComparisonExpr: lt(RangeExpr, RangeExpr) = 123 */
	{  220,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  210,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  200,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  190,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  180,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  170,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  160,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  150,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  140,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  130,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  120,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  100,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   90,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   80,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   60,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   40,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   40,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #79: ltlt(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   50,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   40,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   30,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   10,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{    0,    0,    0,    0}, /* ComparisonExpr: ltlt(RangeExpr, RangeExpr) = 135 */
	{  220,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  210,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  200,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  190,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  180,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  170,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  160,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  150,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  140,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  130,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  120,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  100,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   90,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   80,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   60,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   40,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   40,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #80: main_mod(nil, dot) */
	{0},
	{   10,    0,    0,    0}, /* Query: MainModule = 1 */
	{    0,    0,    0,    0}, /* MainModule: main_mod(Prolog, QueryBody) = 3 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #81: minus(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   80,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   70,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   60,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   50,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   80,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   30,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{   20,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{   10,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{    0,    0,    0,    0}, /* AdditiveExpr: minus(AdditiveExpr, MultiplicativeExpr) = 141 */
	{  220,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  210,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  200,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  190,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  180,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  170,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  160,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  150,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  140,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  130,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  120,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{  110,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{  100,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   80,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   80,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   80,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   60,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   60,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #82: mod(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   80,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   70,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   60,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  100,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   40,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{   30,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{   20,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{   10,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{    0,    0,    0,    0}, /* MultiplicativeExpr: mod(UnionExpr, MultiplicativeExpr) = 146 */
	{  220,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  210,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  200,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  190,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  180,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  170,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  160,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  150,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  140,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  130,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{  120,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{  110,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{  100,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   90,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   70,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   70,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #83: mod_imp(lit_str, nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: ModuleImport = 21 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* ModuleImport: mod_imp(lit_str, OptImportLoc_) = 37 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #84: mod_ns(lit_str) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* ModuleDecl: mod_ns(lit_str) = 5 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #85: modify(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #86: mult(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   80,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   70,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   60,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  100,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   40,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{   30,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{   20,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{   10,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{    0,    0,    0,    0}, /* MultiplicativeExpr: mult(UnionExpr, MultiplicativeExpr) = 143 */
	{  220,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  210,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  200,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  190,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  180,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  170,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  160,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  150,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  140,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  130,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{  120,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{  110,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{  100,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   90,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   70,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   70,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #87: named_ty */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #88: ne(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   50,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   40,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   30,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   10,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{    0,    0,    0,    0}, /* ComparisonExpr: ne(RangeExpr, RangeExpr) = 122 */
	{  220,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  210,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  200,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  190,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  180,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  170,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  160,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  150,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  140,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  130,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  120,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  100,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   90,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   80,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   60,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   40,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   40,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #89: nil */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* Prolog: DeclsImports = 6 */
	{   10,    0,    0,    0}, /* DeclsImports: nil = 9 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OptImportLoc_: nil = 38 */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OptParamList_: nil = 42 */
	{   10,    0,    0,    0}, /* ParamList_: nil = 44 */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OptSequenceType_: nil = 47 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OptPositionalVar_: nil = 72 */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OptTypeDeclaration_: nil = 75 */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OptWhereClause_: nil = 78 */
	{   10,    0,    0,    0}, /* OptOrderByClause_: nil = 80 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OptVarName_: nil = 95 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OptElemOrAttrType_: nil = 109 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OptFuncArgList_: nil = 198 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* Pragmas: nil = 224 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* TransformBindings: nil = 238 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #90: node_ty(node_ty(nil)) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* ItemType: NodeTest = 101 */
	{    0,    0,    0,    0}, /* NodeTest: node_ty(node_ty(OptElemOrAttrType_)) = 108 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #91: node_ty(nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* ItemType: NodeTest = 101 */
	{   10,    0,    0,    0}, /* NodeTest: node_ty(OptElemOrAttrType_) = 106 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #92: node_ty(lit_str) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* ItemType: NodeTest = 101 */
	{    0,    0,    0,    0}, /* NodeTest: node_ty(lit_str) = 107 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #93: ns_decl(lit_str) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: NamespaceDecl = 18 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* NamespaceDecl: ns_decl(lit_str) = 35 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #94: option(lit_str) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: OptionDecl = 19 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* OptionDecl: option(lit_str) = 31 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #95: or(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   30,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   20,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   10,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* OrExpr: or(AndExpr, OrExpr) = 117 */
	{  220,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  210,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  200,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  190,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  180,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  170,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  160,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  150,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  140,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  130,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  120,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  110,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  100,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   80,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   70,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   60,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   40,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   20,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   20,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #96: ord_ret(nil, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #97: orderby(orderspecs(dot, nil)) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OptOrderByClause_: OrderByClause = 81 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* OrderByClause: orderby(OrderSpecList) = 83 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #98: ordered(dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  220,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  210,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  200,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  180,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  170,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  160,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  150,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  140,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  130,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  120,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  110,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  100,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{   90,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   80,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   70,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   60,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   40,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   30,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   20,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* PrimaryExpr: OrderedExpr = 183 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* OrderedExpr: ordered(OptExpr) = 202 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  230,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  210,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  210,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  210,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #99: ordering_mode */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: OrderingModeDecl = 15 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* OrderingModeDecl: ordering_mode = 32 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #100: orderspecs(dot, orderspecs(dot, nil)) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* OrderSpecList: orderspecs(OptExpr, OrderSpecList) = 85 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #101: orderspecs(dot, nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* OrderSpecList: orderspecs(OptExpr, nil) = 84 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #102: param(nil, varref) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* Param: param(OptSequenceType_, varref) = 46 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #103: params(param(nil, varref), nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OptParamList_: ParamList_ = 43 */
	{    0,    0,    0,    0}, /* ParamList_: params(Param, ParamList_) = 45 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #104: pi(dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  230,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  220,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  210,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  250,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  200,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  190,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  180,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  170,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  160,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  150,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  140,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  130,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  120,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  110,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  100,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   90,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   80,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   70,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   50,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   40,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   30,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* PrimaryExpr: Constructor = 182 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* Constructor: PIConstructor = 208 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* PIConstructor: pi(OptContentExpr) = 214 */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  240,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  220,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  220,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  220,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #105: plus(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   80,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   70,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   60,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   50,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   80,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   30,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{   20,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{   10,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{    0,    0,    0,    0}, /* AdditiveExpr: plus(AdditiveExpr, MultiplicativeExpr) = 140 */
	{  220,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  210,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  200,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  190,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  180,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  170,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  160,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  150,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  140,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  130,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  120,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{  110,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{  100,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   80,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   80,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   80,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   60,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   60,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #106: pragma */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #107: pragmas(pragma, nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* Pragmas: pragmas(pragma, Pragmas) = 225 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #108: pred(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  210,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  200,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  190,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  180,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  220,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  210,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  170,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  160,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  150,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  140,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  130,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  120,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  110,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  100,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{   90,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{   80,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{   70,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   60,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   50,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   40,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   30,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   20,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   10,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{    0,    0,    0,    0}, /* FilterExpr: pred(RelativePathExpr, OptExpr) = 175 */
	{    0,    0,    0,    0}, /* (none) */
	{  220,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  210,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  210,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  210,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  190,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  190,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #109: range(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   60,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   50,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   40,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   80,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   30,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   20,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{   10,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{    0,    0,    0,    0}, /* RangeExpr: range(RangeExpr, RangeExpr) = 138 */
	{  220,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  210,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  200,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  190,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  180,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  170,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  160,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  150,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  140,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  130,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  120,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  110,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{  100,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   90,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   80,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   70,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   50,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   50,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #110: recursion(var_type(varref, nil), seed(dot, dot)) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   30,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   20,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   10,    0,    0,    0}, /* ExprSingle: RecursiveExpr = 246 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  220,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  210,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  200,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  190,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  180,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  170,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  160,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  150,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  140,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  130,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  120,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  110,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  100,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   80,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   70,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   60,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   40,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   20,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   20,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* RecursiveExpr: recursion(var_type(varref, OptTypeDeclaration_), seed(ExprSingle, ExprSingle)) = 247 */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #111: rename(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   30,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   20,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   10,    0,    0,    0}, /* ExprSingle: RenameExpr = 230 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  220,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  210,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  200,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  190,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  180,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  170,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  160,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  150,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  140,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  130,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  120,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  110,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  100,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   80,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   70,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   60,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   40,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* RenameExpr: rename(SourceExpr, NewNameExpr) = 236 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   20,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   20,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #112: replace(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   30,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   20,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   10,    0,    0,    0}, /* ExprSingle: ReplaceExpr = 231 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  220,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  210,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  200,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  190,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  180,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  170,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  160,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  150,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  140,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  130,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  120,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  110,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  100,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   80,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   70,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   60,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   40,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* ReplaceExpr: replace(SourceExpr, TargetExpr) = 235 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   20,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   20,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #113: req_name */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #114: req_ty(nil, named_ty) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OptElemOrAttrType_: ElemOrAttrType = 110 */
	{    0,    0,    0,    0}, /* ElemOrAttrType: req_ty(nil, named_ty) = 112 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #115: req_ty(req_name, nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OptElemOrAttrType_: ElemOrAttrType = 110 */
	{    0,    0,    0,    0}, /* ElemOrAttrType: req_ty(req_name, nil) = 113 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #116: req_ty(nil, nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OptElemOrAttrType_: ElemOrAttrType = 110 */
	{    0,    0,    0,    0}, /* ElemOrAttrType: req_ty(nil, nil) = 111 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #117: req_ty(req_name, named_ty) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OptElemOrAttrType_: ElemOrAttrType = 110 */
	{    0,    0,    0,    0}, /* ElemOrAttrType: req_ty(req_name, named_ty) = 114 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #118: revalid */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: RevalidationDecl = 226 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* RevalidationDecl: revalid = 227 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #119: root_ */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  180,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  170,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  160,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  150,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  180,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  140,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  130,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  120,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  110,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  100,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{   90,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{   80,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{   70,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{   60,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{   50,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{   40,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   30,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   20,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   10,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* PathExpr: root_ = 167 */
	{  220,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{  210,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{  200,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  180,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  180,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  180,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  160,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  160,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  160,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #120: schm_ats(lit_str, nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* OptImportLoc_: schm_ats(lit_str, OptImportLoc_) = 39 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #121: schm_attr */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* ItemType: schm_attr = 105 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #122: schm_elem */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* ItemType: schm_elem = 104 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #123: schm_imp(lit_str, nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: SchemaImport = 20 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* SchemaImport: schm_imp(lit_str, OptImportLoc_) = 36 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #124: seed(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #125: seq_ty(empty_ty) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OptSequenceType_: SequenceType = 48 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* OptTypeDeclaration_: TypeDeclaration = 76 */
	{   10,    0,    0,    0}, /* TypeDeclaration: SequenceType = 77 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* SequenceType: seq_ty(empty_ty) = 99 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #126: seq_ty(schm_attr) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OptSequenceType_: SequenceType = 48 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* OptTypeDeclaration_: TypeDeclaration = 76 */
	{   10,    0,    0,    0}, /* TypeDeclaration: SequenceType = 77 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* SequenceType: seq_ty(ItemType) = 98 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #127: seq_ty(atom_ty(nil)) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* OptSequenceType_: SequenceType = 48 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   30,    0,    0,    0}, /* OptTypeDeclaration_: TypeDeclaration = 76 */
	{   20,    0,    0,    0}, /* TypeDeclaration: SequenceType = 77 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* SequenceType: seq_ty(ItemType) = 98 */
	{    0,    0,    0,    0}, /* SingleType: seq_ty(atom_ty(nil)) = 100 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #128: seq_ty(stmt_ty(nil)) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #129: seq_ty(docmgmt_ty(nil)) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #130: some(binds(let(var_type(varref, nil), dot), binds(let(var_type(varref, nil), dot), nil)), dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   30,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   20,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   10,    0,    0,    0}, /* ExprSingle: QuantifiedExpr = 59 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* QuantifiedExpr: some(binds(VarBinding, VarBindings), OptExpr) = 87 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  220,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  210,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  200,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  190,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  180,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  170,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  160,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  150,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  140,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  130,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  120,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  110,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  100,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   80,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   70,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   60,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   40,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   20,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   20,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #131: some(binds(let(var_type(varref, nil), dot), nil), dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   30,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   20,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   10,    0,    0,    0}, /* ExprSingle: QuantifiedExpr = 59 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* QuantifiedExpr: some(VarBindings, OptExpr) = 86 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  220,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  210,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  200,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  190,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  180,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  170,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  160,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  150,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  140,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  130,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  120,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  110,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  100,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   80,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   70,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   60,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   40,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   20,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   20,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #132: step(node_ty(nil)) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  180,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  170,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  160,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  200,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  150,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  140,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  130,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  120,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  110,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  100,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{   90,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{   80,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{   70,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{   60,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{   50,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   40,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   30,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   20,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{    0,    0,    0,    0}, /* RelativePathExpr: step(NodeTest) = 170 */
	{   19,    0,    0,    0}, /* StepExpr: Step_ = 172 */
	{  210,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    9,    0,    0,    0}, /* Step_: step(NodeTest) = 176 */
	{  200,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  190,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  170,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  170,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  170,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #133: stmt_ty(nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #134: tag */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* TagName: tag = 216 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #135: text(dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  230,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  220,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  210,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  250,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  200,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  190,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  180,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  170,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  160,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  150,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  140,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  130,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  120,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  110,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  100,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   90,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   80,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   70,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   50,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   40,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   30,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* PrimaryExpr: Constructor = 182 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* Constructor: TextConstructor = 206 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* TextConstructor: text(OptContentExpr) = 212 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  240,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  220,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  220,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  220,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #136: then_else(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #137: transbinds(let(var_type(varref, nil), dot), nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* TransformBindings: transbinds(TransformBinding, TransformBindings) = 239 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #138: transform(nil, modify(dot, dot)) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   30,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   20,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   10,    0,    0,    0}, /* ExprSingle: TransformExpr = 232 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  220,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  210,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  200,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  190,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  180,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  170,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  160,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  150,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  140,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  130,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  120,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  110,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  100,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   80,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   70,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   60,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   40,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* TransformExpr: transform(TransformBindings, modify(ExprSingle, ExprSingle)) = 237 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   20,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   20,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #139: treat(dot, seq_ty(schm_attr)) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  130,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  120,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  110,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  100,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  140,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  130,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   80,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{   70,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{   60,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{   50,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{   40,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{   30,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{   20,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{   10,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{    0,    0,    0,    0}, /* TreatExpr: treat(CastableExpr, SequenceType) = 155 */
	{  220,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  210,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  200,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  190,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  180,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  170,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{  160,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{  150,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{  140,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  130,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  130,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  130,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  110,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  110,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #140: typeswitch(dot, cases(case_(var_type(varref, seq_ty(schm_attr)), dot), cases(default_(varref, dot), nil))) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   30,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   20,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   10,    0,    0,    0}, /* ExprSingle: TypeswitchExpr = 60 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* TypeswitchExpr: typeswitch(OptExpr, cases(CaseClause, cases(DefaultClause_, nil))) = 90 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  220,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  210,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  200,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  190,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  180,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  170,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  160,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  150,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  140,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  130,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  120,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  110,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  100,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   80,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   70,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   60,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   40,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   20,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   20,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #141: typeswitch(dot, cases(case_(var_type(varref, seq_ty(schm_attr)), dot), cases(case_(var_type(varref, seq_ty(schm_attr)), dot), cases(default_(varref, dot), nil)))) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   30,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   20,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   10,    0,    0,    0}, /* ExprSingle: TypeswitchExpr = 60 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* TypeswitchExpr: typeswitch(OptExpr, cases(CaseClause, CaseClauses_)) = 91 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  220,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  210,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  200,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  190,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  180,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  170,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  160,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  150,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  140,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  130,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  120,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  110,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  100,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   90,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   80,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   70,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   60,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   40,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   20,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   20,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #142: uminus(dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  160,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  150,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  140,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  130,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  170,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  160,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  120,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  110,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  100,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{   90,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{   80,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{   70,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{   60,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{   50,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{   40,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{   30,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{   20,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   10,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{    0,    0,    0,    0}, /* UnaryExpr: uminus(ValueExpr) = 161 */
	{  220,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  210,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  200,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{  190,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{  180,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{  170,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  160,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  160,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  160,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  140,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  140,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  140,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #143: union_(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  100,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   90,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   80,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   70,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  100,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   50,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{   40,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{   30,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{   20,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{   10,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{    0,    0,    0,    0}, /* UnionExpr: union_(IntersectExceptExpr, UnionExpr) = 148 */
	{  220,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  210,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  200,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  190,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  180,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  170,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  160,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  150,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  140,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{  130,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{  120,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  100,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  100,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  100,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   80,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   80,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   80,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #144: unordered(dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  220,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  210,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  200,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  180,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  170,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  160,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  150,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  140,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  130,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  120,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  110,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  100,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{   90,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   80,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   70,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   60,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   40,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   30,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   20,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* PrimaryExpr: UnorderedExpr = 184 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* UnorderedExpr: unordered(OptExpr) = 203 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  230,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  210,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  210,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  210,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #145: uplus(dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  160,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  150,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  140,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  130,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  170,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  160,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  120,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  110,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  100,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{   90,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{   80,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{   70,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{   60,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{   50,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{   40,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{   30,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{   20,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   10,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{    0,    0,    0,    0}, /* UnaryExpr: uplus(ValueExpr) = 162 */
	{  220,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  210,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  200,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{  190,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{  180,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{  170,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  160,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  160,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  160,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  140,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  140,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  140,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #146: val_eq(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   50,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   40,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   30,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   10,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{    0,    0,    0,    0}, /* ComparisonExpr: val_eq(RangeExpr, RangeExpr) = 127 */
	{  220,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  210,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  200,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  190,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  180,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  170,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  160,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  150,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  140,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  130,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  120,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  100,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   90,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   80,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   60,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   40,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   40,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #147: val_ge(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   50,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   40,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   30,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   10,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{    0,    0,    0,    0}, /* ComparisonExpr: val_ge(RangeExpr, RangeExpr) = 132 */
	{  220,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  210,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  200,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  190,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  180,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  170,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  160,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  150,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  140,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  130,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  120,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  100,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   90,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   80,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   60,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   40,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   40,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #148: val_gt(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   50,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   40,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   30,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   10,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{    0,    0,    0,    0}, /* ComparisonExpr: val_gt(RangeExpr, RangeExpr) = 131 */
	{  220,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  210,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  200,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  190,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  180,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  170,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  160,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  150,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  140,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  130,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  120,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  100,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   90,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   80,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   60,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   40,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   40,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #149: val_le(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   50,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   40,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   30,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   10,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{    0,    0,    0,    0}, /* ComparisonExpr: val_le(RangeExpr, RangeExpr) = 130 */
	{  220,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  210,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  200,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  190,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  180,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  170,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  160,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  150,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  140,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  130,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  120,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  100,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   90,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   80,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   60,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   40,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   40,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #150: val_lt(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   50,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   40,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   30,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   10,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{    0,    0,    0,    0}, /* ComparisonExpr: val_lt(RangeExpr, RangeExpr) = 129 */
	{  220,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  210,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  200,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  190,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  180,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  170,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  160,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  150,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  140,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  130,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  120,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  100,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   90,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   80,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   60,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   40,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   40,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #151: val_ne(dot, dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{   50,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{   40,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{   30,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{   10,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{    0,    0,    0,    0}, /* ComparisonExpr: val_ne(RangeExpr, RangeExpr) = 128 */
	{  220,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  210,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  200,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  190,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  180,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  170,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  160,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{  150,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{  140,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{  130,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{  120,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{  110,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  100,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   90,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   80,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   70,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   60,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{   60,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   40,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{   40,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{   40,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #152: validate(dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  180,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  170,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  160,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  150,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  180,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  140,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  130,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  120,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  110,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  100,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{   90,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{   80,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{   70,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{   60,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{   50,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{   40,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   30,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   20,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   10,    0,    0,    0}, /* ValueExpr: ValidateExpr = 163 */
	{    0,    0,    0,    0}, /* ValidateExpr: validate(OptExpr) = 166 */
	{  230,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{  220,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{  210,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{  200,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* PrimaryExpr: ParenthesizedExpr = 179 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  180,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  180,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  180,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  160,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  160,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  160,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #153: var_decl(var_type(varref, nil), dot) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: VariableDeclaration = 23 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* VariableDeclaration: var_decl(var_type(varref, OptSequenceType_), OptExpr) = 49 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #154: var_decl(var_type(varref, nil), external) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* DeclImport: VariableDeclaration = 23 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* VariableDeclaration: var_decl(var_type(varref, OptSequenceType_), external) = 50 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #155: var_type(varref, seq_ty(schm_attr)) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #156: var_type(nil, seq_ty(schm_attr)) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #157: var_type(varref, nil) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #158: varref */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  220,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  210,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  200,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   20,    0,    0,    0}, /* OptPositionalVar_: PositionalVar = 73 */
	{   10,    0,    0,    0}, /* PositionalVar: varref = 74 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* OptVarName_: varref = 96 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  180,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  170,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  160,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  150,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  140,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  130,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  120,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  110,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  100,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{   90,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   80,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   70,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   60,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   40,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   30,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   20,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* PrimaryExpr: varref = 178 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  230,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  210,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  210,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  210,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #159: vars(var_type(varref, nil), varref) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #160: where(dot, ord_ret(nil, dot)) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
},
{ /* state #161: xrpc(dot, fun_ref(nil)) */
	{0},
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* QueryBody: OptExpr = 51 */
	{  220,    0,    0,    0}, /* OptExpr: Expr = 53 */
	{  210,    0,    0,    0}, /* Expr: ExprSingle = 57 */
	{  200,    0,    0,    0}, /* ExprSingle: OrExpr = 62 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  240,    0,    0,    0}, /* OptWhereClause_: WhereClause = 79 */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* WhereClause: OptExpr = 82 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  190,    0,    0,    0}, /* OrExpr: AndExpr = 116 */
	{  180,    0,    0,    0}, /* AndExpr: ComparisonExpr = 118 */
	{  170,    0,    0,    0}, /* ComparisonExpr: RangeExpr = 120 */
	{  160,    0,    0,    0}, /* RangeExpr: AdditiveExpr = 137 */
	{  150,    0,    0,    0}, /* AdditiveExpr: MultiplicativeExpr = 139 */
	{  140,    0,    0,    0}, /* MultiplicativeExpr: UnionExpr = 142 */
	{  130,    0,    0,    0}, /* UnionExpr: IntersectExceptExpr = 147 */
	{  120,    0,    0,    0}, /* IntersectExceptExpr: InstanceofExpr = 149 */
	{  110,    0,    0,    0}, /* InstanceofExpr: TreatExpr = 152 */
	{  100,    0,    0,    0}, /* TreatExpr: CastableExpr = 154 */
	{   90,    0,    0,    0}, /* CastableExpr: CastExpr = 156 */
	{   80,    0,    0,    0}, /* CastExpr: UnaryExpr = 158 */
	{   70,    0,    0,    0}, /* UnaryExpr: ValueExpr = 160 */
	{   60,    0,    0,    0}, /* ValueExpr: PathExpr = 164 */
	{    0,    0,    0,    0}, /* (none) */
	{   50,    0,    0,    0}, /* PathExpr: RelativePathExpr = 168 */
	{   40,    0,    0,    0}, /* RelativePathExpr: StepExpr = 169 */
	{   30,    0,    0,    0}, /* StepExpr: FilterExpr = 173 */
	{   20,    0,    0,    0}, /* FilterExpr: PrimaryExpr = 174 */
	{    0,    0,    0,    0}, /* (none) */
	{   10,    0,    0,    0}, /* PrimaryExpr: XRPCCall = 185 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* ParenthesizedExpr: OptExpr = 195 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  230,    0,    0,    0}, /* TagName: OptExpr = 217 */
	{  230,    0,    0,    0}, /* OptContentExpr: OptExpr = 218 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* (none) */
	{  210,    0,    0,    0}, /* SourceExpr: ExprSingle = 241 */
	{  210,    0,    0,    0}, /* TargetExpr: ExprSingle = 242 */
	{  210,    0,    0,    0}, /* NewNameExpr: ExprSingle = 243 */
	{    0,    0,    0,    0}, /* (none) */
	{    0,    0,    0,    0}, /* XRPCCall: xrpc(Expr, FunctionCall) = 248 */
},
};

char * PFnormalize_state_string[] = {
" not a state", /* state 0 */
	"and(dot, dot)", /* state #1 */
	"args(dot, nil)", /* state #2 */
	"args(dot, args(dot, nil))", /* state #3 */
	"atom_ty(nil)", /* state #4 */
	"attr(tag, dot)", /* state #5 */
	"base_uri(lit_str)", /* state #6 */
	"bind(vars(var_type(varref, nil), varref), dot)", /* state #7 */
	"binds(let(var_type(varref, nil), dot), nil)", /* state #8 */
	"binds(let(var_type(varref, nil), dot), binds(let(var_type(varref, nil), dot), nil))", /* state #9 */
	"boundspc_decl", /* state #10 */
	"case_(var_type(varref, seq_ty(schm_attr)), dot)", /* state #11 */
	"cases(case_(var_type(varref, seq_ty(schm_attr)), dot), cases(case_(var_type(varref, seq_ty(schm_attr)), dot), cases(default_(varref, dot), nil)))", /* state #12 */
	"cases(default_(varref, dot), nil)", /* state #13 */
	"cases(case_(var_type(varref, seq_ty(schm_attr)), dot), cases(default_(varref, dot), nil))", /* state #14 */
	"cast(dot, seq_ty(atom_ty(nil)))", /* state #15 */
	"castable(dot, seq_ty(atom_ty(nil)))", /* state #16 */
	"coll_decl(lit_str)", /* state #17 */
	"comment(dot)", /* state #18 */
	"constr_decl", /* state #19 */
	"contseq(empty_seq, empty_seq)", /* state #20 */
	"contseq(dot, dot)", /* state #21 */
	"contseq(contseq(dot, dot), dot)", /* state #22 */
	"copy_ns", /* state #23 */
	"decl_imps(revalid, nil)", /* state #24 */
	"decl_imps(nil, nil)", /* state #25 */
	"def_order", /* state #26 */
	"default_(varref, dot)", /* state #27 */
	"delete(dot)", /* state #28 */
	"div(dot, dot)", /* state #29 */
	"doc(dot)", /* state #30 */
	"docmgmt_ty(nil)", /* state #31 */
	"dot", /* state #32 */
	"elem(tag, dot)", /* state #33 */
	"empty_seq", /* state #34 */
	"empty_ty", /* state #35 */
	"ens_decl(lit_str)", /* state #36 */
	"eq(dot, dot)", /* state #37 */
	"every(binds(let(var_type(varref, nil), dot), nil), dot)", /* state #38 */
	"every(binds(let(var_type(varref, nil), dot), binds(let(var_type(varref, nil), dot), nil)), dot)", /* state #39 */
	"except(dot, dot)", /* state #40 */
	"exprseq(exprseq(dot, dot), dot)", /* state #41 */
	"exprseq(dot, dot)", /* state #42 */
	"exprseq(empty_seq, dot)", /* state #43 */
	"ext_expr(nil, dot)", /* state #44 */
	"ext_expr(nil, nil)", /* state #45 */
	"external", /* state #46 */
	"flwr(binds(let(var_type(varref, nil), dot), nil), where(dot, ord_ret(nil, dot)))", /* state #47 */
	"flwr(binds(let(var_type(varref, nil), dot), binds(let(var_type(varref, nil), dot), nil)), where(dot, ord_ret(nil, dot)))", /* state #48 */
	"fns_decl(lit_str)", /* state #49 */
	"fun_decl(fun_sig(nil, seq_ty(schm_attr)), dot)", /* state #50 */
	"fun_decl(fun_sig(nil, seq_ty(stmt_ty(nil))), external)", /* state #51 */
	"fun_decl(fun_sig(nil, seq_ty(schm_attr)), external)", /* state #52 */
	"fun_decl(fun_sig(nil, seq_ty(docmgmt_ty(nil))), dot)", /* state #53 */
	"fun_decl(fun_sig(nil, seq_ty(stmt_ty(nil))), dot)", /* state #54 */
	"fun_ref(nil)", /* state #55 */
	"fun_sig(nil, seq_ty(stmt_ty(nil)))", /* state #56 */
	"fun_sig(nil, seq_ty(schm_attr))", /* state #57 */
	"fun_sig(nil, seq_ty(docmgmt_ty(nil)))", /* state #58 */
	"ge(dot, dot)", /* state #59 */
	"gt(dot, dot)", /* state #60 */
	"gtgt(dot, dot)", /* state #61 */
	"idiv(dot, dot)", /* state #62 */
	"if_(dot, then_else(dot, dot))", /* state #63 */
	"insert(dot, dot)", /* state #64 */
	"instof(dot, seq_ty(schm_attr))", /* state #65 */
	"intersect(dot, dot)", /* state #66 */
	"is(dot, dot)", /* state #67 */
	"item_ty(nil)", /* state #68 */
	"le(dot, dot)", /* state #69 */
	"let(var_type(varref, seq_ty(schm_attr)), dot)", /* state #70 */
	"let(var_type(varref, nil), dot)", /* state #71 */
	"lib_mod(mod_ns(lit_str), nil)", /* state #72 */
	"lit_dbl", /* state #73 */
	"lit_dec", /* state #74 */
	"lit_int", /* state #75 */
	"lit_str", /* state #76 */
	"locpath(dot, dot)", /* state #77 */
	"lt(dot, dot)", /* state #78 */
	"ltlt(dot, dot)", /* state #79 */
	"main_mod(nil, dot)", /* state #80 */
	"minus(dot, dot)", /* state #81 */
	"mod(dot, dot)", /* state #82 */
	"mod_imp(lit_str, nil)", /* state #83 */
	"mod_ns(lit_str)", /* state #84 */
	"modify(dot, dot)", /* state #85 */
	"mult(dot, dot)", /* state #86 */
	"named_ty", /* state #87 */
	"ne(dot, dot)", /* state #88 */
	"nil", /* state #89 */
	"node_ty(node_ty(nil))", /* state #90 */
	"node_ty(nil)", /* state #91 */
	"node_ty(lit_str)", /* state #92 */
	"ns_decl(lit_str)", /* state #93 */
	"option(lit_str)", /* state #94 */
	"or(dot, dot)", /* state #95 */
	"ord_ret(nil, dot)", /* state #96 */
	"orderby(orderspecs(dot, nil))", /* state #97 */
	"ordered(dot)", /* state #98 */
	"ordering_mode", /* state #99 */
	"orderspecs(dot, orderspecs(dot, nil))", /* state #100 */
	"orderspecs(dot, nil)", /* state #101 */
	"param(nil, varref)", /* state #102 */
	"params(param(nil, varref), nil)", /* state #103 */
	"pi(dot)", /* state #104 */
	"plus(dot, dot)", /* state #105 */
	"pragma", /* state #106 */
	"pragmas(pragma, nil)", /* state #107 */
	"pred(dot, dot)", /* state #108 */
	"range(dot, dot)", /* state #109 */
	"recursion(var_type(varref, nil), seed(dot, dot))", /* state #110 */
	"rename(dot, dot)", /* state #111 */
	"replace(dot, dot)", /* state #112 */
	"req_name", /* state #113 */
	"req_ty(nil, named_ty)", /* state #114 */
	"req_ty(req_name, nil)", /* state #115 */
	"req_ty(nil, nil)", /* state #116 */
	"req_ty(req_name, named_ty)", /* state #117 */
	"revalid", /* state #118 */
	"root_", /* state #119 */
	"schm_ats(lit_str, nil)", /* state #120 */
	"schm_attr", /* state #121 */
	"schm_elem", /* state #122 */
	"schm_imp(lit_str, nil)", /* state #123 */
	"seed(dot, dot)", /* state #124 */
	"seq_ty(empty_ty)", /* state #125 */
	"seq_ty(schm_attr)", /* state #126 */
	"seq_ty(atom_ty(nil))", /* state #127 */
	"seq_ty(stmt_ty(nil))", /* state #128 */
	"seq_ty(docmgmt_ty(nil))", /* state #129 */
	"some(binds(let(var_type(varref, nil), dot), binds(let(var_type(varref, nil), dot), nil)), dot)", /* state #130 */
	"some(binds(let(var_type(varref, nil), dot), nil), dot)", /* state #131 */
	"step(node_ty(nil))", /* state #132 */
	"stmt_ty(nil)", /* state #133 */
	"tag", /* state #134 */
	"text(dot)", /* state #135 */
	"then_else(dot, dot)", /* state #136 */
	"transbinds(let(var_type(varref, nil), dot), nil)", /* state #137 */
	"transform(nil, modify(dot, dot))", /* state #138 */
	"treat(dot, seq_ty(schm_attr))", /* state #139 */
	"typeswitch(dot, cases(case_(var_type(varref, seq_ty(schm_attr)), dot), cases(default_(varref, dot), nil)))", /* state #140 */
	"typeswitch(dot, cases(case_(var_type(varref, seq_ty(schm_attr)), dot), cases(case_(var_type(varref, seq_ty(schm_attr)), dot), cases(default_(varref, dot), nil))))", /* state #141 */
	"uminus(dot)", /* state #142 */
	"union_(dot, dot)", /* state #143 */
	"unordered(dot)", /* state #144 */
	"uplus(dot)", /* state #145 */
	"val_eq(dot, dot)", /* state #146 */
	"val_ge(dot, dot)", /* state #147 */
	"val_gt(dot, dot)", /* state #148 */
	"val_le(dot, dot)", /* state #149 */
	"val_lt(dot, dot)", /* state #150 */
	"val_ne(dot, dot)", /* state #151 */
	"validate(dot)", /* state #152 */
	"var_decl(var_type(varref, nil), dot)", /* state #153 */
	"var_decl(var_type(varref, nil), external)", /* state #154 */
	"var_type(varref, seq_ty(schm_attr))", /* state #155 */
	"var_type(nil, seq_ty(schm_attr))", /* state #156 */
	"var_type(varref, nil)", /* state #157 */
	"varref", /* state #158 */
	"vars(var_type(varref, nil), varref)", /* state #159 */
	"where(dot, ord_ret(nil, dot))", /* state #160 */
	"xrpc(dot, fun_ref(nil))", /* state #161 */
};
char *PFnormalize_ntname[] = {
	"Error: Nonterminals are > 0",
	"Query",
	"MainModule",
	"LibraryModule",
	"ModuleDecl",
	"Prolog",
	"DeclsImports",
	"DeclImport",
	"DefaultNamespaceDecl",
	"BoundarySpaceDecl",
	"DefaultCollationDecl",
	"BaseURIDecl",
	"ConstructionDecl",
	"OptionDecl",
	"OrderingModeDecl",
	"EmptyOrderingDecl",
	"CopyNamespacesDecl",
	"NamespaceDecl",
	"SchemaImport",
	"ModuleImport",
	"OptImportLoc_",
	"FunctionDeclaration",
	"OptParamList_",
	"ParamList_",
	"Param",
	"OptSequenceType_",
	"VariableDeclaration",
	"QueryBody",
	"OptExpr",
	"Expr",
	"ExprSingle",
	"FLWORExpr",
	"VarBindings",
	"VarBinding",
	"LetClause",
	"VarPosBinding",
	"LetBinding_",
	"OptPositionalVar_",
	"PositionalVar",
	"OptTypeDeclaration_",
	"TypeDeclaration",
	"OptWhereClause_",
	"OptOrderByClause_",
	"WhereClause",
	"OrderByClause",
	"OrderSpecList",
	"QuantifiedExpr",
	"TypeswitchExpr",
	"CaseClauses_",
	"DefaultClause_",
	"OptVarName_",
	"CaseClause",
	"SequenceType",
	"SingleType",
	"ItemType",
	"NodeTest",
	"OptElemOrAttrType_",
	"ElemOrAttrType",
	"IfExpr",
	"OrExpr",
	"AndExpr",
	"ComparisonExpr",
	"RangeExpr",
	"AdditiveExpr",
	"MultiplicativeExpr",
	"UnionExpr",
	"IntersectExceptExpr",
	"InstanceofExpr",
	"TreatExpr",
	"CastableExpr",
	"CastExpr",
	"UnaryExpr",
	"ValueExpr",
	"ValidateExpr",
	"PathExpr",
	"RelativePathExpr",
	"StepExpr",
	"FilterExpr",
	"Step_",
	"PrimaryExpr",
	"Literal",
	"NumericLiteral",
	"IntegerLiteral",
	"DecimalLiteral",
	"DoubleLiteral",
	"StringLiteral",
	"ParenthesizedExpr",
	"ContextItemExpr",
	"FunctionCall",
	"OptFuncArgList_",
	"FuncArgList_",
	"OrderedExpr",
	"UnorderedExpr",
	"Constructor",
	"ElementConstructor",
	"AttributeConstructor",
	"TextConstructor",
	"CommentConstructor",
	"PIConstructor",
	"DocumentConstructor",
	"TagName",
	"OptContentExpr",
	"ExtensionExpr",
	"Pragmas",
	"RevalidationDecl",
	"InsertExpr",
	"DeleteExpr",
	"ReplaceExpr",
	"RenameExpr",
	"TransformExpr",
	"TransformBindings",
	"TransformBinding",
	"SourceExpr",
	"TargetExpr",
	"NewNameExpr",
	"RecursiveExpr",
	"XRPCCall",
	0
};

short PFnormalize_closure[117][117] = {
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    1,    2,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    6,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,   24,    0,    0,    0,    0,   10,   11,
	    12,   13,   14,   19,   15,   16,   17,   18,   20,   21,
	     0,   22,    0,    0,    0,    0,   23,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,  226,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,   43,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,   48,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,   51,   51,
	    51,   51,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,   51,   51,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,   51,   51,
	    51,   51,   51,   51,   51,   51,   51,   51,   51,   51,
	    51,   51,   51,   51,   51,   51,   51,   51,   51,   51,
	    51,   51,   51,   51,   51,   51,   51,   51,   51,    0,
	     0,   51,   51,   51,   51,   51,   51,   51,   51,   51,
	     0,    0,   51,    0,    0,   51,   51,   51,   51,   51,
	     0,    0,    0,    0,    0,   51,   51,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,   53,
	    53,   53,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,   53,   53,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,   53,   53,
	    53,   53,   53,   53,   53,   53,   53,   53,   53,   53,
	    53,   53,   53,   53,   53,   53,   53,   53,   53,   53,
	    53,   53,   53,   53,   53,   53,   53,   53,   53,    0,
	     0,   53,   53,   53,   53,   53,   53,   53,   53,   53,
	     0,    0,   53,    0,    0,   53,   53,   53,   53,   53,
	     0,    0,    0,    0,    0,   53,   53,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,   57,    0,
	    57,   57,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,   57,   57,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,   57,   57,
	    57,   57,   57,   57,   57,   57,   57,   57,   57,   57,
	    57,   57,   57,   57,   57,   57,   57,   57,   57,   57,
	    57,   57,   57,   57,   57,   57,   57,   57,   57,    0,
	     0,   57,   57,   57,   57,   57,   57,   57,   57,   57,
	     0,    0,   57,    0,    0,   57,   57,   57,   57,   57,
	     0,    0,    0,    0,    0,   57,   57,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,   62,   62,
	     0,   58,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,   59,   60,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,   61,   62,
	    62,   62,   62,   62,   62,   62,   62,   62,   62,   62,
	    62,   62,   62,   62,   62,   62,   62,   62,   62,   62,
	    62,   62,   62,   62,   62,   62,   62,   62,   62,    0,
	     0,   62,   62,   62,   62,   62,   62,   62,   62,   62,
	     0,    0,   62,    0,    0,  228,  229,  231,  230,  232,
	     0,    0,    0,    0,    0,  246,   62,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,   68,   67,   68,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,   69,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,   73,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	    76,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,   76,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,   77,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,   79,   79,
	    79,   79,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,   79,    0,    0,   79,   79,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,   79,   79,
	    79,   79,   79,   79,   79,   79,   79,   79,   79,   79,
	    79,   79,   79,   79,   79,   79,   79,   79,   79,   79,
	    79,   79,   79,   79,   79,   79,   79,   79,   79,    0,
	     0,   79,   79,   79,   79,   79,   79,   79,   79,   79,
	     0,    0,   79,    0,    0,   79,   79,   79,   79,   79,
	     0,    0,    0,    0,    0,   79,   79,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,   81,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,   82,   82,
	    82,   82,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,   82,   82,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,   82,   82,
	    82,   82,   82,   82,   82,   82,   82,   82,   82,   82,
	    82,   82,   82,   82,   82,   82,   82,   82,   82,   82,
	    82,   82,   82,   82,   82,   82,   82,   82,   82,    0,
	     0,   82,   82,   82,   82,   82,   82,   82,   82,   82,
	     0,    0,   82,    0,    0,   82,   82,   82,   82,   82,
	     0,    0,    0,    0,    0,   82,   82,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,  101,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,  110,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  116,  116,
	   116,  116,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  116,  116,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  116,    0,
	   116,  116,  116,  116,  116,  116,  116,  116,  116,  116,
	   116,  116,  116,  116,  116,  116,  116,  116,  116,  116,
	   116,  116,  116,  116,  116,  116,  116,  116,  116,    0,
	     0,  116,  116,  116,  116,  116,  116,  116,  116,  116,
	     0,    0,  116,    0,    0,  116,  116,  116,  116,  116,
	     0,    0,    0,    0,    0,  116,  116,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  118,  118,
	   118,  118,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  118,  118,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  118,  118,
	     0,  118,  118,  118,  118,  118,  118,  118,  118,  118,
	   118,  118,  118,  118,  118,  118,  118,  118,  118,  118,
	   118,  118,  118,  118,  118,  118,  118,  118,  118,    0,
	     0,  118,  118,  118,  118,  118,  118,  118,  118,  118,
	     0,    0,  118,    0,    0,  118,  118,  118,  118,  118,
	     0,    0,    0,    0,    0,  118,  118,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  120,  120,
	   120,  120,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  120,  120,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  120,  120,
	   120,    0,  120,  120,  120,  120,  120,  120,  120,  120,
	   120,  120,  120,  120,  120,  120,  120,  120,  120,  120,
	   120,  120,  120,  120,  120,  120,  120,  120,  120,    0,
	     0,  120,  120,  120,  120,  120,  120,  120,  120,  120,
	     0,    0,  120,    0,    0,  120,  120,  120,  120,  120,
	     0,    0,    0,    0,    0,  120,  120,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  137,  137,
	   137,  137,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  137,  137,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  137,  137,
	   137,  137,    0,  137,  137,  137,  137,  137,  137,  137,
	   137,  137,  137,  137,  137,  137,  137,  137,  137,  137,
	   137,  137,  137,  137,  137,  137,  137,  137,  137,    0,
	     0,  137,  137,  137,  137,  137,  137,  137,  137,  137,
	     0,    0,  137,    0,    0,  137,  137,  137,  137,  137,
	     0,    0,    0,    0,    0,  137,  137,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  139,  139,
	   139,  139,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  139,  139,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  139,  139,
	   139,  139,  139,    0,  139,  139,  139,  139,  139,  139,
	   139,  139,  139,  139,  139,  139,  139,  139,  139,  139,
	   139,  139,  139,  139,  139,  139,  139,  139,  139,    0,
	     0,  139,  139,  139,  139,  139,  139,  139,  139,  139,
	     0,    0,  139,    0,    0,  139,  139,  139,  139,  139,
	     0,    0,    0,    0,    0,  139,  139,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  142,  142,
	   142,  142,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  142,  142,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  142,  142,
	   142,  142,  142,  142,    0,  142,  142,  142,  142,  142,
	   142,  142,  142,  142,  142,  142,  142,  142,  142,  142,
	   142,  142,  142,  142,  142,  142,  142,  142,  142,    0,
	     0,  142,  142,  142,  142,  142,  142,  142,  142,  142,
	     0,    0,  142,    0,    0,  142,  142,  142,  142,  142,
	     0,    0,    0,    0,    0,  142,  142,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  147,  147,
	   147,  147,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  147,  147,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  147,  147,
	   147,  147,  147,  147,  147,    0,  147,  147,  147,  147,
	   147,  147,  147,  147,  147,  147,  147,  147,  147,  147,
	   147,  147,  147,  147,  147,  147,  147,  147,  147,    0,
	     0,  147,  147,  147,  147,  147,  147,  147,  147,  147,
	     0,    0,  147,    0,    0,  147,  147,  147,  147,  147,
	     0,    0,    0,    0,    0,  147,  147,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  149,  149,
	   149,  149,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  149,  149,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  149,  149,
	   149,  149,  149,  149,  149,  149,    0,  149,  149,  149,
	   149,  149,  149,  149,  149,  149,  149,  149,  149,  149,
	   149,  149,  149,  149,  149,  149,  149,  149,  149,    0,
	     0,  149,  149,  149,  149,  149,  149,  149,  149,  149,
	     0,    0,  149,    0,    0,  149,  149,  149,  149,  149,
	     0,    0,    0,    0,    0,  149,  149,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  152,  152,
	   152,  152,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  152,  152,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  152,  152,
	   152,  152,  152,  152,  152,  152,  152,    0,  152,  152,
	   152,  152,  152,  152,  152,  152,  152,  152,  152,  152,
	   152,  152,  152,  152,  152,  152,  152,  152,  152,    0,
	     0,  152,  152,  152,  152,  152,  152,  152,  152,  152,
	     0,    0,  152,    0,    0,  152,  152,  152,  152,  152,
	     0,    0,    0,    0,    0,  152,  152,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  154,  154,
	   154,  154,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  154,  154,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  154,  154,
	   154,  154,  154,  154,  154,  154,  154,  154,    0,  154,
	   154,  154,  154,  154,  154,  154,  154,  154,  154,  154,
	   154,  154,  154,  154,  154,  154,  154,  154,  154,    0,
	     0,  154,  154,  154,  154,  154,  154,  154,  154,  154,
	     0,    0,  154,    0,    0,  154,  154,  154,  154,  154,
	     0,    0,    0,    0,    0,  154,  154,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  156,  156,
	   156,  156,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  156,  156,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  156,  156,
	   156,  156,  156,  156,  156,  156,  156,  156,  156,    0,
	   156,  156,  156,  156,  156,  156,  156,  156,  156,  156,
	   156,  156,  156,  156,  156,  156,  156,  156,  156,    0,
	     0,  156,  156,  156,  156,  156,  156,  156,  156,  156,
	     0,    0,  156,    0,    0,  156,  156,  156,  156,  156,
	     0,    0,    0,    0,    0,  156,  156,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  158,  158,
	   158,  158,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  158,  158,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  158,  158,
	   158,  158,  158,  158,  158,  158,  158,  158,  158,  158,
	     0,  158,  158,  158,  158,  158,  158,  158,  158,  158,
	   158,  158,  158,  158,  158,  158,  158,  158,  158,    0,
	     0,  158,  158,  158,  158,  158,  158,  158,  158,  158,
	     0,    0,  158,    0,    0,  158,  158,  158,  158,  158,
	     0,    0,    0,    0,    0,  158,  158,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  160,  160,
	   160,  160,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  160,  160,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  160,  160,
	   160,  160,  160,  160,  160,  160,  160,  160,  160,  160,
	   160,    0,  160,  160,  160,  160,  160,  160,  160,  160,
	   160,  160,  160,  160,  160,  160,  160,  160,  160,    0,
	     0,  160,  160,  160,  160,  160,  160,  160,  160,  160,
	     0,    0,  160,    0,    0,  160,  160,  160,  160,  160,
	     0,    0,    0,    0,    0,  160,  160,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  164,  164,
	   164,  164,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  164,  164,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  164,  164,
	   164,  164,  164,  164,  164,  164,  164,  164,  164,  164,
	   164,  164,    0,  163,  164,  164,  164,  164,  164,  164,
	   164,  164,  164,  164,  164,  164,  164,  164,  164,    0,
	     0,  164,  164,  164,  164,  164,  164,  164,  164,  164,
	     0,    0,  165,    0,    0,  164,  164,  164,  164,  164,
	     0,    0,    0,    0,    0,  164,  164,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  168,  168,
	   168,  168,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  168,  168,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  168,  168,
	   168,  168,  168,  168,  168,  168,  168,  168,  168,  168,
	   168,  168,  168,  168,    0,  168,  168,  168,  168,  168,
	   168,  168,  168,  168,  168,  168,  168,  168,  168,    0,
	     0,  168,  168,  168,  168,  168,  168,  168,  168,  168,
	     0,    0,  168,    0,    0,  168,  168,  168,  168,  168,
	     0,    0,    0,    0,    0,  168,  168,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  169,  169,
	   169,  169,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  169,  169,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  169,  169,
	   169,  169,  169,  169,  169,  169,  169,  169,  169,  169,
	   169,  169,  169,  169,  169,    0,  169,  169,  169,  169,
	   169,  169,  169,  169,  169,  169,  169,  169,  169,    0,
	     0,  169,  169,  169,  169,  169,  169,  169,  169,  169,
	     0,    0,  169,    0,    0,  169,  169,  169,  169,  169,
	     0,    0,    0,    0,    0,  169,  169,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  173,  173,
	   173,  173,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  173,  173,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  173,  173,
	   173,  173,  173,  173,  173,  173,  173,  173,  173,  173,
	   173,  173,  173,  173,  173,  173,    0,  173,  172,  173,
	   173,  173,  173,  173,  173,  173,  173,  173,  173,    0,
	     0,  173,  173,  173,  173,  173,  173,  173,  173,  173,
	     0,    0,  173,    0,    0,  173,  173,  173,  173,  173,
	     0,    0,    0,    0,    0,  173,  173,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  174,  174,
	   174,  174,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  174,  174,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  174,  174,
	   174,  174,  174,  174,  174,  174,  174,  174,  174,  174,
	   174,  174,  174,  174,  174,  174,  174,    0,  174,  174,
	   174,  174,  174,  174,  174,  174,  174,  174,  174,    0,
	     0,  174,  174,  174,  174,  174,  174,  174,  174,  174,
	     0,    0,  174,    0,    0,  174,  174,  174,  174,  174,
	     0,    0,    0,    0,    0,  174,  174,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  179,  179,
	   179,  179,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  179,  179,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  179,  179,
	   179,  179,  179,  179,  179,  179,  179,  179,  179,  179,
	   179,  179,  179,  179,  179,  179,  179,  179,  179,    0,
	   177,  177,  177,  177,  177,  177,  179,  180,  181,    0,
	     0,  183,  184,  182,  182,  182,  182,  182,  182,  182,
	     0,    0,  179,    0,    0,  179,  179,  179,  179,  179,
	     0,    0,    0,    0,    0,  179,  185,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,  186,  186,  186,  186,  187,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,  188,  189,  190,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  195,  195,
	   195,  195,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  195,  195,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  195,  195,
	   195,  195,  195,  195,  195,  195,  195,  195,  195,  195,
	   195,  195,  195,  195,  195,  195,  195,  195,  195,  195,
	   195,  195,  195,  195,  195,  195,    0,  195,  195,    0,
	     0,  195,  195,  195,  195,  195,  195,  195,  195,  195,
	     0,    0,  195,    0,    0,  195,  195,  195,  195,  195,
	     0,    0,    0,    0,    0,  195,  195,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	   199,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,  204,  205,  206,  207,  208,  209,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  217,  217,
	   217,  217,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  217,  217,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  217,  217,
	   217,  217,  217,  217,  217,  217,  217,  217,  217,  217,
	   217,  217,  217,  217,  217,  217,  217,  217,  217,  217,
	   217,  217,  217,  217,  217,  217,  217,  217,  217,    0,
	     0,  217,  217,  217,  217,  217,  217,  217,  217,  217,
	     0,    0,  217,    0,    0,  217,  217,  217,  217,  217,
	     0,    0,    0,    0,    0,  217,  217,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  218,  218,
	   218,  218,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  218,  218,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  218,  218,
	   218,  218,  218,  218,  218,  218,  218,  218,  218,  218,
	   218,  218,  218,  218,  218,  218,  218,  218,  218,  218,
	   218,  218,  218,  218,  218,  218,  218,  218,  218,    0,
	     0,  218,  218,  218,  218,  218,  218,  218,  218,  218,
	     0,    0,  218,    0,    0,  218,  218,  218,  218,  218,
	     0,    0,    0,    0,    0,  218,  218,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  241,  241,
	   241,  241,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  241,  241,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  241,  241,
	   241,  241,  241,  241,  241,  241,  241,  241,  241,  241,
	   241,  241,  241,  241,  241,  241,  241,  241,  241,  241,
	   241,  241,  241,  241,  241,  241,  241,  241,  241,    0,
	     0,  241,  241,  241,  241,  241,  241,  241,  241,  241,
	     0,    0,  241,    0,    0,  241,  241,  241,  241,  241,
	     0,    0,    0,    0,    0,  241,  241,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  242,  242,
	   242,  242,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  242,  242,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  242,  242,
	   242,  242,  242,  242,  242,  242,  242,  242,  242,  242,
	   242,  242,  242,  242,  242,  242,  242,  242,  242,  242,
	   242,  242,  242,  242,  242,  242,  242,  242,  242,    0,
	     0,  242,  242,  242,  242,  242,  242,  242,  242,  242,
	     0,    0,  242,    0,    0,  242,  242,  242,  242,  242,
	     0,    0,    0,    0,    0,  242,  242,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  243,  243,
	   243,  243,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,  243,  243,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,  243,  243,
	   243,  243,  243,  243,  243,  243,  243,  243,  243,  243,
	   243,  243,  243,  243,  243,  243,  243,  243,  243,  243,
	   243,  243,  243,  243,  243,  243,  243,  243,  243,    0,
	     0,  243,  243,  243,  243,  243,  243,  243,  243,  243,
	     0,    0,  243,    0,    0,  243,  243,  243,  243,  243,
	     0,    0,    0,    0,    0,  243,  243,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
	{    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,},
};
#line 622 "normalize.brg"


/** Maximum number of pattern leaves */
#define MAX_KIDS 10

static void relabel (PFpnode_t *p,  PFpnode_t **kids);

/* construct a nil node */
#define nil(loc) leaf (p_nil, (loc))

#define leaf(t,loc)       p_leaf ((t), (loc))
#define wire1(t,loc,a)    p_wire1 ((t), (loc), (a))
#define wire2(t,loc,a,b)  p_wire2 ((t), (loc), (a), (b))

static PFpnode_t * varref (PFqname_raw_t q, PFloc_t loc);
static PFqname_raw_t new_varname (char *prefix, unsigned int* varcount);
static bool nonpos_predicate(PFpnode_t *p);
static bool has_pos_or_nonstepdot(PFpnode_t *p, bool dots_allowed);
                
/**
 * The pull-predicate-up rewrite (169) can only be done if the predicate
 * does not contain position/last/dot; as checked by this function.
 */
static bool
has_pos_or_nonstepdot(PFpnode_t *p, bool dots_allowed) 
{
    while(p) {
        if (p->kind != p_locpath || L(p)->kind != p_step || R(p)->kind != p_dot) {
            /* we excluded the only legitimate use of dot (in R(p)); now check for it */
            if (p->kind == p_dot && !dots_allowed) return true;
   
            /* check for the forbidden special functions */ 
            if (p->kind == p_fun_ref) {
                if (strcmp(p->sem.qname_raw.loc, "position") == 0) return true;
                if (strcmp(p->sem.qname_raw.loc, "last") == 0) return true;
                if (strcmp(p->sem.qname_raw.loc, "first") == 0 && dots_allowed) return true;
            }
            /* allow the forbidden elements inside nested predicates (i.e. do not check R(p) then) */
            if (p->kind != p_pred && R(p) && has_pos_or_nonstepdot(R(p), dots_allowed)) return true; 
        }
        p = L(p); /* tail recurse rewritten into loop */
    }
    return false;
}

static bool 
nonpos_predicate(PFpnode_t *p) 
{
    if (p->kind == p_exprseq) /* sequence construction may be ok */
        return nonpos_predicate(L(p)) && nonpos_predicate(R(p));
    return /* allow obvious boolean expressions, (), [] and locpaths */
       (p->kind == p_eq   || p->kind == p_ne || p->kind == p_or || 
        p->kind == p_and  || p->kind == p_gt || p->kind == p_ge ||  
        p->kind == p_lt   || p->kind == p_le || p->kind == p_empty_seq || 
        p->kind == p_pred ||(p->kind == p_locpath && L(p)->kind == p_step));
}

bool
PFposition_safe_predicate(PFpnode_t *p, bool dots_allowed) 
{
    return nonpos_predicate(p) && !has_pos_or_nonstepdot(p, dots_allowed); 
}

/**
 * Reducer function. This is the heart of this source file. It
 * contains all the action code for the above burg patterns.
 */
static bool
do_reduce (PFpnode_t * p, unsigned int *varcount, int rule, PFpnode_t * kids[]);

static bool
reduce (PFpnode_t * p, int goalnt, unsigned int *varcount)
{
    int           rule;           /* rule number that matches for this node */
    short        *nts;            /* target non-terminals for the leaf nodes of
                                     the current rule */
    PFpnode_t *   kids[MAX_KIDS]; /* leaf nodes of this rule */
    bool          rewritten;
    short         old_state_label;
    unsigned short i;

    /* guard against too deep recursion */
    PFrecursion_fence();

    do {
        /* determine rule that matches for this non-terminal */
        rule = PFnormalize_rule (STATE_LABEL (p), goalnt);

        /* PFinfo (OOPS_NOTICE, "match for rule %i", rule); */

        /* initialize the kids[] vector */
        for (i = 0; i < MAX_KIDS; i++)
            kids[i] = NULL;

        /*
         * prepare recursive traversal: get information on leaf nodes of
         * this rule
         */
        nts = PFnormalize_nts[rule];
        PFnormalize_kids (p, rule, kids);

        /*
         * Recursively invoke compilation.  This means bottom-up compilation.
         */
        rewritten = false;
        for (i = 0; nts[i]; i++)
            if ((rewritten = reduce (kids[i], nts[i], varcount)))
                break;  /* abort if a subtree was rewritten */

        if (rewritten) {
            /*
             * If a subtree has been rewritten, we have to
             *  - re-label that part of the tree, and
             *  - possibly propagate the `rewritten' information upwards.
             */

            /* remember our old state_label (so we know if it has changed) */
            old_state_label = STATE_LABEL(p);

            /*
             * Re-label current tree pattern.
             * (at most down to the pattern leaves, as they should already be
             * correctly labeled by the above reduce() call)
             */
            relabel (p, kids);

            /* If our own state_label has changed, notify our caller. */
            if (old_state_label != STATE_LABEL(p))
                return true;
        }
    } while (rewritten);

    return do_reduce (p, varcount, rule, kids);
}

static bool
do_reduce (PFpnode_t * p, unsigned int *varcount, int rule, PFpnode_t * kids[])
{
    PFpnode_t * c;

    switch (rule) {
        /* DeclsImports: decl_imps (nil, DeclsImports) */
        case 8:
            *p = *R(p);
            return true;

        /* Expr: exprseq (empty_seq, ExprSingle) */
        case 55:
            *p = *(R(p));
            return true;

        /* Expr: exprseq (exprseq (Expr, OptExpr), OptExpr) */
        case 56:
            c = wire2 (p_exprseq, p->loc,
                       LL(p),
                       wire2 (p_exprseq, p->loc, LR(p), R(p)));
            *p = *c;
            relabel (p, kids);
            return true;

        /* QuantifiedExpr: some (binds (VarBinding, VarBindings),
                                 OptExpr) */
        case 87:
            /* fall through (treat some and every equivalently) */

        /* QuantifiedExpr: every (binds (VarBinding, VarBindings),
                                  OptExpr) */
        case 89:
            /*
             * Current parse tree situation: multi-bind some/every
             * (may bind any number n >= 2 variables):
             *
             * some/every $a as t1 in e1, $b as t2 in e2
             *   satisfies e0
             *
             *                         some/every
             *                         /       \
             *                   binds          e0
             *                  /    \
             *              bind    binds
             *            /    \    /   \
             *          vars   e1  bind  ...
             *         /   \       /  \
             *  var_type   nil   vars  e2
             *  /     \         /    \
             * $a     t1    var_type  nil
             *              /     \
             *             $b     t2
             *
             * Rewrite this into n nested single-bind some/every:
             *
             * some $a as t1 in e1 satisfies
             *   some $b as t2 in e2 satisfies e0
             *
             *
             *                 some/every
             *                 /        \
             *             binds         some/every
             *             /   \           /      \
             *           bind  nil      binds     e0
             *           /   \          /   \  
             *         vars  e1       bind  nil
             *        /    \          /   \
             *    var_type  nil     vars  e2
             *     /     \         /    \
             *    $a     t1    var_type  nil
             *                  /     \
             *                 $b     t2
             */
            c = wire2 (p->kind, p->loc,
                       L(p),
                       wire2 (p->kind, p->loc,
                              LR(p),
                              R(p)));
            LR(c) = nil (p->loc);
            *p = *c;
            relabel (p, kids);
            return true;

        /* TypeswitchExpr: typeswitch (OptExpr,
                                       cases (CaseClause,
                                              CaseClauses_)) */
        case 91:
        {
            /*
             * Unnest typeswitch expressions.
             *
             * Rewrite
             *
             *   typeswitch e1
             *     case $a as type1    return ret1
             *     case $b as type2    return ret2
             *     case $c as type3    return ret3
             *     ...
             *     default $d          return def
             *
             * to
             *
             *   typeswitch e1
             *     case $a as type1              return ret1
             *     default $v1 return
             *       typeswitch $v1
             *         case $b as type2          return ret2
             *         default $v2 return
             *           typeswitch $v2
             *             case $c as type3      return ret3
             *             default $v3 return
             *               typeswitch
             *                 ...
             *                   default $d      return def
             *
             * That is:
             *
             *              typeswitch
             *             /          \
             *           e1           cases
             *                       /     \
             *                   case       cases
             *                  /   \      /     \
             *           var_type  ret1   case    ...
             *          /     \          /    \      \   
             *         $a    type1   var_type  ret2   cases
             *                        /   \           /     \
             *                       $b  type2   default    nil
             *                                   /    \
             *                                 $d     def
             *
             * becomes
             *
             *          typeswitch
             *         /          \
             *       e1           cases
             *                   /     \
             *               case       cases
             *              /   \      /     \
             *        var_type  ret1 default  nil
             *        /     \        /     \
             *       $a    type1    $v1    typeswitch
             *                            /        \
             *                          $v1       cases
             *                                   /     \
             *                             var_type     ...
             *                            /      \         \
             *                           $b     type2      cases
             *                                            /    \
             *                                      default    nil
             *                                      /    \
             *                                     $d    def
             */
            PFqname_raw_t v1 = new_varname ("tsw", varcount);

            c = wire2 (p_typeswitch, p->loc,
                       L(p),
                       wire2 (p_cases, R(p)->loc,
                              RL(p),
                              wire2 (p_cases, RR(p)->loc,
                                     wire2 (p_default, RR(p)->loc,
                                            varref (v1, RR(p)->loc),
                                            wire2 (p_typeswitch, RR(p)->loc,
                                                   varref (v1, RR(p)->loc),
                                                   RR(p))),
                                     nil (RR(p)->loc))));
            *p = *c;
            relabel (p, kids);
            return true;
        }


        /* RelativePathExpr: step (NodeTest) */
        case 170:
            /*
             * Make sure any relative path expression starts from
             * the context node `.'
             */
            c = wire2 (p_locpath, p->loc,
                       wire1 (p_step, p->loc, L(p)),
                       leaf (p_dot, p->loc));
            c->child[0]->sem.axis = p->sem.axis;
            *p = *c;
            relabel (p, kids);
            return true;

        /* RelativePathExpr: locpath (StepExpr, RelativePathExpr) */
        case 171:
            if (L(p)->kind == p_pred && 
                /* rule 168 introduces this always (check for sanity): */
                LL(p)->kind == p_locpath && R(LL(p))->kind == p_dot && 
                /* only rewrite certain "safe" predicates: */
                LR(p)->kind == p_exprseq && PFposition_safe_predicate(L(LR(p)), false))  
            {
                /*         locpath                  pred 
                 *        /       \                /    \
                 *     pred        Expr1      locpath   Expr3
                 *     /  \             =>    /    \       
                 * Expr2 Expr3             Expr2    Expr1   
                 * "pull-predicate-up", avoids an extra for-loop later on     
                 */
                c = wire2 (p_pred, p->loc,
                       wire2 (p_exprseq, p->loc,
                              wire2 (p_locpath, p->loc,
                                     L(LL(p)),
                                     R(p)),
                              leaf(p_empty_seq, p->loc)),
                       LR(p));
                *p = *c;
                relabel (p, kids);
                return true;
            } 
            break;

        /* OptContentExpr: contseq (empty_seq, empty_seq) */
        case 220:
            *p = *R(p);
            return true;

        /* OptContentExpr: contseq (contseq (OptContentExpr,
                                             OptContentExpr),
                                    OptContentExpr)              */
        case 221:
            c = wire2 (p_contseq, p->loc,
                       LL(p),
                       wire2 (p_contseq, p->loc, LR(p), R(p)));
            *p = *c;
            relabel (p, kids);
            return true;

        default:
            break;
    }

    return false;
}

/*
 * Re-label a match pattern.
 *
 * Start with node @a p and relabel the subtree below. However, stop
 * if @a p is one of the pattern leaves, passed as the argument @a kids.
 */
static void 
relabel (PFpnode_t *p,  PFpnode_t **kids)
{
    unsigned int i;

    for (i = 0; i < MAX_KIDS; i++) {
        if (kids[i] && p == kids[i])
            return;
    }

    /* Relabel p's children. */
    if (!L(p) && !R(p)) {
        STATE_LABEL(p) = PFnormalize_state (OP_LABEL(p), 0, 0);
    }
    else if (L(p) && !R(p)) {
        relabel (L(p), kids);
        STATE_LABEL(p) = PFnormalize_state (OP_LABEL(p),
                                            STATE_LABEL(L(p)),
                                            0);
    }
    else if (!L(p) && R(p)) {
        relabel (R(p), kids);
        STATE_LABEL(p) = PFnormalize_state (OP_LABEL(p),
                                            STATE_LABEL(R(p)),
                                            0);
    }
    else {
        relabel (L(p), kids);
        relabel (R(p), kids);
        STATE_LABEL(p) = PFnormalize_state (OP_LABEL(p),
                                            STATE_LABEL(L(p)),
                                            STATE_LABEL(R(p)));
    }

}

/**
 * Generate a parse tree node for a variable reference
 * @param q   QName for the variable
 * @param loc location information as #PFloc_t struct
 * @return    a parse tree node representing the variable
 */
static PFpnode_t *
varref (PFqname_raw_t q, PFloc_t loc)
{
    PFpnode_t * n = p_leaf (p_varref, loc);
    
    n->sem.qname_raw = q;
    
    return n;
}

/**
 * Generate a new (unique) variable name. This unique name is created
 * by incrementing a number that is appended to the variable name.
 * A prefix may be given that may not be longer than 3 characters
 * (the length of 'dot'). If no prefix is given (by passing NULL),
 * the default 'v' is used. The namespace for the newly created
 * variable is the internal Pathfinder namespace `pf:...' (PFns_pf).
 *
 * @param prefix The prefix to use. The variable name will be
 *   "[prefix]_[num]", where [prefix] is the prefix and [num] is
 *   a three digit, zero-padded integer number
 * @return A QName describing the new variable name
 */

static PFqname_raw_t
new_varname (char *prefix, unsigned int* varcount)
{
    char                varname[sizeof("dot_000")];
    
    /* construct new variable name */
    if (prefix)
        snprintf (varname, sizeof ("dot_000"), "%3s_%03u", prefix, (*varcount)++);
    else
        snprintf (varname, sizeof ("dot_000"), "v_%03u", (*varcount)++);
    
    /* assign internal Pathfinder namespace `pf:...' */
    return (PFqname_raw_t) { .prefix = PFns_pf.prefix, .loc = PFstrdup (varname) };
}


/**
 * Normalize abstract syntax tree.
 *
 * Remove ambiguities in abstract syntax tree and bring it into
 * normal form.
 * 
 * @param r root of the abstract syntax tree (note that this
 *   might change during normalization)
 * @return the normalized abstract syntax tree
 */
PFpnode_t *
PFnormalize_abssyn (PFpnode_t *r)
{
    unsigned int varcount = 0; 
    assert (r);

    /* initiliaze global */
    varcount = 0;

    /* label the parse tree bottom up */
    PFnormalize_label (r);

    /* invoke rewriting */
    while (reduce (r, 1, &varcount))
        /* rewrite as long as there is something to do. */;

    return r;
}


/* vim:set shiftwidth=4 expandtab filetype=c: */
